.popover-wrapper.svelte-1df65jy{min-height:var(--min-height, 0)}.popover.svelte-1df65jy{box-shadow:0 8px 10px -5px #14141433,0 16px 24px 2px #14141424,0 6px 30px 5px #1414141f;align-self:flex-start;border-radius:var(--border-radius);background-color:var(--canvas-elevated);border:1px solid var(--border-subtle);min-width:var(--popover-width, 1rem);max-width:95vw;padding:var(--popover-padding-block, 6px) var(--popover-padding-inline, 6px)}.popover.scrollable.svelte-1df65jy{max-height:400px;overflow:hidden auto}.popover.hidden.svelte-1df65jy{visibility:hidden}.popover.top.svelte-1df65jy,.popover.left.svelte-1df65jy{align-self:flex-end}.popover.bottom.svelte-1df65jy,.popover.right.svelte-1df65jy{align-self:flex-start}button.svelte-1gs18n1,button.focus.svelte-1gs18n1{display:flex;justify-content:start;width:100%;padding:.25rem 1rem;white-space:nowrap;font-size:var(--dropdown-font-size, small);background:none;box-shadow:none!important;border:none;border-radius:0;color:var(--fg)}button.svelte-1gs18n1:hover,button.focus.svelte-1gs18n1{border:none}button.svelte-1gs18n1:hover:not([disabled]),button.focus.svelte-1gs18n1:not([disabled]){background:var(--highlight-bg);color:var(--highlight-fg)}button[disabled].svelte-1gs18n1{cursor:default;color:var(--fg-disabled)}button.svelte-1gs18n1:not(.rtl){border-left:3px solid transparent}button.rtl.svelte-1gs18n1{border-right:3px solid transparent}button.active.svelte-1gs18n1:not(.rtl){border-left-color:var(--border-focus)}button.active.rtl.svelte-1gs18n1{border-right-color:var(--border-focus)}.arrow.svelte-uvrlso{background-color:var(--canvas-elevated);width:10px;height:10px;z-index:60;border:1px solid var(--border-subtle);border-right:none;border-bottom:none}span.floating-reference.svelte-1djxfit{line-height:1}.floating.svelte-1djxfit{position:absolute;border-radius:5px;z-index:890}.floating-arrow.svelte-1djxfit{position:absolute}
