# Copyright: Ankitects Pty Ltd and contributors
# License: GNU AGPL, version 3 or later; http://www.gnu.org/licenses/agpl.html

# This file was automatically generated from _root-vars.scss
FONT_SIZE = {
    "comment": "",
    "light": "15px",
    "dark": "15px",
}
BORDER_RADIUS = {
    "comment": "Used to round corners of various UI elements",
    "light": "5px",
    "dark": "5px",
}
BORDER_RADIUS_MEDIUM = {
    "comment": "Used for container corners",
    "light": "12px",
    "dark": "12px",
}
BORDER_RADIUS_LARGE = {
    "comment": "Used for pill-shaped buttons",
    "light": "15px",
    "dark": "15px",
}
TRANSITION = {
    "comment": "Default duration of transitions in milliseconds",
    "light": "180",
    "dark": "180",
}
TRANSITION_MEDIUM = {
    "comment": "Slightly longer transition duration in milliseconds",
    "light": "500",
    "dark": "500",
}
TRANSITION_SLOW = {
    "comment": "Long transition duration in milliseconds",
    "light": "1000",
    "dark": "1000",
}
BLUR = {
    "comment": "Default background blur value",
    "light": "20px",
    "dark": "20px",
}
