div.svelte-1chwxj3{width:var(--width, auto);height:var(--height, auto)}.button-group.svelte-1huz2aw{display:flex;flex-flow:row var(--buttons-wrap)}.save.svelte-5g891u{margin:.15rem .75rem}.arrow-container.svelte-1fy2iva{margin:0 .5em}.clickable.svelte-h1dnv7{cursor:pointer;font-weight:700}
