.container.svelte-zuiu8u{width:100%;background:var(--canvas-elevated);border:1px solid var(--border-subtle);border-radius:var(--border-radius-medium, 10px);padding:1rem 1.75rem .75rem 1.25rem;page-break-inside:avoid}.container.light.svelte-zuiu8u{box-shadow:0 3px 3px -2px #14141433,0 3px 4px #14141424,0 1px 8px #1414141f}.container.dark.svelte-zuiu8u{box-shadow:0 2px 4px -1px #14141433,0 4px 5px #14141424,0 1px 10px #1414141f}.container.rtl.svelte-zuiu8u{padding:1rem 1.25rem .75rem 1.75rem}h1.svelte-zuiu8u{border-bottom:1px solid var(--border);padding-bottom:.25em}.help-badge.svelte-zuiu8u{right:0;top:0;color:#555}.help-badge.rtl.svelte-zuiu8u{right:unset;left:0}.night-mode .help-badge.svelte-zuiu8u{color:var(--fg)}
