import{aL as i,aJ as v,_ as y,n as b,bi as f,bF as l,bz as m,bA as _,p as T,aK as A}from"./W1VwgJDT.mjs";import{B as R}from"./D-j9Rl0W.mjs";function F(d,h,e){var s;i&&(s=A,v());var r=new R(d);y(()=>{var o;var a=(o=h())!=null?o:null;if(i){var c=f(s),u=c===T,p=a!==null;if(u!==p){var t=l();m(t),r.anchor=t,_(!1),r.ensure(a,a&&(n=>e(n,a))),_(!0);return}}r.ensure(a,a&&(n=>e(n,a)))},b)}export{F as c};
