import{W as _,a as w,e as f}from"./D78duh1q.mjs";import"./E-vkuI0X.mjs";import{i as M}from"./8UuMqzDP.mjs";import{a as E,j as k,g as u}from"./CCT7NDl_.mjs";import{bg as z,a3 as C,a5 as I,aq as P,W as m,bo as S,bP as X,bd as x,au as y,aw as d,bu as B,b8 as D}from"./W1VwgJDT.mjs";import{a7 as W}from"./FVFIwcTn.mjs";import{fD as j,fE as q}from"./ceRbsEBD.mjs";import{b as H}from"./Jlfsg1EP.mjs";import{p as i}from"./D-j9Rl0W.mjs";function T(o){throw new Error(`unreachable: ${o}`)}function V(...o){return()=>{for(const t of o)t()}}function Y(o){let t=!1;function n(s){t=s}const r=_(document,"keydown",s=>{s.key==="Escape"&&t&&(s.stopImmediatePropagation(),o&&o())},{capture:!0});return{set:n,remove:r}}var N=y('<button tabindex="-1"><!></button>');function Z(o,t){z(t,!1);let n=i(t,"class",8,""),r=i(t,"tooltip",8,void 0),s=i(t,"iconSize",8,100),b=i(t,"widthMultiplier",8,1),p=i(t,"flipX",8,!1);const h=E();let l=D();k(()=>{h("mount",{span:d(l)})}),M();var a=N(),g=C(a);w(g,{get iconSize(){return s()},get widthMultiplier(){return b()},get flipX(){return p()},children:(e,O)=>{var c=I(),v=P(c);W(v,t,"default",{}),m(e,c)},$$slots:{default:!0}}),S(a),H(a,e=>B(l,e),()=>d(l)),X(()=>{var e;j(a,"title",r()),q(a,1,`badge ${(e=n())!=null?e:""}`,"svelte-1fmrmgo")}),u("click",a,function(e){f.call(this,t,e)}),u("mouseenter",a,function(e){f.call(this,t,e)}),u("mouseleave",a,function(e){f.call(this,t,e)}),m(o,a),x()}export{Z as B,T as a,Y as r,V as s};
