@charset "UTF-8";.select-container.svelte-124tqn{width:100%;pointer-events:all;-webkit-appearance:none;cursor:pointer;border:1px solid var(--border-subtle);border-bottom-color:var(--shadow);background:var(--button-bg);color:var(--fg);border-radius:var(--border-radius);line-height:1.5;height:100%;position:relative;display:flex;flex-flow:row;justify-content:space-between}.select-container.svelte-124tqn:hover{background:linear-gradient(180deg,var(--button-gradient-start) 0%,var(--button-gradient-end) 100%);border:1px solid var(--shadow)}.select-container.svelte-124tqn:active{box-shadow:inset 0 calc(var(--buttons-size, 10px) / 15) calc(var(--buttons-size, 10px) / 5) #00000059;border-color:var(--border-subtle)}.select-container.rtl.svelte-124tqn{direction:rtl}.select-container.svelte-124tqn .inner:where(.svelte-124tqn){flex-grow:1;position:relative}.select-container.svelte-124tqn .inner:where(.svelte-124tqn) .label:where(.svelte-124tqn){position:absolute;top:0;right:.5rem;bottom:0;left:.5rem;white-space:nowrap;overflow:hidden;text-overflow:ellipsis}.disabled.svelte-124tqn{pointer-events:none;opacity:.5}.chevron.svelte-124tqn{height:100%;align-self:flex-end;border-left:1px solid var(--border-subtle)}[dir=rtl] .chevron.svelte-124tqn{border-left:none;border-right:1px solid var(--border-subtle)}
