@charset "UTF-8";.sticky-container.svelte-123l061{position:sticky;top:var(--sticky-top, 0);bottom:0;left:0;right:0;z-index:var(--z-index, 50);background:var(--sticky-bg, var(--canvas));border-style:solid;border-color:var(--sticky-border, var(--border));border-width:var(--sticky-borders, 0)}.label-button.svelte-qwvah{-webkit-appearance:none;cursor:pointer;border:1px solid var(--border-subtle);border-bottom-color:var(--shadow);background:var(--button-bg);color:var(--fg);border-top-left-radius:var(--border-left-radius);border-bottom-left-radius:var(--border-left-radius);border-top-right-radius:var(--border-right-radius);border-bottom-right-radius:var(--border-right-radius);white-space:nowrap;padding:0 calc(var(--buttons-size) / 3);font-size:var(--font-size);width:auto;height:var(--buttons-size)}.label-button.svelte-qwvah:hover{background:linear-gradient(180deg,var(--button-gradient-start) 0%,var(--button-gradient-end) 100%);border:1px solid var(--shadow)}.label-button.svelte-qwvah:active{box-shadow:inset 0 calc(var(--buttons-size, 10px) / 15) calc(var(--buttons-size, 10px) / 5) #00000059;border-color:var(--border-subtle)}.label-button.active.svelte-qwvah{box-shadow:inset 0 calc(var(--buttons-size, 10px) / 15) calc(var(--buttons-size, 10px) / 5) #00000059;background:var(--button-primary-bg);color:#fff;border-color:var(--border)}.label-button[disabled].svelte-qwvah,.label-button[disabled].svelte-qwvah:hover{cursor:not-allowed;color:var(--fg-disabled);box-shadow:none!important;background-color:var(--button-gradient-end);border-bottom-color:var(--border-subtle)}.label-button.primary.svelte-qwvah{-webkit-appearance:none;cursor:pointer;border:none;background:var(--button-primary-bg);color:#fff}.label-button.primary.svelte-qwvah:hover{background:linear-gradient(180deg,var(--button-primary-gradient-start) 0%,var(--button-primary-gradient-end) 100%)}.label-button.primary.svelte-qwvah:active{box-shadow:inset 0 calc(var(--buttons-size, 10px) / 15) calc(var(--buttons-size, 10px) / 5) #00000059;border-color:var(--border-subtle)}.label-button.primary[disabled].svelte-qwvah,.label-button.primary[disabled].svelte-qwvah:hover{cursor:not-allowed;color:var(--fg-disabled);box-shadow:none!important;background-color:var(--button-gradient-end);border-bottom-color:var(--border-subtle)}.label-button.ellipsis.svelte-qwvah{overflow:hidden;text-overflow:ellipsis}
