import"./E-vkuI0X.mjs";import{i as L}from"./8UuMqzDP.mjs";import{a3 as y,bo as w,bP as C,W as v,a5 as N,aq as P,au as D,bg as S,bd as j,aw as g,bu as z,b8 as B}from"./W1VwgJDT.mjs";import{a7 as q}from"./FVFIwcTn.mjs";import{fD as o,fE as x}from"./ceRbsEBD.mjs";import{b as H,p as M}from"./DhwOeGsE.mjs";import{p as e}from"./D-j9Rl0W.mjs";import{C as R}from"./V_xEZX8m.mjs";import{a as W,j as A,g as p}from"./CCT7NDl_.mjs";import{b as F}from"./Jlfsg1EP.mjs";import{e as k}from"./D78duh1q.mjs";var G=D("<div><!></div>");function tt(r,t){let d=e(t,"id",8,void 0),f=e(t,"class",8,""),b=e(t,"height",12,0),m=e(t,"breakpoint",8,"fluid");var i=G(),u=y(i);R(u,{get breakpoint(){return m()},children:(s,h)=>{var n=N(),c=P(n);q(c,t,"default",{}),v(s,n)},$$slots:{default:!0}}),w(i),C(()=>{var s;o(i,"id",d()),x(i,1,`sticky-container ${(s=f())!=null?s:""}`,"svelte-123l061")}),H(i,"offsetHeight",b),v(r,i)}var I=D("<button><!></button>");function et(r,t){S(t,!1);let d=e(t,"id",8,void 0),f=e(t,"class",8,""),b=e(t,"primary",8,!1),m=e(t,"tooltip",8,void 0),i=e(t,"active",8,!1),u=e(t,"disabled",8,!1),s=e(t,"tabbable",8,!1),h=e(t,"ellipsis",8,!1),n=B();const c=W();A(()=>c("mount",{button:g(n)})),L();var a=I();let _;var E=y(a);q(E,t,"default",{}),w(a),F(a,l=>z(n,l),()=>g(n)),C(()=>{var l;o(a,"id",d()),_=x(a,1,`label-button ${(l=f())!=null?l:""}`,"svelte-qwvah",_,{active:i(),primary:b(),ellipsis:h()}),o(a,"title",m()),a.disabled=u(),o(a,"tabindex",s()?0:-1)}),p("click",a,function(l){k.call(this,t,l)}),p("mousedown",a,M(function(l){k.call(this,t,l)})),v(r,a),j()}export{et as L,tt as S};
