import{aL as t,aJ as i,a_ as c,_ as f}from"./W1VwgJDT.mjs";import{B as y}from"./D-j9Rl0W.mjs";const l=Symbol("NaN");function h(r,e,n){t&&i();var o=new y(r),s=!c();f(()=>{var a=e();a!==a&&(a=l),s&&a!==null&&typeof a=="object"&&(a={}),o.ensure(a,n)})}export{h as k};
