.setting-title.svelte-fly1c9{cursor:help}.setting-title.svelte-fly1c9:hover{text-decoration:underline dotted var(--fg-subtle)}.revert.svelte-1pfrxat{position:absolute;right:-1.7em;bottom:-1px;color:var(--fg-faint)}.revert.rtl.svelte-1pfrxat{right:unset;left:-1.7em}.config-input.svelte-1pfrxat:hover .revert:where(.svelte-1pfrxat),.config-input.svelte-1pfrxat:focus-within .revert:where(.svelte-1pfrxat){color:var(--fg-subtle)}.config-input.svelte-1pfrxat .revert:where(.svelte-1pfrxat):hover{color:var(--fg)}.badge{cursor:pointer}.hide.svelte-hl2p3x .badge{display:none;cursor:initial}label.svelte-17f4zrl{display:inline}.form-switch.svelte-184lr4n{padding-left:.5em}.form-check-input.svelte-184lr4n{-webkit-appearance:none;-moz-appearance:none;appearance:none;height:1.5em;margin-top:0}.form-switch.svelte-184lr4n .form-check-input:where(.svelte-184lr4n){width:3em;margin-left:1.5em;cursor:pointer}.nightMode.svelte-184lr4n:not(:checked){background-color:var(--canvas-elevated);border-color:var(--border)}.form-switch.rtl.svelte-184lr4n{padding-left:0;padding-right:.5em}.form-switch.rtl.svelte-184lr4n .form-check-input:where(.svelte-184lr4n){margin-left:0;margin-right:1.5em}
