@charset "UTF-8";.tag-input.svelte-3n5c0h{width:100%;color:var(--fg);background:none;resize:none;-webkit-appearance:none;-moz-appearance:none;appearance:none;font:inherit;font-size:var(--font-size);outline:none;border:none;margin:0}.tag-input.svelte-3n5c0h{border:1px solid transparent!important}.tag-add-button.svelte-a3z2w1{line-height:1}.tag-add-button.svelte-a3z2w1 svg:last-child{display:none}.tag-add-button.svelte-a3z2w1:hover svg:first-child{display:none}.tag-add-button.svelte-a3z2w1:hover svg:last-child{display:block}.tag-add-button.svelte-a3z2w1 svg{padding-bottom:2px;cursor:pointer;fill:currentColor;opacity:.6}.tag-add-button.svelte-a3z2w1 svg:hover{opacity:1}.tag-add-button.svelte-a3z2w1 .tags-info:where(.svelte-a3z2w1){cursor:pointer;color:var(--fg-subtle);margin-left:.75rem}[dir=rtl] .tags-info.svelte-a3z2w1{margin-right:.75rem}.tags-selected-button.svelte-ebguzy{line-height:1}.tags-selected-button.svelte-ebguzy svg{cursor:pointer;fill:currentColor;opacity:.6}.tags-selected-button.svelte-ebguzy svg:hover{opacity:1}.tag-options-button.svelte-1se1fg8{transition:opacity .2s linear;opacity:var(--button-opacity, 1)}@keyframes svelte-17ae53l-flash{0%{filter:invert(0)}50%{filter:invert(.4)}to{filter:invert(0)}}.tag.svelte-17ae53l{-webkit-appearance:none;cursor:pointer;border:1px solid var(--border-subtle);border-bottom-color:var(--shadow);background:var(--button-bg);color:var(--fg);vertical-align:middle;font-size:var(--font-size);padding:0;--border-color: var(--border);border:1px solid var(--border-color)!important;border-radius:5px}.tag.svelte-17ae53l:hover{background:linear-gradient(180deg,var(--button-gradient-start) 0%,var(--button-gradient-end) 100%);border:1px solid var(--shadow)}.tag.svelte-17ae53l:focus,.tag.svelte-17ae53l:active{outline:none;box-shadow:none}.tag.flashing.svelte-17ae53l{animation:svelte-17ae53l-flash .3s linear}.tag.selected.svelte-17ae53l{box-shadow:0 0 0 2px var(--border-focus);--border-color: var(--border-focus)}.select-mode.svelte-1hb1ruc button:hover{display:contents;cursor:crosshair}.select-mode.svelte-1hb1ruc button:hover .tag-icon-hover{opacity:0}.tag-icon-hover svg:hover{border-radius:5px;background-color:#0000001a}.night-mode.svelte-1hb1ruc .tag-icon-hover svg:hover{background-color:#ffffff59}.empty.svelte-1hb1ruc{visibility:hidden}.tag-spacer.svelte-2r68kp{cursor:text;flex-grow:1;align-self:stretch;margin-top:3px}.autocomplete-item.svelte-1cj1gug{padding:4px 8px;text-align:start;white-space:nowrap;flex-grow:1;border-radius:0;border:1px solid transparent}.autocomplete-item.svelte-1cj1gug:not(:first-child){border-top-color:var(--border-subtle)}.autocomplete-item.svelte-1cj1gug:hover{-webkit-appearance:none;cursor:pointer;border:1px solid var(--border-subtle);border-bottom-color:var(--shadow);background:var(--button-bg);color:var(--fg)}.autocomplete-item.svelte-1cj1gug:hover:hover{background:linear-gradient(180deg,var(--button-gradient-start) 0%,var(--button-gradient-end) 100%);border:1px solid var(--shadow)}.autocomplete-item.svelte-1cj1gug:hover:active{box-shadow:inset 0 calc(var(--buttons-size, 10px) / 15) calc(var(--buttons-size, 10px) / 5) #00000059;border-color:var(--border-subtle)}.autocomplete-item:hover.active.svelte-1cj1gug{box-shadow:inset 0 calc(var(--buttons-size, 10px) / 15) calc(var(--buttons-size, 10px) / 5) #00000059;background:var(--button-primary-bg);color:#fff;border-color:var(--border)}.autocomplete-item.selected.svelte-1cj1gug{-webkit-appearance:none;cursor:pointer;border:none;background:var(--button-primary-bg);color:#fff}.autocomplete-item.selected.svelte-1cj1gug:hover{background:linear-gradient(180deg,var(--button-primary-gradient-start) 0%,var(--button-primary-gradient-end) 100%)}.autocomplete-item.selected.svelte-1cj1gug:active{box-shadow:inset 0 calc(var(--buttons-size, 10px) / 15) calc(var(--buttons-size, 10px) / 5) #00000059;border-color:var(--border-subtle)}.autocomplete-item.selected.active.svelte-1cj1gug{box-shadow:inset 0 calc(var(--buttons-size, 10px) / 15) calc(var(--buttons-size, 10px) / 5) #00000059;background:var(--button-primary-bg);color:#fff;border-color:var(--border)}.autocomplete-reference.svelte-yt7vbl{position:absolute;top:0;left:0;right:0;bottom:0;border-left:1px solid transparent;border-bottom:1px solid transparent}.autocomplete-menu.svelte-yt7vbl{display:flex;flex-flow:column nowrap;width:80vw;max-height:30vh;font-size:13px;overflow-x:hidden;text-overflow:ellipsis;overflow-y:auto}.tag-editor.svelte-1jt07lg{display:flex;flex-grow:1;flex-flow:row wrap;align-items:flex-end;background:var(--canvas-elevated);border:1px solid var(--border);border-radius:var(--border-radius);padding:6px;margin:1px 3px 3px 1px}.tag-editor.svelte-1jt07lg:focus-within{outline-offset:-1px;outline:2px solid var(--border-focus)}.tag-relative.svelte-1jt07lg{position:relative;padding:0 1px}.hide-tag.svelte-1jt07lg .tag{visibility:hidden}
