# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: anki/ankihub.proto
# Protobuf Python Version: 7.34.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    7,
    34,
    1,
    '',
    'anki/ankihub.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from anki import generic_pb2 as anki_dot_generic__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12\x61nki/ankihub.proto\x12\x0c\x61nki.ankihub\x1a\x12\x61nki/generic.proto\"\x1e\n\rLoginResponse\x12\r\n\x05token\x18\x01 \x01(\t\",\n\x0cLoginRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12\x10\n\x08password\x18\x02 \x01(\t\"\x1e\n\rLogoutRequest\x12\r\n\x05token\x18\x01 \x01(\t2\x10\n\x0e\x41nkiHubService2\xa3\x01\n\x15\x42\x61\x63kendAnkiHubService\x12G\n\x0c\x41nkihubLogin\x12\x1a.anki.ankihub.LoginRequest\x1a\x1b.anki.ankihub.LoginResponse\x12\x41\n\rAnkihubLogout\x12\x1b.anki.ankihub.LogoutRequest\x1a\x13.anki.generic.EmptyB\x02P\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'anki.ankihub_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'P\001'
  _globals['_LOGINRESPONSE']._serialized_start=56
  _globals['_LOGINRESPONSE']._serialized_end=86
  _globals['_LOGINREQUEST']._serialized_start=88
  _globals['_LOGINREQUEST']._serialized_end=132
  _globals['_LOGOUTREQUEST']._serialized_start=134
  _globals['_LOGOUTREQUEST']._serialized_end=164
  _globals['_ANKIHUBSERVICE']._serialized_start=166
  _globals['_ANKIHUBSERVICE']._serialized_end=182
  _globals['_BACKENDANKIHUBSERVICE']._serialized_start=185
  _globals['_BACKENDANKIHUBSERVICE']._serialized_end=348
# @@protoc_insertion_point(module_scope)
