# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: anki/image_occlusion.proto
# Protobuf Python Version: 7.34.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    7,
    34,
    1,
    '',
    'anki/image_occlusion.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from anki import collection_pb2 as anki_dot_collection__pb2
from anki import generic_pb2 as anki_dot_generic__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1a\x61nki/image_occlusion.proto\x12\x14\x61nki.image_occlusion\x1a\x15\x61nki/collection.proto\x1a\x12\x61nki/generic.proto\"+\n\x1bGetImageForOcclusionRequest\x12\x0c\n\x04path\x18\x01 \x01(\t\":\n\x1cGetImageForOcclusionResponse\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\x12\x0c\n\x04name\x18\x02 \x01(\t\"\x8d\x01\n\x1c\x41\x64\x64ImageOcclusionNoteRequest\x12\x12\n\nimage_path\x18\x01 \x01(\t\x12\x12\n\nocclusions\x18\x02 \x01(\t\x12\x0e\n\x06header\x18\x03 \x01(\t\x12\x12\n\nback_extra\x18\x04 \x01(\t\x12\x0c\n\x04tags\x18\x05 \x03(\t\x12\x13\n\x0bnotetype_id\x18\x06 \x01(\x03\"/\n\x1cGetImageOcclusionNoteRequest\x12\x0f\n\x07note_id\x18\x01 \x01(\x03\"\xb3\x05\n\x1dGetImageOcclusionNoteResponse\x12V\n\x04note\x18\x01 \x01(\x0b\x32\x46.anki.image_occlusion.GetImageOcclusionNoteResponse.ImageOcclusionNoteH\x00\x12\x0f\n\x05\x65rror\x18\x02 \x01(\tH\x00\x1a\x35\n\x16ImageOcclusionProperty\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\x1a\x84\x01\n\x13ImageOcclusionShape\x12\r\n\x05shape\x18\x01 \x01(\t\x12^\n\nproperties\x18\x02 \x03(\x0b\x32J.anki.image_occlusion.GetImageOcclusionNoteResponse.ImageOcclusionProperty\x1az\n\x0eImageOcclusion\x12W\n\x06shapes\x18\x01 \x03(\x0b\x32G.anki.image_occlusion.GetImageOcclusionNoteResponse.ImageOcclusionShape\x12\x0f\n\x07ordinal\x18\x02 \x01(\r\x1a\xe5\x01\n\x12ImageOcclusionNote\x12\x12\n\nimage_data\x18\x01 \x01(\x0c\x12V\n\nocclusions\x18\x02 \x03(\x0b\x32\x42.anki.image_occlusion.GetImageOcclusionNoteResponse.ImageOcclusion\x12\x0e\n\x06header\x18\x03 \x01(\t\x12\x12\n\nback_extra\x18\x04 \x01(\t\x12\x0c\n\x04tags\x18\x05 \x03(\t\x12\x17\n\x0fimage_file_name\x18\x06 \x01(\t\x12\x18\n\x10occlude_inactive\x18\x07 \x01(\x08\x42\x07\n\x05value\"x\n\x1fUpdateImageOcclusionNoteRequest\x12\x0f\n\x07note_id\x18\x01 \x01(\x03\x12\x12\n\nocclusions\x18\x02 \x01(\t\x12\x0e\n\x06header\x18\x03 \x01(\t\x12\x12\n\nback_extra\x18\x04 \x01(\t\x12\x0c\n\x04tags\x18\x05 \x03(\t\"5\n\x1eGetImageOcclusionFieldsRequest\x12\x13\n\x0bnotetype_id\x18\x01 \x01(\x03\"c\n\x1fGetImageOcclusionFieldsResponse\x12@\n\x06\x66ields\x18\x01 \x01(\x0b\x32\x30.anki.image_occlusion.ImageOcclusionFieldIndexes\"c\n\x1aImageOcclusionFieldIndexes\x12\x12\n\nocclusions\x18\x01 \x01(\r\x12\r\n\x05image\x18\x02 \x01(\r\x12\x0e\n\x06header\x18\x03 \x01(\r\x12\x12\n\nback_extra\x18\x04 \x01(\r2\xc8\x05\n\x15ImageOcclusionService\x12}\n\x14GetImageForOcclusion\x12\x31.anki.image_occlusion.GetImageForOcclusionRequest\x1a\x32.anki.image_occlusion.GetImageForOcclusionResponse\x12\x80\x01\n\x15GetImageOcclusionNote\x12\x32.anki.image_occlusion.GetImageOcclusionNoteRequest\x1a\x33.anki.image_occlusion.GetImageOcclusionNoteResponse\x12\x86\x01\n\x17GetImageOcclusionFields\x12\x34.anki.image_occlusion.GetImageOcclusionFieldsRequest\x1a\x35.anki.image_occlusion.GetImageOcclusionFieldsResponse\x12L\n\x19\x41\x64\x64ImageOcclusionNotetype\x12\x13.anki.generic.Empty\x1a\x1a.anki.collection.OpChanges\x12g\n\x15\x41\x64\x64ImageOcclusionNote\x12\x32.anki.image_occlusion.AddImageOcclusionNoteRequest\x1a\x1a.anki.collection.OpChanges\x12m\n\x18UpdateImageOcclusionNote\x12\x35.anki.image_occlusion.UpdateImageOcclusionNoteRequest\x1a\x1a.anki.collection.OpChanges2\x1e\n\x1c\x42\x61\x63kendImageOcclusionServiceB\x02P\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'anki.image_occlusion_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'P\001'
  _globals['_GETIMAGEFOROCCLUSIONREQUEST']._serialized_start=95
  _globals['_GETIMAGEFOROCCLUSIONREQUEST']._serialized_end=138
  _globals['_GETIMAGEFOROCCLUSIONRESPONSE']._serialized_start=140
  _globals['_GETIMAGEFOROCCLUSIONRESPONSE']._serialized_end=198
  _globals['_ADDIMAGEOCCLUSIONNOTEREQUEST']._serialized_start=201
  _globals['_ADDIMAGEOCCLUSIONNOTEREQUEST']._serialized_end=342
  _globals['_GETIMAGEOCCLUSIONNOTEREQUEST']._serialized_start=344
  _globals['_GETIMAGEOCCLUSIONNOTEREQUEST']._serialized_end=391
  _globals['_GETIMAGEOCCLUSIONNOTERESPONSE']._serialized_start=394
  _globals['_GETIMAGEOCCLUSIONNOTERESPONSE']._serialized_end=1085
  _globals['_GETIMAGEOCCLUSIONNOTERESPONSE_IMAGEOCCLUSIONPROPERTY']._serialized_start=532
  _globals['_GETIMAGEOCCLUSIONNOTERESPONSE_IMAGEOCCLUSIONPROPERTY']._serialized_end=585
  _globals['_GETIMAGEOCCLUSIONNOTERESPONSE_IMAGEOCCLUSIONSHAPE']._serialized_start=588
  _globals['_GETIMAGEOCCLUSIONNOTERESPONSE_IMAGEOCCLUSIONSHAPE']._serialized_end=720
  _globals['_GETIMAGEOCCLUSIONNOTERESPONSE_IMAGEOCCLUSION']._serialized_start=722
  _globals['_GETIMAGEOCCLUSIONNOTERESPONSE_IMAGEOCCLUSION']._serialized_end=844
  _globals['_GETIMAGEOCCLUSIONNOTERESPONSE_IMAGEOCCLUSIONNOTE']._serialized_start=847
  _globals['_GETIMAGEOCCLUSIONNOTERESPONSE_IMAGEOCCLUSIONNOTE']._serialized_end=1076
  _globals['_UPDATEIMAGEOCCLUSIONNOTEREQUEST']._serialized_start=1087
  _globals['_UPDATEIMAGEOCCLUSIONNOTEREQUEST']._serialized_end=1207
  _globals['_GETIMAGEOCCLUSIONFIELDSREQUEST']._serialized_start=1209
  _globals['_GETIMAGEOCCLUSIONFIELDSREQUEST']._serialized_end=1262
  _globals['_GETIMAGEOCCLUSIONFIELDSRESPONSE']._serialized_start=1264
  _globals['_GETIMAGEOCCLUSIONFIELDSRESPONSE']._serialized_end=1363
  _globals['_IMAGEOCCLUSIONFIELDINDEXES']._serialized_start=1365
  _globals['_IMAGEOCCLUSIONFIELDINDEXES']._serialized_end=1464
  _globals['_IMAGEOCCLUSIONSERVICE']._serialized_start=1467
  _globals['_IMAGEOCCLUSIONSERVICE']._serialized_end=2179
  _globals['_BACKENDIMAGEOCCLUSIONSERVICE']._serialized_start=2181
  _globals['_BACKENDIMAGEOCCLUSIONSERVICE']._serialized_end=2211
# @@protoc_insertion_point(module_scope)
