# Form implementation generated from reading ui file 'qt/aqt/forms/preferences.ui'
#
# Created by: PyQt6 UI code generator 6.11.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets
from aqt.utils import tr



class Ui_Preferences(object):
    def setupUi(self, Preferences):
        Preferences.setObjectName("Preferences")
        Preferences.resize(636, 638)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(Preferences)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.tabWidget = QtWidgets.QTabWidget(parent=Preferences)
        self.tabWidget.setFocusPolicy(QtCore.Qt.FocusPolicy.StrongFocus)
        self.tabWidget.setObjectName("tabWidget")
        self.appearanceTab = QtWidgets.QWidget()
        self.appearanceTab.setObjectName("appearanceTab")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout(self.appearanceTab)
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.generalGroup = QtWidgets.QGroupBox(parent=self.appearanceTab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.generalGroup.sizePolicy().hasHeightForWidth())
        self.generalGroup.setSizePolicy(sizePolicy)
        self.generalGroup.setObjectName("generalGroup")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.generalGroup)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.label_9 = QtWidgets.QLabel(parent=self.generalGroup)
        self.label_9.setObjectName("label_9")
        self.gridLayout_5.addWidget(self.label_9, 0, 0, 1, 1)
        self.video_driver = QtWidgets.QComboBox(parent=self.generalGroup)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.video_driver.sizePolicy().hasHeightForWidth())
        self.video_driver.setSizePolicy(sizePolicy)
        self.video_driver.setObjectName("video_driver")
        self.gridLayout_5.addWidget(self.video_driver, 1, 1, 1, 1)
        self.video_driver_label = QtWidgets.QLabel(parent=self.generalGroup)
        self.video_driver_label.setObjectName("video_driver_label")
        self.gridLayout_5.addWidget(self.video_driver_label, 1, 0, 1, 1)
        self.lang = QtWidgets.QComboBox(parent=self.generalGroup)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lang.sizePolicy().hasHeightForWidth())
        self.lang.setSizePolicy(sizePolicy)
        self.lang.setSizeAdjustPolicy(QtWidgets.QComboBox.SizeAdjustPolicy.AdjustToMinimumContentsLengthWithIcon)
        self.lang.setObjectName("lang")
        self.gridLayout_5.addWidget(self.lang, 0, 1, 1, 1)
        self.check_for_updates = QtWidgets.QCheckBox(parent=self.generalGroup)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Fixed, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.check_for_updates.sizePolicy().hasHeightForWidth())
        self.check_for_updates.setSizePolicy(sizePolicy)
        self.check_for_updates.setObjectName("check_for_updates")
        self.gridLayout_5.addWidget(self.check_for_updates, 2, 0, 1, 1)
        self.verticalLayout_8.addWidget(self.generalGroup)
        self.uiGroup = QtWidgets.QGroupBox(parent=self.appearanceTab)
        self.uiGroup.setObjectName("uiGroup")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.uiGroup)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.theme = QtWidgets.QComboBox(parent=self.uiGroup)
        self.theme.setObjectName("theme")
        self.gridLayout_2.addWidget(self.theme, 0, 1, 1, 1)
        self.uiScale = QtWidgets.QSpinBox(parent=self.uiGroup)
        self.uiScale.setSuffix("%")
        self.uiScale.setMinimum(100)
        self.uiScale.setMaximum(200)
        self.uiScale.setSingleStep(5)
        self.uiScale.setObjectName("uiScale")
        self.gridLayout_2.addWidget(self.uiScale, 2, 1, 1, 1)
        self.styleComboBox = QtWidgets.QComboBox(parent=self.uiGroup)
        self.styleComboBox.setCurrentText("")
        self.styleComboBox.setObjectName("styleComboBox")
        self.gridLayout_2.addWidget(self.styleComboBox, 1, 1, 1, 1)
        self.styleLabel = QtWidgets.QLabel(parent=self.uiGroup)
        self.styleLabel.setObjectName("styleLabel")
        self.gridLayout_2.addWidget(self.styleLabel, 1, 0, 1, 1)
        self.themeLabel = QtWidgets.QLabel(parent=self.uiGroup)
        self.themeLabel.setObjectName("themeLabel")
        self.gridLayout_2.addWidget(self.themeLabel, 0, 0, 1, 1)
        self.uiSizeLabel = QtWidgets.QLabel(parent=self.uiGroup)
        self.uiSizeLabel.setObjectName("uiSizeLabel")
        self.gridLayout_2.addWidget(self.uiSizeLabel, 2, 0, 1, 1)
        self.resetWindowSizes = QtWidgets.QPushButton(parent=self.uiGroup)
        self.resetWindowSizes.setObjectName("resetWindowSizes")
        self.gridLayout_2.addWidget(self.resetWindowSizes, 3, 0, 1, 2)
        self.verticalLayout_8.addWidget(self.uiGroup)
        self.distractionsGroup = QtWidgets.QGroupBox(parent=self.appearanceTab)
        self.distractionsGroup.setObjectName("distractionsGroup")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.distractionsGroup)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.reduce_motion = QtWidgets.QCheckBox(parent=self.distractionsGroup)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Fixed, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.reduce_motion.sizePolicy().hasHeightForWidth())
        self.reduce_motion.setSizePolicy(sizePolicy)
        self.reduce_motion.setObjectName("reduce_motion")
        self.gridLayout_3.addWidget(self.reduce_motion, 3, 0, 1, 1)
        self.hide_bottom_bar = QtWidgets.QCheckBox(parent=self.distractionsGroup)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.MinimumExpanding, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.hide_bottom_bar.sizePolicy().hasHeightForWidth())
        self.hide_bottom_bar.setSizePolicy(sizePolicy)
        self.hide_bottom_bar.setObjectName("hide_bottom_bar")
        self.gridLayout_3.addWidget(self.hide_bottom_bar, 2, 0, 1, 1)
        self.bottomBarComboBox = QtWidgets.QComboBox(parent=self.distractionsGroup)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.MinimumExpanding, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.bottomBarComboBox.sizePolicy().hasHeightForWidth())
        self.bottomBarComboBox.setSizePolicy(sizePolicy)
        self.bottomBarComboBox.setObjectName("bottomBarComboBox")
        self.gridLayout_3.addWidget(self.bottomBarComboBox, 2, 1, 1, 1)
        self.hide_top_bar = QtWidgets.QCheckBox(parent=self.distractionsGroup)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.MinimumExpanding, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.hide_top_bar.sizePolicy().hasHeightForWidth())
        self.hide_top_bar.setSizePolicy(sizePolicy)
        self.hide_top_bar.setObjectName("hide_top_bar")
        self.gridLayout_3.addWidget(self.hide_top_bar, 0, 0, 1, 1)
        self.minimalist_mode = QtWidgets.QCheckBox(parent=self.distractionsGroup)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Fixed, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.minimalist_mode.sizePolicy().hasHeightForWidth())
        self.minimalist_mode.setSizePolicy(sizePolicy)
        self.minimalist_mode.setObjectName("minimalist_mode")
        self.gridLayout_3.addWidget(self.minimalist_mode, 4, 0, 1, 1)
        self.topBarComboBox = QtWidgets.QComboBox(parent=self.distractionsGroup)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.MinimumExpanding, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.topBarComboBox.sizePolicy().hasHeightForWidth())
        self.topBarComboBox.setSizePolicy(sizePolicy)
        self.topBarComboBox.setObjectName("topBarComboBox")
        self.gridLayout_3.addWidget(self.topBarComboBox, 0, 1, 1, 1)
        self.verticalLayout_8.addWidget(self.distractionsGroup)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.verticalLayout_8.addItem(spacerItem)
        self.tabWidget.addTab(self.appearanceTab, "")
        self.tab_3 = QtWidgets.QWidget()
        self.tab_3.setObjectName("tab_3")
        self.review_options_layout = QtWidgets.QVBoxLayout(self.tab_3)
        self.review_options_layout.setObjectName("review_options_layout")
        self.schedulerGroup = QtWidgets.QGroupBox(parent=self.tab_3)
        self.schedulerGroup.setObjectName("schedulerGroup")
        self.verticalLayout_17 = QtWidgets.QVBoxLayout(self.schedulerGroup)
        self.verticalLayout_17.setObjectName("verticalLayout_17")
        self.gridLayout_4 = QtWidgets.QGridLayout()
        self.gridLayout_4.setSpacing(12)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.lrnCutoff = QtWidgets.QSpinBox(parent=self.schedulerGroup)
        self.lrnCutoff.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.lrnCutoff.setMaximum(999)
        self.lrnCutoff.setObjectName("lrnCutoff")
        self.gridLayout_4.addWidget(self.lrnCutoff, 1, 1, 1, 1)
        self.label_30 = QtWidgets.QLabel(parent=self.schedulerGroup)
        self.label_30.setObjectName("label_30")
        self.gridLayout_4.addWidget(self.label_30, 2, 0, 1, 1)
        self.dayOffset = QtWidgets.QSpinBox(parent=self.schedulerGroup)
        self.dayOffset.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.dayOffset.setMaximum(23)
        self.dayOffset.setObjectName("dayOffset")
        self.gridLayout_4.addWidget(self.dayOffset, 0, 1, 1, 1)
        self.label_24 = QtWidgets.QLabel(parent=self.schedulerGroup)
        self.label_24.setObjectName("label_24")
        self.gridLayout_4.addWidget(self.label_24, 1, 0, 1, 1)
        self.label_23 = QtWidgets.QLabel(parent=self.schedulerGroup)
        self.label_23.setObjectName("label_23")
        self.gridLayout_4.addWidget(self.label_23, 0, 0, 1, 1)
        self.timeLimit = QtWidgets.QSpinBox(parent=self.schedulerGroup)
        self.timeLimit.setMaximum(9999)
        self.timeLimit.setObjectName("timeLimit")
        self.gridLayout_4.addWidget(self.timeLimit, 2, 1, 1, 1)
        self.verticalLayout_17.addLayout(self.gridLayout_4)
        self.review_options_layout.addWidget(self.schedulerGroup)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.reviewerGroup = QtWidgets.QGroupBox(parent=self.tab_3)
        self.reviewerGroup.setObjectName("reviewerGroup")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.reviewerGroup)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.showPlayButtons = QtWidgets.QCheckBox(parent=self.reviewerGroup)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Fixed, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.showPlayButtons.sizePolicy().hasHeightForWidth())
        self.showPlayButtons.setSizePolicy(sizePolicy)
        self.showPlayButtons.setObjectName("showPlayButtons")
        self.verticalLayout_5.addWidget(self.showPlayButtons)
        self.interrupt_audio = QtWidgets.QCheckBox(parent=self.reviewerGroup)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Fixed, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.interrupt_audio.sizePolicy().hasHeightForWidth())
        self.interrupt_audio.setSizePolicy(sizePolicy)
        self.interrupt_audio.setObjectName("interrupt_audio")
        self.verticalLayout_5.addWidget(self.interrupt_audio)
        self.showProgress = QtWidgets.QCheckBox(parent=self.reviewerGroup)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Fixed, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.showProgress.sizePolicy().hasHeightForWidth())
        self.showProgress.setSizePolicy(sizePolicy)
        self.showProgress.setObjectName("showProgress")
        self.verticalLayout_5.addWidget(self.showProgress)
        self.showEstimates = QtWidgets.QCheckBox(parent=self.reviewerGroup)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Fixed, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.showEstimates.sizePolicy().hasHeightForWidth())
        self.showEstimates.setSizePolicy(sizePolicy)
        self.showEstimates.setObjectName("showEstimates")
        self.verticalLayout_5.addWidget(self.showEstimates)
        self.spacebar_rates_card = QtWidgets.QCheckBox(parent=self.reviewerGroup)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Fixed, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.spacebar_rates_card.sizePolicy().hasHeightForWidth())
        self.spacebar_rates_card.setSizePolicy(sizePolicy)
        self.spacebar_rates_card.setObjectName("spacebar_rates_card")
        self.verticalLayout_5.addWidget(self.spacebar_rates_card)
        self.render_latex = QtWidgets.QCheckBox(parent=self.reviewerGroup)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Fixed, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.render_latex.sizePolicy().hasHeightForWidth())
        self.render_latex.setSizePolicy(sizePolicy)
        self.render_latex.setObjectName("render_latex")
        self.verticalLayout_5.addWidget(self.render_latex)
        self.url_schemes = QtWidgets.QPushButton(parent=self.reviewerGroup)
        self.url_schemes.setObjectName("url_schemes")
        self.verticalLayout_5.addWidget(self.url_schemes)
        self.horizontalLayout_5.addWidget(self.reviewerGroup)
        self.preferences_answer_keys = QtWidgets.QGroupBox(parent=self.tab_3)
        self.preferences_answer_keys.setObjectName("preferences_answer_keys")
        self.horizontalLayout_5.addWidget(self.preferences_answer_keys)
        self.review_options_layout.addLayout(self.horizontalLayout_5)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.review_options_layout.addItem(spacerItem1)
        self.tabWidget.addTab(self.tab_3, "")
        self.tab_1 = QtWidgets.QWidget()
        self.tab_1.setObjectName("tab_1")
        self.verticalLayout_10 = QtWidgets.QVBoxLayout(self.tab_1)
        self.verticalLayout_10.setObjectName("verticalLayout_10")
        self.editorGroupBox = QtWidgets.QGroupBox(parent=self.tab_1)
        self.editorGroupBox.setObjectName("editorGroupBox")
        self.verticalLayout_9 = QtWidgets.QVBoxLayout(self.editorGroupBox)
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        self.pastePNG = QtWidgets.QCheckBox(parent=self.editorGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Fixed, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.pastePNG.sizePolicy().hasHeightForWidth())
        self.pastePNG.setSizePolicy(sizePolicy)
        self.pastePNG.setObjectName("pastePNG")
        self.verticalLayout_9.addWidget(self.pastePNG)
        self.paste_strips_formatting = QtWidgets.QCheckBox(parent=self.editorGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Fixed, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.paste_strips_formatting.sizePolicy().hasHeightForWidth())
        self.paste_strips_formatting.setSizePolicy(sizePolicy)
        self.paste_strips_formatting.setObjectName("paste_strips_formatting")
        self.verticalLayout_9.addWidget(self.paste_strips_formatting)
        spacerItem2 = QtWidgets.QSpacerItem(20, 10, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Fixed)
        self.verticalLayout_9.addItem(spacerItem2)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.deck_label = QtWidgets.QLabel(parent=self.editorGroupBox)
        self.deck_label.setObjectName("deck_label")
        self.horizontalLayout_7.addWidget(self.deck_label)
        self.useCurrent = QtWidgets.QComboBox(parent=self.editorGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.MinimumExpanding, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.useCurrent.sizePolicy().hasHeightForWidth())
        self.useCurrent.setSizePolicy(sizePolicy)
        self.useCurrent.setObjectName("useCurrent")
        self.useCurrent.addItem("")
        self.useCurrent.addItem("")
        self.horizontalLayout_7.addWidget(self.useCurrent)
        self.verticalLayout_9.addLayout(self.horizontalLayout_7)
        self.verticalLayout_10.addWidget(self.editorGroupBox)
        self.browserGroupBox = QtWidgets.QGroupBox(parent=self.tab_1)
        self.browserGroupBox.setObjectName("browserGroupBox")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.browserGroupBox)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.search_text_label = QtWidgets.QLabel(parent=self.browserGroupBox)
        self.search_text_label.setObjectName("search_text_label")
        self.horizontalLayout_2.addWidget(self.search_text_label)
        self.default_search_text = QtWidgets.QLineEdit(parent=self.browserGroupBox)
        self.default_search_text.setObjectName("default_search_text")
        self.horizontalLayout_2.addWidget(self.default_search_text)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.ignore_accents_in_search = QtWidgets.QCheckBox(parent=self.browserGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Fixed, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.ignore_accents_in_search.sizePolicy().hasHeightForWidth())
        self.ignore_accents_in_search.setSizePolicy(sizePolicy)
        self.ignore_accents_in_search.setObjectName("ignore_accents_in_search")
        self.verticalLayout.addWidget(self.ignore_accents_in_search)
        self.verticalLayout_10.addWidget(self.browserGroupBox)
        spacerItem3 = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.verticalLayout_10.addItem(spacerItem3)
        self.tabWidget.addTab(self.tab_1, "")
        self.tab_2 = QtWidgets.QWidget()
        self.tab_2.setObjectName("tab_2")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.tab_2)
        self.verticalLayout_4.setContentsMargins(12, 12, 12, 12)
        self.verticalLayout_4.setSpacing(12)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.groupBox_5 = QtWidgets.QGroupBox(parent=self.tab_2)
        self.groupBox_5.setObjectName("groupBox_5")
        self.verticalLayout_12 = QtWidgets.QVBoxLayout(self.groupBox_5)
        self.verticalLayout_12.setObjectName("verticalLayout_12")
        self.syncMedia = QtWidgets.QCheckBox(parent=self.groupBox_5)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Fixed, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.syncMedia.sizePolicy().hasHeightForWidth())
        self.syncMedia.setSizePolicy(sizePolicy)
        self.syncMedia.setObjectName("syncMedia")
        self.verticalLayout_12.addWidget(self.syncMedia)
        self.syncOnProgramOpen = QtWidgets.QCheckBox(parent=self.groupBox_5)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Fixed, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.syncOnProgramOpen.sizePolicy().hasHeightForWidth())
        self.syncOnProgramOpen.setSizePolicy(sizePolicy)
        self.syncOnProgramOpen.setObjectName("syncOnProgramOpen")
        self.verticalLayout_12.addWidget(self.syncOnProgramOpen)
        self.autoSyncMedia = QtWidgets.QCheckBox(parent=self.groupBox_5)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Fixed, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.autoSyncMedia.sizePolicy().hasHeightForWidth())
        self.autoSyncMedia.setSizePolicy(sizePolicy)
        self.autoSyncMedia.setObjectName("autoSyncMedia")
        self.verticalLayout_12.addWidget(self.autoSyncMedia)
        self.fullSync = QtWidgets.QCheckBox(parent=self.groupBox_5)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Fixed, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.fullSync.sizePolicy().hasHeightForWidth())
        self.fullSync.setSizePolicy(sizePolicy)
        self.fullSync.setObjectName("fullSync")
        self.verticalLayout_12.addWidget(self.fullSync)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.label_2 = QtWidgets.QLabel(parent=self.groupBox_5)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_4.addWidget(self.label_2)
        self.network_timeout = QtWidgets.QSpinBox(parent=self.groupBox_5)
        self.network_timeout.setMinimum(30)
        self.network_timeout.setMaximum(99999)
        self.network_timeout.setObjectName("network_timeout")
        self.horizontalLayout_4.addWidget(self.network_timeout)
        self.verticalLayout_12.addLayout(self.horizontalLayout_4)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem4)
        self.media_log = QtWidgets.QPushButton(parent=self.groupBox_5)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Fixed, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.media_log.sizePolicy().hasHeightForWidth())
        self.media_log.setSizePolicy(sizePolicy)
        self.media_log.setText("")
        self.media_log.setObjectName("media_log")
        self.horizontalLayout_3.addWidget(self.media_log)
        self.verticalLayout_12.addLayout(self.horizontalLayout_3)
        self.verticalLayout_4.addWidget(self.groupBox_5)
        self.groupBox_6 = QtWidgets.QGroupBox(parent=self.tab_2)
        self.groupBox_6.setObjectName("groupBox_6")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.groupBox_6)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.syncUser = QtWidgets.QLabel(parent=self.groupBox_6)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.syncUser.sizePolicy().hasHeightForWidth())
        self.syncUser.setSizePolicy(sizePolicy)
        self.syncUser.setText("")
        self.syncUser.setWordWrap(True)
        self.syncUser.setObjectName("syncUser")
        self.horizontalLayout.addWidget(self.syncUser)
        self.syncLogout = QtWidgets.QPushButton(parent=self.groupBox_6)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Fixed, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.syncLogout.sizePolicy().hasHeightForWidth())
        self.syncLogout.setSizePolicy(sizePolicy)
        self.syncLogout.setAutoDefault(False)
        self.syncLogout.setObjectName("syncLogout")
        self.horizontalLayout.addWidget(self.syncLogout)
        self.syncLogin = QtWidgets.QPushButton(parent=self.groupBox_6)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Fixed, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.syncLogin.sizePolicy().hasHeightForWidth())
        self.syncLogin.setSizePolicy(sizePolicy)
        self.syncLogin.setAutoDefault(False)
        self.syncLogin.setObjectName("syncLogin")
        self.horizontalLayout.addWidget(self.syncLogin)
        self.verticalLayout_4.addWidget(self.groupBox_6)
        spacerItem5 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.verticalLayout_4.addItem(spacerItem5)
        spacerItem6 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.verticalLayout_4.addItem(spacerItem6)
        self.gridLayout_21 = QtWidgets.QGridLayout()
        self.gridLayout_21.setObjectName("gridLayout_21")
        self.custom_sync_url = QtWidgets.QLineEdit(parent=self.tab_2)
        self.custom_sync_url.setObjectName("custom_sync_url")
        self.gridLayout_21.addWidget(self.custom_sync_url, 0, 1, 1, 1)
        self.label = QtWidgets.QLabel(parent=self.tab_2)
        self.label.setObjectName("label")
        self.gridLayout_21.addWidget(self.label, 0, 0, 1, 1)
        self.verticalLayout_4.addLayout(self.gridLayout_21)
        self.tabWidget.addTab(self.tab_2, "")
        self.tab = QtWidgets.QWidget()
        self.tab.setObjectName("tab")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.tab)
        self.verticalLayout_3.setContentsMargins(12, 12, 12, 12)
        self.verticalLayout_3.setSpacing(12)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.groupBox_8 = QtWidgets.QGroupBox(parent=self.tab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_8.sizePolicy().hasHeightForWidth())
        self.groupBox_8.setSizePolicy(sizePolicy)
        self.groupBox_8.setObjectName("groupBox_8")
        self.verticalLayout_15 = QtWidgets.QVBoxLayout(self.groupBox_8)
        self.verticalLayout_15.setObjectName("verticalLayout_15")
        self.backup_explanation = QtWidgets.QLabel(parent=self.groupBox_8)
        self.backup_explanation.setWordWrap(True)
        self.backup_explanation.setObjectName("backup_explanation")
        self.verticalLayout_15.addWidget(self.backup_explanation)
        spacerItem7 = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Fixed)
        self.verticalLayout_15.addItem(spacerItem7)
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.minutes_between_backups = QtWidgets.QSpinBox(parent=self.groupBox_8)
        self.minutes_between_backups.setMinimum(5)
        self.minutes_between_backups.setMaximum(9999)
        self.minutes_between_backups.setObjectName("minutes_between_backups")
        self.gridLayout.addWidget(self.minutes_between_backups, 1, 2, 1, 1)
        self.monthly_backups = QtWidgets.QSpinBox(parent=self.groupBox_8)
        self.monthly_backups.setMaximum(9999)
        self.monthly_backups.setObjectName("monthly_backups")
        self.gridLayout.addWidget(self.monthly_backups, 5, 2, 1, 1)
        spacerItem8 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.gridLayout.addItem(spacerItem8, 1, 3, 1, 1)
        self.label_3 = QtWidgets.QLabel(parent=self.groupBox_8)
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3, 2, 0, 1, 1)
        self.weekly_backups = QtWidgets.QSpinBox(parent=self.groupBox_8)
        self.weekly_backups.setMaximum(9999)
        self.weekly_backups.setObjectName("weekly_backups")
        self.gridLayout.addWidget(self.weekly_backups, 4, 2, 1, 1)
        self.label_6 = QtWidgets.QLabel(parent=self.groupBox_8)
        self.label_6.setObjectName("label_6")
        self.gridLayout.addWidget(self.label_6, 5, 0, 1, 1)
        self.label_5 = QtWidgets.QLabel(parent=self.groupBox_8)
        self.label_5.setObjectName("label_5")
        self.gridLayout.addWidget(self.label_5, 4, 0, 1, 1)
        self.label_7 = QtWidgets.QLabel(parent=self.groupBox_8)
        self.label_7.setObjectName("label_7")
        self.gridLayout.addWidget(self.label_7, 1, 0, 1, 1)
        self.daily_backups = QtWidgets.QSpinBox(parent=self.groupBox_8)
        self.daily_backups.setMaximum(9999)
        self.daily_backups.setObjectName("daily_backups")
        self.gridLayout.addWidget(self.daily_backups, 2, 2, 1, 1)
        spacerItem9 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.gridLayout.addItem(spacerItem9, 1, 1, 1, 1)
        self.verticalLayout_15.addLayout(self.gridLayout)
        spacerItem10 = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Fixed)
        self.verticalLayout_15.addItem(spacerItem10)
        self.openBackupFolder = QtWidgets.QLabel(parent=self.groupBox_8)
        self.openBackupFolder.setObjectName("openBackupFolder")
        self.verticalLayout_15.addWidget(self.openBackupFolder)
        self.verticalLayout_3.addWidget(self.groupBox_8)
        self.groupBox_9 = QtWidgets.QGroupBox(parent=self.tab)
        self.groupBox_9.setObjectName("groupBox_9")
        self.verticalLayout_13 = QtWidgets.QVBoxLayout(self.groupBox_9)
        self.verticalLayout_13.setObjectName("verticalLayout_13")
        self.label_4 = QtWidgets.QLabel(parent=self.groupBox_9)
        self.label_4.setWordWrap(True)
        self.label_4.setObjectName("label_4")
        self.verticalLayout_13.addWidget(self.label_4)
        self.verticalLayout_3.addWidget(self.groupBox_9)
        spacerItem11 = QtWidgets.QSpacerItem(20, 59, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.verticalLayout_3.addItem(spacerItem11)
        self.tabWidget.addTab(self.tab, "")
        self.tab_31 = QtWidgets.QWidget()
        self.tab_31.setObjectName("tab_31")
        self.verticalLayout_41 = QtWidgets.QVBoxLayout(self.tab_31)
        self.verticalLayout_41.setContentsMargins(12, 12, 12, 12)
        self.verticalLayout_41.setSpacing(12)
        self.verticalLayout_41.setObjectName("verticalLayout_41")
        self.label1 = QtWidgets.QLabel(parent=self.tab_31)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label1.sizePolicy().hasHeightForWidth())
        self.label1.setSizePolicy(sizePolicy)
        self.label1.setObjectName("label1")
        self.verticalLayout_41.addWidget(self.label1)
        spacerItem12 = QtWidgets.QSpacerItem(20, 12, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Maximum)
        self.verticalLayout_41.addItem(spacerItem12)
        self.groupBox_61 = QtWidgets.QGroupBox(parent=self.tab_31)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_61.sizePolicy().hasHeightForWidth())
        self.groupBox_61.setSizePolicy(sizePolicy)
        self.groupBox_61.setTitle("AnkiHub")
        self.groupBox_61.setObjectName("groupBox_61")
        self.horizontalLayout1 = QtWidgets.QHBoxLayout(self.groupBox_61)
        self.horizontalLayout1.setObjectName("horizontalLayout1")
        self.syncAnkiHubUser = QtWidgets.QLabel(parent=self.groupBox_61)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.syncAnkiHubUser.sizePolicy().hasHeightForWidth())
        self.syncAnkiHubUser.setSizePolicy(sizePolicy)
        self.syncAnkiHubUser.setText("")
        self.syncAnkiHubUser.setWordWrap(True)
        self.syncAnkiHubUser.setObjectName("syncAnkiHubUser")
        self.horizontalLayout1.addWidget(self.syncAnkiHubUser)
        self.syncAnkiHubLogout = QtWidgets.QPushButton(parent=self.groupBox_61)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Fixed, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.syncAnkiHubLogout.sizePolicy().hasHeightForWidth())
        self.syncAnkiHubLogout.setSizePolicy(sizePolicy)
        self.syncAnkiHubLogout.setAutoDefault(False)
        self.syncAnkiHubLogout.setObjectName("syncAnkiHubLogout")
        self.horizontalLayout1.addWidget(self.syncAnkiHubLogout)
        self.syncAnkiHubLogin = QtWidgets.QPushButton(parent=self.groupBox_61)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Fixed, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.syncAnkiHubLogin.sizePolicy().hasHeightForWidth())
        self.syncAnkiHubLogin.setSizePolicy(sizePolicy)
        self.syncAnkiHubLogin.setAutoDefault(False)
        self.syncAnkiHubLogin.setObjectName("syncAnkiHubLogin")
        self.horizontalLayout1.addWidget(self.syncAnkiHubLogin)
        self.verticalLayout_41.addWidget(self.groupBox_61)
        spacerItem13 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.verticalLayout_41.addItem(spacerItem13)
        self.tabWidget.addTab(self.tab_31, "")
        self.verticalLayout_2.addWidget(self.tabWidget)
        self.someSettingsLabel = QtWidgets.QLabel(parent=Preferences)
        self.someSettingsLabel.setAlignment(QtCore.Qt.AlignmentFlag.AlignCenter)
        self.someSettingsLabel.setObjectName("someSettingsLabel")
        self.verticalLayout_2.addWidget(self.someSettingsLabel)
        self.buttonBox = QtWidgets.QDialogButtonBox(parent=Preferences)
        self.buttonBox.setOrientation(QtCore.Qt.Orientation.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.StandardButton.Close|QtWidgets.QDialogButtonBox.StandardButton.Help)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout_2.addWidget(self.buttonBox)
        self.label_9.setBuddy(self.lang)
        self.video_driver_label.setBuddy(self.video_driver)
        self.styleLabel.setBuddy(self.styleComboBox)
        self.themeLabel.setBuddy(self.theme)
        self.uiSizeLabel.setBuddy(self.uiScale)
        self.label_30.setBuddy(self.timeLimit)
        self.label_24.setBuddy(self.lrnCutoff)
        self.label_23.setBuddy(self.dayOffset)
        self.deck_label.setBuddy(self.useCurrent)
        self.search_text_label.setBuddy(self.default_search_text)
        self.label_2.setBuddy(self.network_timeout)
        self.label.setBuddy(self.custom_sync_url)
        self.label_3.setBuddy(self.daily_backups)
        self.label_6.setBuddy(self.monthly_backups)
        self.label_5.setBuddy(self.weekly_backups)
        self.label_7.setBuddy(self.minutes_between_backups)

        self.retranslateUi(Preferences)
        self.tabWidget.setCurrentIndex(0)
        self.buttonBox.accepted.connect(Preferences.accept) # type: ignore  # type: ignore
        self.buttonBox.rejected.connect(Preferences.reject) # type: ignore  # type: ignore
        QtCore.QMetaObject.connectSlotsByName(Preferences)
        Preferences.setTabOrder(self.lang, self.video_driver)
        Preferences.setTabOrder(self.video_driver, self.check_for_updates)
        Preferences.setTabOrder(self.check_for_updates, self.theme)
        Preferences.setTabOrder(self.theme, self.styleComboBox)
        Preferences.setTabOrder(self.styleComboBox, self.uiScale)
        Preferences.setTabOrder(self.uiScale, self.resetWindowSizes)
        Preferences.setTabOrder(self.resetWindowSizes, self.hide_top_bar)
        Preferences.setTabOrder(self.hide_top_bar, self.topBarComboBox)
        Preferences.setTabOrder(self.topBarComboBox, self.hide_bottom_bar)
        Preferences.setTabOrder(self.hide_bottom_bar, self.bottomBarComboBox)
        Preferences.setTabOrder(self.bottomBarComboBox, self.reduce_motion)
        Preferences.setTabOrder(self.reduce_motion, self.minimalist_mode)
        Preferences.setTabOrder(self.minimalist_mode, self.dayOffset)
        Preferences.setTabOrder(self.dayOffset, self.lrnCutoff)
        Preferences.setTabOrder(self.lrnCutoff, self.timeLimit)
        Preferences.setTabOrder(self.timeLimit, self.showPlayButtons)
        Preferences.setTabOrder(self.showPlayButtons, self.interrupt_audio)
        Preferences.setTabOrder(self.interrupt_audio, self.showProgress)
        Preferences.setTabOrder(self.showProgress, self.showEstimates)
        Preferences.setTabOrder(self.showEstimates, self.spacebar_rates_card)
        Preferences.setTabOrder(self.spacebar_rates_card, self.render_latex)
        Preferences.setTabOrder(self.render_latex, self.url_schemes)
        Preferences.setTabOrder(self.url_schemes, self.pastePNG)
        Preferences.setTabOrder(self.pastePNG, self.paste_strips_formatting)
        Preferences.setTabOrder(self.paste_strips_formatting, self.useCurrent)
        Preferences.setTabOrder(self.useCurrent, self.default_search_text)
        Preferences.setTabOrder(self.default_search_text, self.ignore_accents_in_search)
        Preferences.setTabOrder(self.ignore_accents_in_search, self.syncMedia)
        Preferences.setTabOrder(self.syncMedia, self.syncOnProgramOpen)
        Preferences.setTabOrder(self.syncOnProgramOpen, self.autoSyncMedia)
        Preferences.setTabOrder(self.autoSyncMedia, self.fullSync)
        Preferences.setTabOrder(self.fullSync, self.network_timeout)
        Preferences.setTabOrder(self.network_timeout, self.media_log)
        Preferences.setTabOrder(self.media_log, self.syncLogout)
        Preferences.setTabOrder(self.syncLogout, self.syncLogin)
        Preferences.setTabOrder(self.syncLogin, self.custom_sync_url)
        Preferences.setTabOrder(self.custom_sync_url, self.minutes_between_backups)
        Preferences.setTabOrder(self.minutes_between_backups, self.daily_backups)
        Preferences.setTabOrder(self.daily_backups, self.weekly_backups)
        Preferences.setTabOrder(self.weekly_backups, self.monthly_backups)
        Preferences.setTabOrder(self.monthly_backups, self.syncAnkiHubLogout)
        Preferences.setTabOrder(self.syncAnkiHubLogout, self.syncAnkiHubLogin)
        Preferences.setTabOrder(self.syncAnkiHubLogin, self.buttonBox)
        Preferences.setTabOrder(self.buttonBox, self.tabWidget)

    def retranslateUi(self, Preferences):
        _translate = QtCore.QCoreApplication.translate
        Preferences.setWindowTitle(tr.preferences_preferences())
        self.generalGroup.setTitle(tr.preferences_general())
        self.label_9.setText(tr.preferences_language())
        self.video_driver_label.setText(tr.preferences_video_driver())
        self.check_for_updates.setText(tr.preferences_check_for_updates())
        self.uiGroup.setTitle(tr.preferences_user_interface())
        self.styleLabel.setText(tr.preferences_style())
        self.themeLabel.setText(tr.preferences_theme())
        self.uiSizeLabel.setText(tr.preferences_user_interface_size())
        self.resetWindowSizes.setText(tr.preferences_reset_window_sizes())
        self.distractionsGroup.setTitle(tr.preferences_distractions())
        self.reduce_motion.setToolTip(tr.preferences_reduce_motion_tooltip())
        self.reduce_motion.setText(tr.preferences_reduce_motion())
        self.hide_bottom_bar.setText(tr.preferences_hide_bottom_bar_during_review())
        self.hide_top_bar.setText(tr.preferences_hide_top_bar_during_review())
        self.minimalist_mode.setToolTip(tr.preferences_minimalist_mode_tooltip())
        self.minimalist_mode.setText(tr.preferences_minimalist_mode())
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.appearanceTab), tr.preferences_appearance())
        self.schedulerGroup.setTitle(tr.preferences_scheduler())
        self.lrnCutoff.setSuffix(tr.preferences_mins())
        self.label_30.setText(tr.preferences_timebox_time_limit())
        self.dayOffset.setSuffix(tr.preferences_hours_past_midnight())
        self.label_24.setText(tr.preferences_learn_ahead_limit())
        self.label_23.setText(tr.preferences_next_day_starts_at())
        self.timeLimit.setSuffix(tr.preferences_mins())
        self.reviewerGroup.setTitle(tr.preferences_review())
        self.showPlayButtons.setText(tr.preferences_show_play_buttons_on_cards_with())
        self.interrupt_audio.setText(tr.preferences_interrupt_current_audio_when_answering())
        self.showProgress.setText(tr.preferences_show_remaining_card_count())
        self.showEstimates.setText(tr.preferences_show_next_review_time_above_answer())
        self.spacebar_rates_card.setText(tr.preferences_spacebar_rates_card())
        self.render_latex.setText(tr.preferences_generate_latex_images_automatically())
        self.url_schemes.setText(tr.preferences_url_schemes())
        self.preferences_answer_keys.setTitle(tr.preferences_answer_keys())
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_3), tr.preferences_review())
        self.editorGroupBox.setTitle(tr.preferences_editing())
        self.pastePNG.setText(tr.preferences_paste_clipboard_images_as_png())
        self.paste_strips_formatting.setText(tr.preferences_paste_without_shift_key_strips_formatting())
        self.deck_label.setText(tr.preferences_default_deck())
        self.useCurrent.setItemText(0, tr.preferences_when_adding_default_to_current_deck())
        self.useCurrent.setItemText(1, tr.preferences_change_deck_depending_on_note_type())
        self.browserGroupBox.setTitle(tr.preferences_browsing())
        self.search_text_label.setText(tr.preferences_default_search_text())
        self.default_search_text.setPlaceholderText(tr.preferences_default_search_text_example())
        self.ignore_accents_in_search.setText(tr.preferences_ignore_accents_in_search())
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_1), tr.preferences_editing())
        self.groupBox_5.setTitle(tr.preferences_tab_synchronisation())
        self.syncMedia.setText(tr.preferences_synchronize_audio_and_images_too())
        self.syncOnProgramOpen.setText(tr.preferences_automatically_sync_on_profile_openclose())
        self.autoSyncMedia.setText(tr.preferences_periodically_sync_media())
        self.fullSync.setText(tr.preferences_on_next_sync_force_changes_in())
        self.label_2.setText(tr.preferences_network_timeout())
        self.network_timeout.setSuffix(tr.scheduling_seconds())
        self.groupBox_6.setTitle(tr.preferences_account())
        self.syncLogout.setText(tr.sync_log_out_button())
        self.syncLogin.setText(tr.sync_log_in_button())
        self.custom_sync_url.setPlaceholderText(tr.preferences_custom_sync_url_disclaimer())
        self.label.setText(tr.preferences_custom_sync_url())
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_2), tr.preferences_network())
        self.groupBox_8.setTitle(tr.preferences_backups())
        self.backup_explanation.setText(tr.preferences_backup_explanation())
        self.label_3.setText(tr.preferences_daily_backups())
        self.label_6.setText(tr.preferences_monthly_backups())
        self.label_5.setText(tr.preferences_weekly_backups())
        self.label_7.setText(tr.preferences_minutes_between_backups())
        self.openBackupFolder.setText(tr.preferences_you_can_restore_backups_via_fileswitch())
        self.groupBox_9.setTitle(tr.preferences_note())
        self.label_4.setText(tr.preferences_media_is_not_backed_up())
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab), tr.preferences_backups())
        self.label1.setText(tr.preferences_third_party_description())
        self.syncAnkiHubLogout.setText(tr.sync_log_out_button())
        self.syncAnkiHubLogin.setText(tr.sync_log_in_button())
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_31), tr.preferences_third_party_services())
        self.someSettingsLabel.setText(tr.preferences_some_settings_will_take_effect_after())