import{_ as n}from"./FVFIwcTn.mjs";async function i(e,s){let r=!1;async function t(){const a=await n({});s(a),!r&&setTimeout(t,100)}setTimeout(t,100);try{return await e()}finally{r=!0}}export{i as r};
