"use strict";$(init);function init(){$("tr.deck").draggable({scroll:!1,helper:function(e){return $(this).clone(!1)},delay:200,opacity:.7}),$("tr.deck").droppable({drop:handleDropEvent,hoverClass:"drag-hover"}),$("tr.top-level-drag-row").droppable({drop:handleDropEvent,hoverClass:"drag-hover"})}function handleDropEvent(e,r){const t=r.draggable.attr("id"),d=$(this).attr("id")||"";pycmd("drag:"+t+","+d)}
