from typing import Union

class Backend:
    @classmethod
    def command(cls, service: int, method: int, data: bytes) -> bytes: ...
    def db_command(self, data: bytes) -> bytes: ...

def buildhash() -> str: ...
def open_backend(data: bytes) -> Backend: ...
def initialize_logging(log_file: Union[str, None]) -> Backend: ...
def syncserver() -> None: ...
