# Copyright: Ankitects Pty Ltd and contributors
# License: GNU AGPL, version 3 or later; http://www.gnu.org/licenses/agpl.html

# This file was automatically generated from _root-vars.scss
FG = {
    "comment": "Default text/icon color",
    "light": "#020202",
    "dark": "#fcfcfc",
}
FG_SUBTLE = {
    "comment": "Placeholder text, icons in idle state",
    "light": "#737373",
    "dark": "#858585",
}
FG_DISABLED = {
    "comment": "Foreground color of disabled UI elements",
    "light": "#858585",
    "dark": "#737373",
}
FG_FAINT = {
    "comment": "Foreground color that barely stands out against canvas",
    "light": "#afafaf",
    "dark": "#545454",
}
FG_LINK = {
    "comment": "Hyperlink foreground color",
    "light": "#1d4ed8",
    "dark": "#bfdbfe",
}
CANVAS = {
    "comment": "Window background",
    "light": "#f5f5f5",
    "dark": "#2c2c2c",
}
CANVAS_ELEVATED = {
    "comment": "Background of containers",
    "light": "white",
    "dark": "#363636",
}
CANVAS_INSET = {
    "comment": "Background of inputs inside containers",
    "light": "white",
    "dark": "#2c2c2c",
}
CANVAS_OVERLAY = {
    "comment": "Background of floating elements (menus, tooltips)",
    "light": "#fcfcfc",
    "dark": "#2c2c2c",
}
CANVAS_CODE = {
    "comment": "Background of code editors",
    "light": "white",
    "dark": "#252525",
}
CANVAS_GLASS = {
    "comment": "Transparent background for surfaces containing text",
    "light": "rgba(255, 255, 255, 0.4)",
    "dark": "rgba(54, 54, 54, 0.4)",
}
BORDER = {
    "comment": "Border color with medium contrast against window background",
    "light": "#c4c4c4",
    "dark": "#202020",
}
BORDER_SUBTLE = {
    "comment": "Border color with low contrast against window background",
    "light": "#e4e4e4",
    "dark": "#252525",
}
BORDER_STRONG = {
    "comment": "Border color with high contrast against window background",
    "light": "#858585",
    "dark": "#020202",
}
BORDER_FOCUS = {
    "comment": "Border color of focused input elements",
    "light": "#3b82f6",
    "dark": "#3b82f6",
}
BUTTON_BG = {
    "comment": "Background color of buttons",
    "light": "#fcfcfc",
    "dark": "#404040",
}
BUTTON_GRADIENT_START = {
    "comment": "Start value of default button gradient",
    "light": "white",
    "dark": "#4a4a4a",
}
BUTTON_GRADIENT_END = {
    "comment": "End value of default button gradient",
    "light": "#fcfcfc",
    "dark": "#404040",
}
BUTTON_HOVER_BORDER = {
    "comment": "Border color of default button in hover state",
    "light": "#999999",
    "dark": "#141414",
}
BUTTON_DISABLED = {
    "comment": "Background color of disabled button",
    "light": "rgba(214, 214, 214, 0.5)",
    "dark": "rgba(69, 69, 69, 0.5)",
}
BUTTON_PRIMARY_BG = {
    "comment": "Background color of primary button",
    "light": "#306bec",
    "dark": "#2652cf",
}
BUTTON_PRIMARY_GRADIENT_START = {
    "comment": "Start value of primary button gradient",
    "light": "#3b82f6",
    "dark": "#2f67e1",
}
BUTTON_PRIMARY_GRADIENT_END = {
    "comment": "End value of primary button gradient",
    "light": "#306bec",
    "dark": "#2652cf",
}
BUTTON_PRIMARY_DISABLED = {
    "comment": "Background color of primary button in disabled state",
    "light": "#93c5fd",
    "dark": "#4484ed",
}
SCROLLBAR_BG = {
    "comment": "Background of scrollbar in idle state (Win/Lin only)",
    "light": "#d6d6d6",
    "dark": "#454545",
}
SCROLLBAR_BG_HOVER = {
    "comment": "Background of scrollbar in hover state (Win/Lin only)",
    "light": "#c4c4c4",
    "dark": "#545454",
}
SCROLLBAR_BG_ACTIVE = {
    "comment": "Background of scrollbar in pressed state (Win/Lin only)",
    "light": "#afafaf",
    "dark": "#636363",
}
SHADOW = {
    "comment": "Default box-shadow color",
    "light": "#c4c4c4",
    "dark": "#141414",
}
SHADOW_INSET = {
    "comment": "Inset box-shadow color",
    "light": "#454545",
    "dark": "#202020",
}
SHADOW_SUBTLE = {
    "comment": "Box-shadow color with lower contrast against window background",
    "light": "#737373",
    "dark": "#363636",
}
SHADOW_FOCUS = {
    "comment": "Box-shadow color for elements in focused state",
    "light": "#6366f1",
    "dark": "#6366f1",
}
ACCENT_CARD = {
    "comment": "Accent color for cards",
    "light": "#60a5fa",
    "dark": "#93c5fd",
}
ACCENT_NOTE = {
    "comment": "Accent color for notes",
    "light": "#22c55e",
    "dark": "#4ade80",
}
ACCENT_DANGER = {
    "comment": "Saturated accent color to grab attention",
    "light": "#ef4444",
    "dark": "#f87171",
}
FLAG_1 = {
    "comment": "Flag 1 (red)",
    "light": "#ef4444",
    "dark": "#f87171",
}
FLAG_2 = {
    "comment": "Flag 2 (orange)",
    "light": "#fb923c",
    "dark": "#fdba74",
}
FLAG_3 = {
    "comment": "Flag 3 (green)",
    "light": "#4ade80",
    "dark": "#86efac",
}
FLAG_4 = {
    "comment": "Flag 4 (blue)",
    "light": "#3b82f6",
    "dark": "#60a5fa",
}
FLAG_5 = {
    "comment": "Flag 5 (pink)",
    "light": "#e879f9",
    "dark": "#f0abfc",
}
FLAG_6 = {
    "comment": "Flag 6 (turquoise)",
    "light": "#2dd4bf",
    "dark": "#5eead4",
}
FLAG_7 = {
    "comment": "Flag 7 (purple)",
    "light": "#a855f7",
    "dark": "#c084fc",
}
STATE_NEW = {
    "comment": "Accent color for new cards",
    "light": "#3b82f6",
    "dark": "#93c5fd",
}
STATE_LEARN = {
    "comment": "Accent color for cards in learning state",
    "light": "#dc2626",
    "dark": "#f87171",
}
STATE_REVIEW = {
    "comment": "Accent color for cards in review state",
    "light": "#16a34a",
    "dark": "#22c55e",
}
STATE_BURIED = {
    "comment": "Accent color for buried cards",
    "light": "#f59e0b",
    "dark": "#92400e",
}
STATE_SUSPENDED = {
    "comment": "Accent color for suspended cards",
    "light": "#facc15",
    "dark": "#fef9c3",
}
STATE_MARKED = {
    "comment": "Accent color for marked cards",
    "light": "#6366f1",
    "dark": "#a855f7",
}
HIGHLIGHT_BG = {
    "comment": "Background color of highlighted items",
    "light": "rgba(37, 99, 235, 0.5)",
    "dark": "rgba(147, 197, 253, 0.5)",
}
HIGHLIGHT_FG = {
    "comment": "Foreground color of highlighted items",
    "light": "black",
    "dark": "white",
}
SELECTED_BG = {
    "comment": "Background color of selected text",
    "light": "rgba(214, 214, 214, 0.5)",
    "dark": "rgba(147, 197, 253, 0.5)",
}
SELECTED_FG = {
    "comment": "Foreground color of selected text",
    "light": "black",
    "dark": "white",
}
