# Form implementation generated from reading ui file 'qt/aqt/forms/profiles.ui'
#
# Created by: PyQt6 UI code generator 6.11.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets
from aqt.utils import tr



class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(423, 356)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap("icons:anki.png"), QtGui.QIcon.Mode.Normal, QtGui.QIcon.State.Off)
        MainWindow.setWindowIcon(icon)
        self.centralwidget = QtWidgets.QWidget(parent=MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.profiles = QtWidgets.QListWidget(parent=self.centralwidget)
        self.profiles.setObjectName("profiles")
        self.verticalLayout_2.addWidget(self.profiles)
        self.horizontalLayout.addLayout(self.verticalLayout_2)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.login = QtWidgets.QPushButton(parent=self.centralwidget)
        self.login.setDefault(True)
        self.login.setObjectName("login")
        self.verticalLayout.addWidget(self.login)
        self.add = QtWidgets.QPushButton(parent=self.centralwidget)
        self.add.setObjectName("add")
        self.verticalLayout.addWidget(self.add)
        self.rename = QtWidgets.QPushButton(parent=self.centralwidget)
        self.rename.setObjectName("rename")
        self.verticalLayout.addWidget(self.rename)
        self.delete_2 = QtWidgets.QPushButton(parent=self.centralwidget)
        self.delete_2.setObjectName("delete_2")
        self.verticalLayout.addWidget(self.delete_2)
        self.quit = QtWidgets.QPushButton(parent=self.centralwidget)
        self.quit.setObjectName("quit")
        self.verticalLayout.addWidget(self.quit)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.openBackup = QtWidgets.QPushButton(parent=self.centralwidget)
        self.openBackup.setObjectName("openBackup")
        self.verticalLayout.addWidget(self.openBackup)
        self.downgrade_button = QtWidgets.QPushButton(parent=self.centralwidget)
        self.downgrade_button.setText("DOWNGRADE")
        self.downgrade_button.setObjectName("downgrade_button")
        self.verticalLayout.addWidget(self.downgrade_button)
        self.horizontalLayout.addLayout(self.verticalLayout)
        self.verticalLayout_3.addLayout(self.horizontalLayout)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(parent=MainWindow)
        self.menubar.setEnabled(False)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 423, 22))
        self.menubar.setObjectName("menubar")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(parent=MainWindow)
        self.statusbar.setEnabled(False)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(tr.profiles_profiles())
        self.login.setText(tr.profiles_open())
        self.add.setText(tr.actions_add())
        self.rename.setText(tr.actions_rename())
        self.delete_2.setText(tr.actions_delete())
        self.quit.setText(tr.profiles_quit())
        self.openBackup.setText(tr.profiles_open_backup())