import"../chunks/E-vkuI0X.mjs";import{i as o}from"../chunks/8UuMqzDP.mjs";import{bg as p,b0 as m,b1 as f,ba as b,bQ as c,bP as i,W as _,bd as n,bu as g,aw as l,b8 as u}from"../chunks/W1VwgJDT.mjs";import{s as x}from"../chunks/CCT7NDl_.mjs";import{p as s}from"../chunks/DDoOjL8A.mjs";function P(t,r){p(r,!1);const e=u();m(()=>s,()=>{g(e,s.error.message)}),f(),o(),b();var a=c();i(()=>x(a,l(e))),_(t,a),n()}export{P as component};
