import"./E-vkuI0X.mjs";import"./8UuMqzDP.mjs";import{a3 as I,a5 as M,aq as S,W as r,bo as X,bP as x,au as D}from"./W1VwgJDT.mjs";import{g as s}from"./CCT7NDl_.mjs";import{a7 as P}from"./FVFIwcTn.mjs";import{fD as l,fE as k}from"./ceRbsEBD.mjs";import{p as q}from"./DhwOeGsE.mjs";import{a as B,e as f}from"./D78duh1q.mjs";import{p as a}from"./D-j9Rl0W.mjs";var C=D("<button><!></button>");function L(m,t){let d=a(t,"id",8,void 0),c=a(t,"class",8,""),u=a(t,"tooltip",8,void 0),b=a(t,"primary",8,!1),v=a(t,"active",8,!1),h=a(t,"disabled",8,!1),_=a(t,"tabbable",8,!1),g=a(t,"iconSize",8,75),p=a(t,"widthMultiplier",8,1),w=a(t,"flipX",8,!1);var e=C();let o;var y=I(e);B(y,{get flipX(){return w()},get widthMultiplier(){return p()},get iconSize(){return g()},children:(i,E)=>{var n=M(),z=S(n);P(z,t,"default",{}),r(i,n)},$$slots:{default:!0}}),X(e),x(()=>{var i;l(e,"id",d()),o=k(e,1,`icon-button ${(i=c())!=null?i:""}`,"svelte-9egv84",o,{active:v(),primary:b()}),l(e,"title",u()),e.disabled=h(),l(e,"tabindex",_()?0:-1)}),s("click",e,function(i){f.call(this,t,i)}),s("mousedown",e,q(function(i){f.call(this,t,i)})),r(m,e)}export{L as I};
