.outer.svelte-15d6yec{overflow:auto;margin-bottom:.5rem}.preview.svelte-15d6yec{border-collapse:collapse;white-space:nowrap}.preview.svelte-15d6yec th:where(.svelte-15d6yec),.preview.svelte-15d6yec td:where(.svelte-15d6yec){text-overflow:ellipsis;overflow:hidden;border:1px solid var(--border-subtle);padding:.25rem .5rem;max-width:15em}.preview.svelte-15d6yec th:where(.svelte-15d6yec){background:var(--border);text-align:center}.preview.svelte-15d6yec tr:where(.svelte-15d6yec):nth-child(2n){background:var(--canvas)}.preview.svelte-15d6yec td:where(.svelte-15d6yec){text-align:start}
