.congrats.svelte-bnvdv9{margin-top:2em;max-width:30em;font-size:var(--font-size)}.congrats.svelte-bnvdv9 a{color:var(--fg-link);text-decoration:none}.description.svelte-bnvdv9{border:1px solid var(--border);padding:1em}
