import{q as s}from"../chunks/FVFIwcTn.mjs";import"../chunks/E-vkuI0X.mjs";import{i}from"../chunks/8UuMqzDP.mjs";import{bg as l,bd as c,ad as p,bU as u}from"../chunks/W1VwgJDT.mjs";import{p as d}from"../chunks/D-j9Rl0W.mjs";import{p as f}from"../chunks/DDoOjL8A.mjs";import{C as g}from"../chunks/BHRo00-9.mjs";import{g as m}from"../chunks/CMI_btIM.mjs";function h(o){try{return BigInt(o)}catch{return null}}const b=(async({params:o})=>{const t=h(o.cardId);return{info:t!==null?await s({cid:t}):null}}),T=Object.freeze(Object.defineProperty({__proto__:null,load:b},Symbol.toStringTag,{value:"Module"}));function j(o,t){l(t,!1);let a=d(t,"data",8);const n=f.url.searchParams.get("revlog")!=="0";globalThis.anki||(globalThis.anki={}),globalThis.anki.updateCard=async e=>{const r=`/card-info/${e}`;return m(r).catch(()=>{window.location.href=r})},i(),g(o,{get stats(){return p(a()),u(()=>a().info)},get showRevlog(){return n}}),c()}export{j as component,T as universal};
