import"./E-vkuI0X.mjs";import{i as h}from"./8UuMqzDP.mjs";import{bg as w,a5 as x,aq as W,W as e,bd as k,a3 as N,bo as P,bP as R,ad as m,bU as U,au as q}from"./W1VwgJDT.mjs";import{s as E}from"./CCT7NDl_.mjs";import{p as n,i as I}from"./D-j9Rl0W.mjs";import{t as f,s as l}from"./DiHRty_X.mjs";import{fE as $}from"./ceRbsEBD.mjs";import{f as p}from"./DTrAx_2Q.mjs";import{R as j}from"./CD0nhXdt.mjs";var y=q("<div> </div>");function L(d,t){w(t,!1);let r=n(t,"warning",8),c=n(t,"className",8,"alert-warning");h();var o=x(),v=W(o);{var _=s=>{j(s,{children:(u,z)=>{var a=y(),b=N(a,!0);P(a),R(g=>{var i;$(a,1,`col-12 alert ${(i=c())!=null?i:""} mb-0`),E(b,g)},[()=>(m(p),m(r()),U(()=>p(r())))]),f(1,a,()=>l),f(2,a,()=>l),e(u,a)},$$slots:{default:!0}})};I(v,s=>{r()&&s(_)})}e(d,o),k()}export{L as W};
