import{s}from"../chunks/DTrAx_2Q.mjs";import"../chunks/E-vkuI0X.mjs";import{i as m}from"../chunks/8UuMqzDP.mjs";import{bg as r,bv as a,a5 as c,aq as i,W as l,bd as d}from"../chunks/W1VwgJDT.mjs";import{a7 as u}from"../chunks/FVFIwcTn.mjs";import{m as f,t as p}from"../chunks/Dbye-yJ_.mjs";function h(){const t=window.location.hash=="#night";return t&&(document.documentElement.className="night-mode",document.documentElement.dataset.bsTheme="dark"),t}const b=!1,g=!1,y=async()=>{h(),await s()},k=Object.freeze(Object.defineProperty({__proto__:null,load:y,prerender:g,ssr:b},Symbol.toStringTag,{value:"Module"}));function x(t,e){r(e,!1),a(f,new Map),a(p,"ontouchstart"in document.documentElement),m();var o=c(),n=i(o);u(n,e,"default",{}),l(t,o),d()}export{x as component,k as universal};
