# Form implementation generated from reading ui file 'qt/aqt/forms/main.ui'
#
# Created by: PyQt6 UI code generator 6.11.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets
from aqt.utils import tr



class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(667, 570)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(MainWindow.sizePolicy().hasHeightForWidth())
        MainWindow.setSizePolicy(sizePolicy)
        MainWindow.setMinimumSize(QtCore.QSize(400, 0))
        MainWindow.setWindowTitle("Anki")
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap("icons:anki.png"), QtGui.QIcon.Mode.Normal, QtGui.QIcon.State.Off)
        MainWindow.setWindowIcon(icon)
        self.centralwidget = QtWidgets.QWidget(parent=MainWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Expanding)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(1)
        sizePolicy.setHeightForWidth(self.centralwidget.sizePolicy().hasHeightForWidth())
        self.centralwidget.setSizePolicy(sizePolicy)
        self.centralwidget.setAutoFillBackground(True)
        self.centralwidget.setObjectName("centralwidget")
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(parent=MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 667, 43))
        self.menubar.setObjectName("menubar")
        self.menuHelp = QtWidgets.QMenu(parent=self.menubar)
        self.menuHelp.setObjectName("menuHelp")
        self.menuEdit = QtWidgets.QMenu(parent=self.menubar)
        self.menuEdit.setObjectName("menuEdit")
        self.menuCol = QtWidgets.QMenu(parent=self.menubar)
        self.menuCol.setObjectName("menuCol")
        self.menuTools = QtWidgets.QMenu(parent=self.menubar)
        self.menuTools.setObjectName("menuTools")
        self.menuqt_accel_view = QtWidgets.QMenu(parent=self.menubar)
        self.menuqt_accel_view.setObjectName("menuqt_accel_view")
        MainWindow.setMenuBar(self.menubar)
        self.actionExit = QtGui.QAction(parent=MainWindow)
        self.actionExit.setShortcut("Ctrl+Q")
        self.actionExit.setObjectName("actionExit")
        self.actionPreferences = QtGui.QAction(parent=MainWindow)
        self.actionPreferences.setShortcut("Ctrl+P")
        self.actionPreferences.setMenuRole(QtGui.QAction.MenuRole.PreferencesRole)
        self.actionPreferences.setObjectName("actionPreferences")
        self.actionAbout = QtGui.QAction(parent=MainWindow)
        self.actionAbout.setMenuRole(QtGui.QAction.MenuRole.ApplicationSpecificRole)
        self.actionAbout.setObjectName("actionAbout")
        self.actionUndo = QtGui.QAction(parent=MainWindow)
        self.actionUndo.setEnabled(False)
        self.actionUndo.setShortcut("Ctrl+Z")
        self.actionUndo.setObjectName("actionUndo")
        self.actionCheckMediaDatabase = QtGui.QAction(parent=MainWindow)
        self.actionCheckMediaDatabase.setObjectName("actionCheckMediaDatabase")
        self.actionDonate = QtGui.QAction(parent=MainWindow)
        self.actionDonate.setObjectName("actionDonate")
        self.actionFullDatabaseCheck = QtGui.QAction(parent=MainWindow)
        self.actionFullDatabaseCheck.setObjectName("actionFullDatabaseCheck")
        self.actionDocumentation = QtGui.QAction(parent=MainWindow)
        self.actionDocumentation.setShortcut("F1")
        self.actionDocumentation.setObjectName("actionDocumentation")
        self.actionSwitchProfile = QtGui.QAction(parent=MainWindow)
        self.actionSwitchProfile.setShortcut("Ctrl+Shift+P")
        self.actionSwitchProfile.setObjectName("actionSwitchProfile")
        self.actionExport = QtGui.QAction(parent=MainWindow)
        self.actionExport.setShortcut("Ctrl+E")
        self.actionExport.setObjectName("actionExport")
        self.actionImport = QtGui.QAction(parent=MainWindow)
        self.actionImport.setShortcut("Ctrl+Shift+I")
        self.actionImport.setObjectName("actionImport")
        self.actionStudyDeck = QtGui.QAction(parent=MainWindow)
        self.actionStudyDeck.setShortcut("/")
        self.actionStudyDeck.setObjectName("actionStudyDeck")
        self.actionEmptyCards = QtGui.QAction(parent=MainWindow)
        self.actionEmptyCards.setObjectName("actionEmptyCards")
        self.actionCreateFiltered = QtGui.QAction(parent=MainWindow)
        self.actionCreateFiltered.setShortcut("F")
        self.actionCreateFiltered.setObjectName("actionCreateFiltered")
        self.actionNoteTypes = QtGui.QAction(parent=MainWindow)
        self.actionNoteTypes.setShortcut("Ctrl+Shift+N")
        self.actionNoteTypes.setObjectName("actionNoteTypes")
        self.actionAdd_ons = QtGui.QAction(parent=MainWindow)
        self.actionAdd_ons.setShortcut("Ctrl+Shift+A")
        self.actionAdd_ons.setObjectName("actionAdd_ons")
        self.actionRedo = QtGui.QAction(parent=MainWindow)
        self.actionRedo.setEnabled(False)
        self.actionRedo.setShortcut("Ctrl+Shift+Z")
        self.actionRedo.setObjectName("actionRedo")
        self.actionFullScreen = QtGui.QAction(parent=MainWindow)
        self.actionFullScreen.setObjectName("actionFullScreen")
        self.actionZoomIn = QtGui.QAction(parent=MainWindow)
        self.actionZoomIn.setObjectName("actionZoomIn")
        self.actionZoomOut = QtGui.QAction(parent=MainWindow)
        self.actionZoomOut.setObjectName("actionZoomOut")
        self.actionResetZoom = QtGui.QAction(parent=MainWindow)
        self.actionResetZoom.setShortcut("Ctrl+0")
        self.actionResetZoom.setObjectName("actionResetZoom")
        self.action_create_backup = QtGui.QAction(parent=MainWindow)
        self.action_create_backup.setObjectName("action_create_backup")
        self.action_open_backup = QtGui.QAction(parent=MainWindow)
        self.action_open_backup.setObjectName("action_open_backup")
        self.action_upgrade_downgrade = QtGui.QAction(parent=MainWindow)
        self.action_upgrade_downgrade.setObjectName("action_upgrade_downgrade")
        self.menuHelp.addAction(self.actionDocumentation)
        self.menuHelp.addSeparator()
        self.menuHelp.addAction(self.actionDonate)
        self.menuHelp.addAction(self.actionAbout)
        self.menuEdit.addAction(self.actionUndo)
        self.menuEdit.addAction(self.actionRedo)
        self.menuCol.addAction(self.actionSwitchProfile)
        self.menuCol.addSeparator()
        self.menuCol.addAction(self.actionImport)
        self.menuCol.addAction(self.actionExport)
        self.menuCol.addSeparator()
        self.menuCol.addAction(self.action_create_backup)
        self.menuCol.addAction(self.action_open_backup)
        self.menuCol.addSeparator()
        self.menuCol.addAction(self.actionExit)
        self.menuTools.addAction(self.actionStudyDeck)
        self.menuTools.addAction(self.actionCreateFiltered)
        self.menuTools.addSeparator()
        self.menuTools.addAction(self.actionFullDatabaseCheck)
        self.menuTools.addAction(self.actionCheckMediaDatabase)
        self.menuTools.addAction(self.actionEmptyCards)
        self.menuTools.addSeparator()
        self.menuTools.addAction(self.actionAdd_ons)
        self.menuTools.addSeparator()
        self.menuTools.addAction(self.actionNoteTypes)
        self.menuTools.addAction(self.action_upgrade_downgrade)
        self.menuTools.addAction(self.actionPreferences)
        self.menuqt_accel_view.addAction(self.actionFullScreen)
        self.menuqt_accel_view.addSeparator()
        self.menuqt_accel_view.addAction(self.actionZoomIn)
        self.menuqt_accel_view.addAction(self.actionZoomOut)
        self.menuqt_accel_view.addAction(self.actionResetZoom)
        self.menubar.addAction(self.menuCol.menuAction())
        self.menubar.addAction(self.menuEdit.menuAction())
        self.menubar.addAction(self.menuqt_accel_view.menuAction())
        self.menubar.addAction(self.menuTools.menuAction())
        self.menubar.addAction(self.menuHelp.menuAction())

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        self.menuHelp.setTitle(tr.qt_accel_help())
        self.menuEdit.setTitle(tr.qt_accel_edit())
        self.menuCol.setTitle(tr.qt_accel_file())
        self.menuTools.setTitle(tr.qt_accel_tools())
        self.menuqt_accel_view.setTitle(tr.qt_accel_view())
        self.actionExit.setText(tr.qt_accel_exit())
        self.actionPreferences.setText(tr.qt_accel_preferences())
        self.actionPreferences.setStatusTip(tr.qt_misc_configure_interface_language_and_options())
        self.actionAbout.setText(tr.qt_accel_about())
        self.actionUndo.setText(tr.qt_accel_undo())
        self.actionCheckMediaDatabase.setText(tr.qt_accel_check_media())
        self.actionCheckMediaDatabase.setStatusTip(tr.qt_misc_check_the_files_in_the_media())
        self.actionDonate.setText(tr.qt_accel_support_anki())
        self.actionFullDatabaseCheck.setText(tr.qt_accel_check_database())
        self.actionDocumentation.setText(tr.qt_accel_guide())
        self.actionSwitchProfile.setText(tr.qt_accel_switch_profile())
        self.actionExport.setText(tr.qt_accel_export())
        self.actionImport.setText(tr.qt_accel_import())
        self.actionStudyDeck.setText(tr.qt_misc_study_deck())
        self.actionEmptyCards.setText(tr.qt_misc_empty_cards())
        self.actionCreateFiltered.setText(tr.qt_misc_create_filtered_deck())
        self.actionNoteTypes.setText(tr.qt_misc_manage_note_types())
        self.actionAdd_ons.setText(tr.qt_misc_addons())
        self.actionRedo.setText(tr.qt_accel_redo())
        self.actionFullScreen.setText(tr.qt_accel_full_screen())
        self.actionZoomIn.setText(tr.qt_accel_zoom_in())
        self.actionZoomOut.setText(tr.qt_accel_zoom_out())
        self.actionResetZoom.setText(tr.qt_accel_reset_zoom())
        self.action_create_backup.setText(tr.qt_accel_create_backup())
        self.action_open_backup.setText(tr.qt_accel_load_backup())
        self.action_upgrade_downgrade.setText(tr.qt_accel_upgrade_downgrade())