# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: anki/cards.proto
# Protobuf Python Version: 7.34.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    7,
    34,
    1,
    '',
    'anki/cards.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from anki import collection_pb2 as anki_dot_collection__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x10\x61nki/cards.proto\x12\nanki.cards\x1a\x15\x61nki/collection.proto\"\x15\n\x06\x43\x61rdId\x12\x0b\n\x03\x63id\x18\x01 \x01(\x03\"\x17\n\x07\x43\x61rdIds\x12\x0c\n\x04\x63ids\x18\x01 \x03(\x03\"\xd9\x04\n\x04\x43\x61rd\x12\n\n\x02id\x18\x01 \x01(\x03\x12\x0f\n\x07note_id\x18\x02 \x01(\x03\x12\x0f\n\x07\x64\x65\x63k_id\x18\x03 \x01(\x03\x12\x14\n\x0ctemplate_idx\x18\x04 \x01(\r\x12\x12\n\nmtime_secs\x18\x05 \x01(\x03\x12\x0b\n\x03usn\x18\x06 \x01(\x11\x12\r\n\x05\x63type\x18\x07 \x01(\r\x12\r\n\x05queue\x18\x08 \x01(\x11\x12\x0b\n\x03\x64ue\x18\t \x01(\x11\x12\x10\n\x08interval\x18\n \x01(\r\x12\x13\n\x0b\x65\x61se_factor\x18\x0b \x01(\r\x12\x0c\n\x04reps\x18\x0c \x01(\r\x12\x0e\n\x06lapses\x18\r \x01(\r\x12\x17\n\x0fremaining_steps\x18\x0e \x01(\r\x12\x14\n\x0coriginal_due\x18\x0f \x01(\x11\x12\x18\n\x10original_deck_id\x18\x10 \x01(\x03\x12\r\n\x05\x66lags\x18\x11 \x01(\r\x12\x1e\n\x11original_position\x18\x12 \x01(\rH\x00\x88\x01\x01\x12\x36\n\x0cmemory_state\x18\x14 \x01(\x0b\x32\x1b.anki.cards.FsrsMemoryStateH\x01\x88\x01\x01\x12\x1e\n\x11\x64\x65sired_retention\x18\x15 \x01(\x02H\x02\x88\x01\x01\x12\x12\n\x05\x64\x65\x63\x61y\x18\x16 \x01(\x02H\x03\x88\x01\x01\x12\"\n\x15last_review_time_secs\x18\x17 \x01(\x03H\x04\x88\x01\x01\x12\x13\n\x0b\x63ustom_data\x18\x13 \x01(\tB\x14\n\x12_original_positionB\x0f\n\r_memory_stateB\x14\n\x12_desired_retentionB\x08\n\x06_decayB\x18\n\x16_last_review_time_secs\"8\n\x0f\x46srsMemoryState\x12\x11\n\tstability\x18\x01 \x01(\x02\x12\x12\n\ndifficulty\x18\x02 \x01(\x02\"N\n\x12UpdateCardsRequest\x12\x1f\n\x05\x63\x61rds\x18\x01 \x03(\x0b\x32\x10.anki.cards.Card\x12\x17\n\x0fskip_undo_entry\x18\x02 \x01(\x08\"&\n\x12RemoveCardsRequest\x12\x10\n\x08\x63\x61rd_ids\x18\x01 \x03(\x03\"3\n\x0eSetDeckRequest\x12\x10\n\x08\x63\x61rd_ids\x18\x01 \x03(\x03\x12\x0f\n\x07\x64\x65\x63k_id\x18\x02 \x01(\x03\"0\n\x0eSetFlagRequest\x12\x10\n\x08\x63\x61rd_ids\x18\x01 \x03(\x03\x12\x0c\n\x04\x66lag\x18\x02 \x01(\r2\xf6\x02\n\x0c\x43\x61rdsService\x12/\n\x07GetCard\x12\x12.anki.cards.CardId\x1a\x10.anki.cards.Card\x12I\n\x0bUpdateCards\x12\x1e.anki.cards.UpdateCardsRequest\x1a\x1a.anki.collection.OpChanges\x12R\n\x0bRemoveCards\x12\x1e.anki.cards.RemoveCardsRequest\x1a#.anki.collection.OpChangesWithCount\x12J\n\x07SetDeck\x12\x1a.anki.cards.SetDeckRequest\x1a#.anki.collection.OpChangesWithCount\x12J\n\x07SetFlag\x12\x1a.anki.cards.SetFlagRequest\x1a#.anki.collection.OpChangesWithCount2\x15\n\x13\x42\x61\x63kendCardsServiceB\x02P\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'anki.cards_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'P\001'
  _globals['_CARDID']._serialized_start=55
  _globals['_CARDID']._serialized_end=76
  _globals['_CARDIDS']._serialized_start=78
  _globals['_CARDIDS']._serialized_end=101
  _globals['_CARD']._serialized_start=104
  _globals['_CARD']._serialized_end=705
  _globals['_FSRSMEMORYSTATE']._serialized_start=707
  _globals['_FSRSMEMORYSTATE']._serialized_end=763
  _globals['_UPDATECARDSREQUEST']._serialized_start=765
  _globals['_UPDATECARDSREQUEST']._serialized_end=843
  _globals['_REMOVECARDSREQUEST']._serialized_start=845
  _globals['_REMOVECARDSREQUEST']._serialized_end=883
  _globals['_SETDECKREQUEST']._serialized_start=885
  _globals['_SETDECKREQUEST']._serialized_end=936
  _globals['_SETFLAGREQUEST']._serialized_start=938
  _globals['_SETFLAGREQUEST']._serialized_end=986
  _globals['_CARDSSERVICE']._serialized_start=989
  _globals['_CARDSSERVICE']._serialized_end=1363
  _globals['_BACKENDCARDSSERVICE']._serialized_start=1365
  _globals['_BACKENDCARDSSERVICE']._serialized_end=1386
# @@protoc_insertion_point(module_scope)
