import"./E-vkuI0X.mjs";import"./8UuMqzDP.mjs";import{a3 as d,bo as c,bP as f,W as p,au as x}from"./W1VwgJDT.mjs";import{a7 as u}from"./FVFIwcTn.mjs";import{fD as v,fE as b}from"./ceRbsEBD.mjs";import{p as i}from"./D-j9Rl0W.mjs";var _=x("<div><!></div>");function D(r,e){let s=i(e,"id",8,void 0),l=i(e,"class",8,""),a=i(e,"breakpoint",8,"fluid");var t=_();let n;var m=d(t);u(m,e,"default",{}),c(t),f(()=>{var o;v(t,"id",s()),n=b(t,1,`container ${(o=l())!=null?o:""}`,"svelte-1qhanys",n,{"container-xs":a()==="xs","container-sm":a()==="sm","container-md":a()==="md","container-lg":a()==="lg","container-xl":a()==="xl","container-xxl":a()==="xxl","container-fluid":a()==="fluid"})}),p(r,t)}export{D as C};
