# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: anki/generic.proto
# Protobuf Python Version: 7.34.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    7,
    34,
    1,
    '',
    'anki/generic.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12\x61nki/generic.proto\x12\x0c\x61nki.generic\"\x07\n\x05\x45mpty\"\x14\n\x05Int32\x12\x0b\n\x03val\x18\x01 \x01(\x11\"\x15\n\x06UInt32\x12\x0b\n\x03val\x18\x01 \x01(\r\"\x14\n\x05Int64\x12\x0b\n\x03val\x18\x01 \x01(\x03\"\x15\n\x06String\x12\x0b\n\x03val\x18\x01 \x01(\t\"\x14\n\x04Json\x12\x0c\n\x04json\x18\x01 \x01(\x0c\"\x13\n\x04\x42ool\x12\x0b\n\x03val\x18\x01 \x01(\x08\"\x1a\n\nStringList\x12\x0c\n\x04vals\x18\x01 \x03(\tB\x02P\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'anki.generic_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'P\001'
  _globals['_EMPTY']._serialized_start=36
  _globals['_EMPTY']._serialized_end=43
  _globals['_INT32']._serialized_start=45
  _globals['_INT32']._serialized_end=65
  _globals['_UINT32']._serialized_start=67
  _globals['_UINT32']._serialized_end=88
  _globals['_INT64']._serialized_start=90
  _globals['_INT64']._serialized_end=110
  _globals['_STRING']._serialized_start=112
  _globals['_STRING']._serialized_end=133
  _globals['_JSON']._serialized_start=135
  _globals['_JSON']._serialized_end=155
  _globals['_BOOL']._serialized_start=157
  _globals['_BOOL']._serialized_end=176
  _globals['_STRINGLIST']._serialized_start=178
  _globals['_STRINGLIST']._serialized_end=204
# @@protoc_insertion_point(module_scope)
