# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: anki/notes.proto
# Protobuf Python Version: 7.34.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    7,
    34,
    1,
    '',
    'anki/notes.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from anki import notetypes_pb2 as anki_dot_notetypes__pb2
from anki import collection_pb2 as anki_dot_collection__pb2
from anki import decks_pb2 as anki_dot_decks__pb2
from anki import cards_pb2 as anki_dot_cards__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x10\x61nki/notes.proto\x12\nanki.notes\x1a\x14\x61nki/notetypes.proto\x1a\x15\x61nki/collection.proto\x1a\x10\x61nki/decks.proto\x1a\x10\x61nki/cards.proto\"\x15\n\x06NoteId\x12\x0b\n\x03nid\x18\x01 \x01(\x03\"\x1b\n\x07NoteIds\x12\x10\n\x08note_ids\x18\x01 \x03(\x03\"t\n\x04Note\x12\n\n\x02id\x18\x01 \x01(\x03\x12\x0c\n\x04guid\x18\x02 \x01(\t\x12\x13\n\x0bnotetype_id\x18\x03 \x01(\x03\x12\x12\n\nmtime_secs\x18\x04 \x01(\r\x12\x0b\n\x03usn\x18\x05 \x01(\x05\x12\x0c\n\x04tags\x18\x06 \x03(\t\x12\x0e\n\x06\x66ields\x18\x07 \x03(\t\"A\n\x0e\x41\x64\x64NoteRequest\x12\x1e\n\x04note\x18\x01 \x01(\x0b\x32\x10.anki.notes.Note\x12\x0f\n\x07\x64\x65\x63k_id\x18\x02 \x01(\x03\"X\n\x0f\x41\x64\x64NoteResponse\x12\x34\n\x07\x63hanges\x18\x01 \x01(\x0b\x32#.anki.collection.OpChangesWithCount\x12\x0f\n\x07note_id\x18\x02 \x01(\x03\"?\n\x0f\x41\x64\x64NotesRequest\x12,\n\x08requests\x18\x01 \x03(\x0b\x32\x1a.anki.notes.AddNoteRequest\"M\n\x10\x41\x64\x64NotesResponse\x12+\n\x07\x63hanges\x18\x01 \x01(\x0b\x32\x1a.anki.collection.OpChanges\x12\x0c\n\x04nids\x18\x02 \x03(\x03\"N\n\x12UpdateNotesRequest\x12\x1f\n\x05notes\x18\x01 \x03(\x0b\x32\x10.anki.notes.Note\x12\x17\n\x0fskip_undo_entry\x18\x02 \x01(\x08\"D\n\x18\x44\x65\x66\x61ultsForAddingRequest\x12(\n home_deck_of_current_review_card\x18\x01 \x01(\x03\"7\n\x0f\x44\x65\x63kAndNotetype\x12\x0f\n\x07\x64\x65\x63k_id\x18\x01 \x01(\x03\x12\x13\n\x0bnotetype_id\x18\x02 \x01(\x03\"8\n\x12RemoveNotesRequest\x12\x10\n\x08note_ids\x18\x01 \x03(\x03\x12\x10\n\x08\x63\x61rd_ids\x18\x02 \x03(\x03\"-\n\x1a\x43lozeNumbersInNoteResponse\x12\x0f\n\x07numbers\x18\x01 \x03(\r\"\\\n\x17\x41\x66terNoteUpdatesRequest\x12\x0c\n\x04nids\x18\x01 \x03(\x03\x12\x1b\n\x13mark_notes_modified\x18\x02 \x01(\x08\x12\x16\n\x0egenerate_cards\x18\x03 \x01(\x08\")\n\x19\x46ieldNamesForNotesRequest\x12\x0c\n\x04nids\x18\x01 \x03(\x03\",\n\x1a\x46ieldNamesForNotesResponse\x12\x0e\n\x06\x66ields\x18\x01 \x03(\t\"\xc2\x01\n\x17NoteFieldsCheckResponse\x12\x38\n\x05state\x18\x01 \x01(\x0e\x32).anki.notes.NoteFieldsCheckResponse.State\"m\n\x05State\x12\n\n\x06NORMAL\x10\x00\x12\t\n\x05\x45MPTY\x10\x01\x12\r\n\tDUPLICATE\x10\x02\x12\x11\n\rMISSING_CLOZE\x10\x03\x12\x16\n\x12NOTETYPE_NOT_CLOZE\x10\x04\x12\x13\n\x0f\x46IELD_NOT_CLOZE\x10\x05\x32\xa6\x08\n\x0cNotesService\x12\x37\n\x07NewNote\x12\x1a.anki.notetypes.NotetypeId\x1a\x10.anki.notes.Note\x12\x42\n\x07\x41\x64\x64Note\x12\x1a.anki.notes.AddNoteRequest\x1a\x1b.anki.notes.AddNoteResponse\x12\x45\n\x08\x41\x64\x64Notes\x12\x1b.anki.notes.AddNotesRequest\x1a\x1c.anki.notes.AddNotesResponse\x12V\n\x11\x44\x65\x66\x61ultsForAdding\x12$.anki.notes.DefaultsForAddingRequest\x1a\x1b.anki.notes.DeckAndNotetype\x12H\n\x16\x44\x65\x66\x61ultDeckForNotetype\x12\x1a.anki.notetypes.NotetypeId\x1a\x12.anki.decks.DeckId\x12I\n\x0bUpdateNotes\x12\x1e.anki.notes.UpdateNotesRequest\x1a\x1a.anki.collection.OpChanges\x12/\n\x07GetNote\x12\x12.anki.notes.NoteId\x1a\x10.anki.notes.Note\x12R\n\x0bRemoveNotes\x12\x1e.anki.notes.RemoveNotesRequest\x1a#.anki.collection.OpChangesWithCount\x12N\n\x12\x43lozeNumbersInNote\x12\x10.anki.notes.Note\x1a&.anki.notes.ClozeNumbersInNoteResponse\x12\\\n\x10\x41\x66terNoteUpdates\x12#.anki.notes.AfterNoteUpdatesRequest\x1a#.anki.collection.OpChangesWithCount\x12\x63\n\x12\x46ieldNamesForNotes\x12%.anki.notes.FieldNamesForNotesRequest\x1a&.anki.notes.FieldNamesForNotesResponse\x12H\n\x0fNoteFieldsCheck\x12\x10.anki.notes.Note\x1a#.anki.notes.NoteFieldsCheckResponse\x12\x36\n\x0b\x43\x61rdsOfNote\x12\x12.anki.notes.NoteId\x1a\x13.anki.cards.CardIds\x12K\n\x18GetSingleNotetypeOfNotes\x12\x13.anki.notes.NoteIds\x1a\x1a.anki.notetypes.NotetypeId2\x15\n\x13\x42\x61\x63kendNotesServiceB\x02P\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'anki.notes_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'P\001'
  _globals['_NOTEID']._serialized_start=113
  _globals['_NOTEID']._serialized_end=134
  _globals['_NOTEIDS']._serialized_start=136
  _globals['_NOTEIDS']._serialized_end=163
  _globals['_NOTE']._serialized_start=165
  _globals['_NOTE']._serialized_end=281
  _globals['_ADDNOTEREQUEST']._serialized_start=283
  _globals['_ADDNOTEREQUEST']._serialized_end=348
  _globals['_ADDNOTERESPONSE']._serialized_start=350
  _globals['_ADDNOTERESPONSE']._serialized_end=438
  _globals['_ADDNOTESREQUEST']._serialized_start=440
  _globals['_ADDNOTESREQUEST']._serialized_end=503
  _globals['_ADDNOTESRESPONSE']._serialized_start=505
  _globals['_ADDNOTESRESPONSE']._serialized_end=582
  _globals['_UPDATENOTESREQUEST']._serialized_start=584
  _globals['_UPDATENOTESREQUEST']._serialized_end=662
  _globals['_DEFAULTSFORADDINGREQUEST']._serialized_start=664
  _globals['_DEFAULTSFORADDINGREQUEST']._serialized_end=732
  _globals['_DECKANDNOTETYPE']._serialized_start=734
  _globals['_DECKANDNOTETYPE']._serialized_end=789
  _globals['_REMOVENOTESREQUEST']._serialized_start=791
  _globals['_REMOVENOTESREQUEST']._serialized_end=847
  _globals['_CLOZENUMBERSINNOTERESPONSE']._serialized_start=849
  _globals['_CLOZENUMBERSINNOTERESPONSE']._serialized_end=894
  _globals['_AFTERNOTEUPDATESREQUEST']._serialized_start=896
  _globals['_AFTERNOTEUPDATESREQUEST']._serialized_end=988
  _globals['_FIELDNAMESFORNOTESREQUEST']._serialized_start=990
  _globals['_FIELDNAMESFORNOTESREQUEST']._serialized_end=1031
  _globals['_FIELDNAMESFORNOTESRESPONSE']._serialized_start=1033
  _globals['_FIELDNAMESFORNOTESRESPONSE']._serialized_end=1077
  _globals['_NOTEFIELDSCHECKRESPONSE']._serialized_start=1080
  _globals['_NOTEFIELDSCHECKRESPONSE']._serialized_end=1274
  _globals['_NOTEFIELDSCHECKRESPONSE_STATE']._serialized_start=1165
  _globals['_NOTEFIELDSCHECKRESPONSE_STATE']._serialized_end=1274
  _globals['_NOTESSERVICE']._serialized_start=1277
  _globals['_NOTESSERVICE']._serialized_end=2339
  _globals['_BACKENDNOTESSERVICE']._serialized_start=2341
  _globals['_BACKENDNOTESSERVICE']._serialized_end=2362
# @@protoc_insertion_point(module_scope)
