.dynamically-slottable.svelte-1xfktww,.item.svelte-1389uhi{display:contents}.inline-icon.svelte-hvklej{display:inline-block;width:1em;fill:currentColor;margin:.25em}.text.svelte-14h9bn2{width:100%;min-height:2em}.card-state-customizer.svelte-14h9bn2{background-color:var(--canvas-code);border:1px solid var(--border-subtle);width:100%;height:10em;font-family:monospace}@supports (-webkit-touch-callout: none){.card-state-customizer.svelte-14h9bn2{font-size:16px;overflow-x:hidden}}.spin-box.svelte-u0qjte{width:100%;background:var(--canvas-inset);border:1px solid var(--border);border-radius:var(--border-radius);overflow:hidden;position:relative;display:flex;justify-content:space-between}.spin-box.svelte-u0qjte .suffix:where(.svelte-u0qjte){position:absolute;pointer-events:none;white-space:pre;left:.5em;top:1px}@supports (-webkit-touch-callout: none){.spin-box.svelte-u0qjte .suffix:where(.svelte-u0qjte){top:3.5px}}.spin-box.svelte-u0qjte .invisible:where(.svelte-u0qjte){color:transparent;pointer-events:none}.spin-box.svelte-u0qjte input:where(.svelte-u0qjte){flex-grow:1;border:none;outline:none;background:transparent;padding-left:.5em;padding-right:.5em}.spin-box.svelte-u0qjte input:where(.svelte-u0qjte)::-webkit-inner-spin-button{display:none}.spin-box.svelte-u0qjte:hover .spinner:where(.svelte-u0qjte),.spin-box.svelte-u0qjte:focus-within .spinner:where(.svelte-u0qjte){opacity:.1}.spin-box.svelte-u0qjte:hover .spinner.active:where(.svelte-u0qjte),.spin-box.svelte-u0qjte:focus-within .spinner.active:where(.svelte-u0qjte){opacity:.4;cursor:pointer}.spin-box.svelte-u0qjte:hover .spinner.active:where(.svelte-u0qjte):hover,.spin-box.svelte-u0qjte:focus-within .spinner.active:where(.svelte-u0qjte):hover{opacity:1}.spinner.svelte-u0qjte{opacity:0;height:100%}input.svelte-1ug4p22{width:100%;-webkit-appearance:none;-moz-appearance:none;appearance:none}.save.svelte-huvyj3{margin:0 .75rem}.chevron{height:100%!important}.nightMode.svelte-1l6jc88{background-color:var(--canvas-inset);border-color:var(--border)}.nightMode.svelte-1l6jc88:focus{background-color:var(--canvas-inset)}.default-colors.svelte-1l6jc88{background-color:var(--canvas);color:var(--fg)}.invert.svelte-1l6jc88{filter:invert(1) grayscale(100%) brightness(200%)}.button-bar.svelte-106yi15{display:flex;flex-wrap:nowrap;justify-content:space-between;flex-grow:1}.mr1{margin-right:1rem}ul.svelte-16kd2x7{width:100%;display:flex;flex-wrap:nowrap;justify-content:space-around;padding-inline:0;margin-bottom:.5rem;list-style:none}ul.svelte-16kd2x7:has(li:where(.svelte-16kd2x7):nth-child(3)){justify-content:space-between}button.svelte-16kd2x7{display:block;white-space:nowrap;cursor:pointer;color:var(--fg-subtle);border:1px solid transparent;background-color:transparent;box-shadow:none;font-size:smaller}li.active.svelte-16kd2x7>button:where(.svelte-16kd2x7){color:var(--fg);border-bottom:4px solid var(--border-focus);border-radius:0}button.svelte-16kd2x7:hover{color:var(--fg)}textarea.svelte-jv4p7k{resize:none;overflow-y:auto}.container.svelte-ir5j43{display:flex;justify-content:center}.easy-days-settings.svelte-ir5j43{width:100%;max-width:1000px;display:grid;grid-template-columns:auto 1fr 1fr 1fr;border-collapse:collapse}.easy-days-settings.svelte-ir5j43>:where(.svelte-ir5j43){padding:8px 16px;border-bottom:var(--border) solid 1px}.input-container.svelte-ir5j43{grid-column:2/span 3}span.svelte-ir5j43{display:flex;align-items:center;justify-content:center}span.min-col.svelte-ir5j43{justify-content:flex-start}span.normal-col.svelte-ir5j43{justify-content:flex-end}.header.svelte-ir5j43{word-wrap:break-word;font-size:smaller}.easy-days-settings.svelte-ir5j43 input[type=range]:where(.svelte-ir5j43){width:100%;cursor:pointer}.day.svelte-ir5j43{word-wrap:break-word;font-size:smaller}.modal.svelte-uzhv81{background-color:#00000080;--bs-modal-margin: 0}.svg-container.svelte-uzhv81{width:100%;max-height:max(calc(100vh - 400px),200px);aspect-ratio:600 / 250;display:flex;align-items:center}svg.svelte-uzhv81{width:100%;height:100%}.modal-header.svelte-uzhv81{position:sticky;top:0;background-color:var(--bs-body-bg);z-index:100}.modal-xl{max-width:100vw}div.radio-group.svelte-uzhv81{margin:.5em}.btn.svelte-uzhv81{margin-bottom:.375rem}summary.svelte-uzhv81{margin-bottom:.5em}.btn.svelte-1fz8wcs{margin-bottom:.375rem}.two-line{white-space:pre-wrap;min-height:calc(2ch + 30px);box-sizing:content-box;display:flex;align-content:center;flex-wrap:wrap}hr.svelte-1fz8wcs{border-top:1px solid var(--border);opacity:1}input.svelte-rgbck3{width:100%}.deck-options-page.svelte-1s0da1s{overflow-x:hidden}.deck-options-page.svelte-1s0da1s .container-columns{display:grid;gap:0px}@media(min-width:992px){.deck-options-page.svelte-1s0da1s .container-columns{grid-template-columns:repeat(2,1fr);gap:20px}}
