# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: anki/frontend.proto
# Protobuf Python Version: 7.34.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    7,
    34,
    1,
    '',
    'anki/frontend.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from anki import scheduler_pb2 as anki_dot_scheduler__pb2
from anki import generic_pb2 as anki_dot_generic__pb2
from anki import search_pb2 as anki_dot_search__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x13\x61nki/frontend.proto\x12\ranki.frontend\x1a\x14\x61nki/scheduler.proto\x1a\x12\x61nki/generic.proto\x1a\x11\x61nki/search.proto\"\x83\x01\n\x1bSchedulingStatesWithContext\x12\x30\n\x06states\x18\x01 \x01(\x0b\x32 .anki.scheduler.SchedulingStates\x12\x32\n\x07\x63ontext\x18\x02 \x01(\x0b\x32!.anki.scheduler.SchedulingContext\"[\n\x1aSetSchedulingStatesRequest\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x30\n\x06states\x18\x02 \x01(\x0b\x32 .anki.scheduler.SchedulingStates2\x86\x04\n\x0f\x46rontendService\x12\x61\n\x1eGetSchedulingStatesWithContext\x12\x13.anki.generic.Empty\x1a*.anki.frontend.SchedulingStatesWithContext\x12U\n\x13SetSchedulingStates\x12).anki.frontend.SetSchedulingStatesRequest\x1a\x13.anki.generic.Empty\x12\x36\n\nImportDone\x12\x13.anki.generic.Empty\x1a\x13.anki.generic.Empty\x12?\n\x0fSearchInBrowser\x12\x17.anki.search.SearchNode\x1a\x13.anki.generic.Empty\x12\x43\n\x17\x64\x65\x63kOptionsRequireClose\x12\x13.anki.generic.Empty\x1a\x13.anki.generic.Empty\x12<\n\x10\x64\x65\x63kOptionsReady\x12\x13.anki.generic.Empty\x1a\x13.anki.generic.Empty\x12=\n\x11SaveCustomColours\x12\x13.anki.generic.Empty\x1a\x13.anki.generic.Empty2\x18\n\x16\x42\x61\x63kendFrontendServiceB\x02P\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'anki.frontend_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'P\001'
  _globals['_SCHEDULINGSTATESWITHCONTEXT']._serialized_start=100
  _globals['_SCHEDULINGSTATESWITHCONTEXT']._serialized_end=231
  _globals['_SETSCHEDULINGSTATESREQUEST']._serialized_start=233
  _globals['_SETSCHEDULINGSTATESREQUEST']._serialized_end=324
  _globals['_FRONTENDSERVICE']._serialized_start=327
  _globals['_FRONTENDSERVICE']._serialized_end=845
  _globals['_BACKENDFRONTENDSERVICE']._serialized_start=847
  _globals['_BACKENDFRONTENDSERVICE']._serialized_end=871
# @@protoc_insertion_point(module_scope)
