.badge.svelte-1fmrmgo{color:var(--badge-color, inherit);border:none;background:transparent;padding:0;box-shadow:none}.badge.svelte-1fmrmgo:hover,.badge.svelte-1fmrmgo:active{border:none;background:transparent;box-shadow:none}.dropdown-toggle.svelte-1fmrmgo:after{display:none}
