.tooltip.svelte-17zmtiy{position:absolute;white-space:nowrap;padding:15px;border-radius:5px;font-family:inherit;font-size:15px;opacity:0;pointer-events:none;transition:opacity var(--transition);color:var(--fg);background:var(--canvas-overlay);box-shadow:0 2px 8px #00000026}.tooltip.svelte-17zmtiy table{border-spacing:1em 0}.graph.svelte-1pm718f .graph-element-clickable{cursor:pointer}.graph.svelte-1pm718f .tick line{opacity:.1}.graph.svelte-1pm718f .tick text{opacity:.5;font-size:10px}@media only screen and (max-width:800px){.graph.svelte-1pm718f .tick text{font-size:13px}}@media only screen and (max-width:600px){.graph.svelte-1pm718f .tick text{font-size:16px}}@media only screen and (max-width:600px){.graph.svelte-1pm718f .tick-odd{display:none}}.graph.svelte-1pm718f:focus{outline:0}.subtitle.svelte-1pm718f{text-align:center;margin-bottom:1em}.no-data.svelte-adkmdc rect:where(.svelte-adkmdc){fill:var(--canvas-elevated)}.no-data.svelte-adkmdc text:where(.svelte-adkmdc){text-anchor:middle;fill:gray}g.svelte-2itjvo .domain{opacity:.05}div.svelte-161b58q{display:flex;justify-content:center}@media only screen and (max-device-width:480px)and (orientation:portrait){div.svelte-161b58q{font-size:smaller}}div.svelte-161b58q>*{padding-left:.5em;padding-right:.5em}.hover-columns.svelte-rl3mgq rect{fill:none;pointer-events:all}.hover-columns.svelte-rl3mgq rect:hover{fill:gray;opacity:.2}
