:root{--area-fill: #000000;--area-fill-opacity: .03;--area-stroke: #000000;--area-stroke-opacity: .08}:root[class*=night-mode]{--area-fill: #ffffff;--area-fill-opacity: .08;--area-stroke: #000000;--area-stroke-opacity: .18}.cumulative-overlay.svelte-conii0{pointer-events:none;fill:var(--area-fill);fill-opacity:var(--area-fill-opacity);stroke:var(--area-stroke);stroke-opacity:var(--area-stroke-opacity)}div.svelte-19er4xn{display:flex;justify-content:center}td.svelte-19er4xn{padding:3px}.align-end.svelte-19er4xn{text-align:end}.align-start.svelte-19er4xn{text-align:start}
