import{O as te,I as C,W as ae}from"../chunks/FVFIwcTn.mjs";import"../chunks/E-vkuI0X.mjs";import{i as F}from"../chunks/8UuMqzDP.mjs";import{bg as J,bd as K,bu as W,ad as i,bU as a,aw as c,aq as se,ba as P,bQ as y,bP as g,W as $,bE as _,a3 as T,bo as q,ai as w,b8 as L,au as oe}from"../chunks/W1VwgJDT.mjs";import{p as A}from"../chunks/D-j9Rl0W.mjs";import{s as m}from"../chunks/CCT7NDl_.mjs";import{eK as re,eJ as le,eL as ie,eP as pe,eO as ne,eN as ue,eM as de,ei as ce,eh as ge,eT as $e,eS as me,ee as fe,dY as he,hC as U,eR as ve,ed as Y}from"../chunks/ceRbsEBD.mjs";import{b as u,a as ke}from"../chunks/CO0SAyvL.mjs";import{a as j,S as b,E as z}from"../chunks/DQbO1fYX.mjs";import{R as Pe}from"../chunks/CD0nhXdt.mjs";import{T as ye}from"../chunks/CQCvsTuO.mjs";import{I as _e}from"../chunks/qod7cbwd.mjs";const be=(async({params:f})=>{const l=await te({});return{path:f.path,options:l}}),We=Object.freeze(Object.defineProperty({__proto__:null,load:be},Symbol.toStringTag,{value:"Module"}));function I(){return[{label:re(),value:C.IF_NEWER},{label:le(),value:C.ALWAYS},{label:ie(),value:C.NEVER}]}var Ne=oe("<!> <!> <details><summary> </summary> <!> <!> <!></details>",1);function we(f,l){J(l,!1);let p=A(l,"path",8),t=A(l,"options",12);const s={withScheduling:{title:he(),help:fe(),url:u.PackageImporting.scheduling},withDeckConfigs:{title:me(),help:$e(),url:u.PackageImporting.scheduling},mergeNotetypes:{title:ge(),help:ce(),url:u.PackageImporting.updating},updateNotes:{title:de(),help:ue(),url:u.PackageImporting.updating},updateNotetypes:{title:ne(),help:pe(),url:u.PackageImporting.updating}},Q=Object.values(s);let E=L(),H=L();function h(O){c(E).show(),c(H).to(O)}F(),_e(f,{get path(){return p()},importer:{doImport:()=>ae({packagePath:p(),options:t()},{alertOnError:!1})},children:(O,Ie)=>{Pe(O,{class:"d-block",children:(B,Oe)=>{{let G=w(()=>(i(U),a(()=>Y())));ye(B,{get title(){return c(G)},children:(x,X)=>{var N=Ne(),v=se(N);j(v,{defaultValue:!1,get value(){return t().withScheduling},set value(e){t(t().withScheduling=e,!0)},children:(e,o)=>{b(e,{$$events:{click:()=>h(Object.keys(s).indexOf("withScheduling"))},children:(n,d)=>{P();var r=y();g(()=>m(r,a(()=>s.withScheduling.title))),$(n,r)},$$slots:{default:!0}})},$$slots:{default:!0},$$legacy:!0});var R=_(v,2);j(R,{defaultValue:!1,get value(){return t().withDeckConfigs},set value(e){t(t().withDeckConfigs=e,!0)},children:(e,o)=>{b(e,{$$events:{click:()=>h(Object.keys(s).indexOf("withDeckConfigs"))},children:(n,d)=>{P();var r=y();g(()=>m(r,a(()=>s.withDeckConfigs.title))),$(n,r)},$$slots:{default:!0}})},$$slots:{default:!0},$$legacy:!0});var D=_(R,2),S=T(D),Z=T(S,!0);q(S);var M=_(S,2);j(M,{defaultValue:!1,get value(){return t().mergeNotetypes},set value(e){t(t().mergeNotetypes=e,!0)},children:(e,o)=>{b(e,{$$events:{click:()=>h(Object.keys(s).indexOf("mergeNotetypes"))},children:(n,d)=>{P();var r=y();g(()=>m(r,a(()=>s.mergeNotetypes.title))),$(n,r)},$$slots:{default:!0}})},$$slots:{default:!0},$$legacy:!0});var V=_(M,2);{let e=w(()=>(i(I),a(I)));z(V,{defaultValue:0,get choices(){return c(e)},get value(){return t().updateNotes},set value(o){t(t().updateNotes=o,!0)},children:(o,n)=>{b(o,{$$events:{click:()=>h(Object.keys(s).indexOf("updateNotes"))},children:(d,r)=>{P();var k=y();g(()=>m(k,a(()=>s.updateNotes.title))),$(d,k)},$$slots:{default:!0}})},$$slots:{default:!0},$$legacy:!0})}var ee=_(V,2);{let e=w(()=>(i(I),a(I)));z(ee,{defaultValue:0,get choices(){return c(e)},get value(){return t().updateNotetypes},set value(o){t(t().updateNotetypes=o,!0)},children:(o,n)=>{b(o,{$$events:{click:()=>h(Object.keys(s).indexOf("updateNotetypes"))},children:(d,r)=>{P();var k=y();g(()=>m(k,a(()=>s.updateNotetypes.title))),$(d,k)},$$slots:{default:!0}})},$$slots:{default:!0},$$legacy:!0})}q(D),g(e=>m(Z,e),[()=>(i(U),a(()=>ve()))]),$(x,N)},$$slots:{default:!0,tooltip:(x,X)=>{{let N=w(()=>(i(U),a(()=>Y())));ke(x,{get title(){return c(N)},get url(){return i(u),a(()=>u.PackageImporting.root)},slot:"tooltip",get helpSections(){return Q},$$events:{mount:v=>{W(E,v.detail.modal),W(H,v.detail.carousel)}}})}}}})}},$$slots:{default:!0}})},$$slots:{default:!0}}),K()}function Te(f,l){J(l,!1);let p=A(l,"data",8);F(),we(f,{get path(){return i(p()),a(()=>p().path)},get options(){return i(p()),a(()=>p().options)}}),K()}export{Te as component,We as universal};
