.container.svelte-1qhanys{display:flex;flex-direction:var(--container-direction, column);padding:var(--gutter-block, 0) var(--gutter-inline, 0);margin:0 auto}.container.container-fluid.svelte-1qhanys{width:100%;height:100%;margin:0}.container-xs.svelte-1qhanys{max-width:360px}@media(min-width:576px){.container-xs.svelte-1qhanys,.container-sm.svelte-1qhanys{max-width:540px}}@media(min-width:768px){.container-xs.svelte-1qhanys,.container-sm.svelte-1qhanys,.container-md.svelte-1qhanys{max-width:720px}}@media(min-width:992px){.container-xs.svelte-1qhanys,.container-sm.svelte-1qhanys,.container-md.svelte-1qhanys,.container-lg.svelte-1qhanys{max-width:960px}}@media(min-width:1200px){.container-xs.svelte-1qhanys,.container-sm.svelte-1qhanys,.container-md.svelte-1qhanys,.container-lg.svelte-1qhanys,.container-xl.svelte-1qhanys{max-width:1140px}}@media(min-width:1400px){.container-xs.svelte-1qhanys,.container-sm.svelte-1qhanys,.container-md.svelte-1qhanys,.container-lg.svelte-1qhanys,.container-xl.svelte-1qhanys,.container-xxl.svelte-1qhanys{max-width:1320px}}
