import"./E-vkuI0X.mjs";import"./8UuMqzDP.mjs";import{a3 as i,bo as p,bP as f,W as x,au as d}from"./W1VwgJDT.mjs";import{a7 as v}from"./FVFIwcTn.mjs";import{fE as n}from"./ceRbsEBD.mjs";import{p as e}from"./D-j9Rl0W.mjs";var b=d("<div><!></div>");function E(m,a){let r=e(a,"class",8,""),l=e(a,"breakpoint",8,"xs");var o=b();let s;var c=i(o);v(c,a,"default",{}),p(o),f(()=>{var t;return s=n(o,1,`col ${(t=r())!=null?t:""}`,"svelte-c0c30t",s,{"col-xs":l()==="xs","col-sm":l()==="sm","col-md":l()==="md","col-lg":l()==="lg","col-xl":l()==="xl","col-xxl":l()==="xxl"})}),x(m,o)}export{E as C};
