import"./E-vkuI0X.mjs";import{i as C}from"./8UuMqzDP.mjs";import{bg as D,a3 as s,bo as a,bE as p,bP as w,W as E,bd as F,bC as I,au as N,bI as P}from"./W1VwgJDT.mjs";import{s as S}from"./CCT7NDl_.mjs";import{a7 as c}from"./FVFIwcTn.mjs";import{fF as W,fD as j,fE as f}from"./ceRbsEBD.mjs";import{p as l}from"./D-j9Rl0W.mjs";import{p as q}from"./CD0nhXdt.mjs";var A=N('<div><div class="position-relative"><h1 class="svelte-zuiu8u"> </h1> <div><!></div></div> <!></div>');function Q(b,t){D(t,!1);const d=()=>P(q,"$pageTheme",_),[_,g]=I(),n=window.getComputedStyle(document.body).direction=="rtl";let h=l(t,"id",8,void 0),T=l(t,"class",8,""),k=l(t,"title",8);C();var e=A();let m;W(e,"",{},{"--gutter-block":"2px","--container-margin":"0"});var i=s(e),o=s(i),x=s(o,!0);a(o);var r=p(o,2);let u;var y=s(r);c(y,t,"tooltip",{}),a(r),a(i);var z=p(i,2);c(z,t,"default",{}),a(e),w(()=>{var v;j(e,"id",h()),m=f(e,1,`container ${(v=T())!=null?v:""}`,"svelte-zuiu8u",m,{light:!d().isDark,dark:d().isDark,rtl:n}),S(x,k()),u=f(r,1,"help-badge position-absolute svelte-zuiu8u",null,u,{rtl:n})}),E(b,e),F(),g()}export{Q as T};
