# Form implementation generated from reading ui file 'qt/aqt/forms/widgets.ui'
#
# Created by: PyQt6 UI code generator 6.11.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets
from aqt.utils import tr



class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(925, 822)
        Dialog.setWindowTitle("Qt Widget Gallery")
        self.verticalLayout = QtWidgets.QVBoxLayout(Dialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.topLayout = QtWidgets.QHBoxLayout()
        self.topLayout.setObjectName("topLayout")
        self.styleLabel = QtWidgets.QLabel(parent=Dialog)
        self.styleLabel.setText("Style")
        self.styleLabel.setAlignment(QtCore.Qt.AlignmentFlag.AlignRight|QtCore.Qt.AlignmentFlag.AlignTrailing|QtCore.Qt.AlignmentFlag.AlignVCenter)
        self.styleLabel.setObjectName("styleLabel")
        self.topLayout.addWidget(self.styleLabel)
        self.styleComboBox = QtWidgets.QComboBox(parent=Dialog)
        self.styleComboBox.setCurrentText("")
        self.styleComboBox.setObjectName("styleComboBox")
        self.topLayout.addWidget(self.styleComboBox)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.topLayout.addItem(spacerItem)
        self.disableCheckBox = QtWidgets.QCheckBox(parent=Dialog)
        self.disableCheckBox.setText("Disable Widgets")
        self.disableCheckBox.setObjectName("disableCheckBox")
        self.topLayout.addWidget(self.disableCheckBox)
        self.verticalLayout.addLayout(self.topLayout)
        self.testGrid = QtWidgets.QWidget(parent=Dialog)
        self.testGrid.setObjectName("testGrid")
        self.gridLayout = QtWidgets.QGridLayout(self.testGrid)
        self.gridLayout.setObjectName("gridLayout")
        self.checkButtonsGroup = QtWidgets.QGroupBox(parent=self.testGrid)
        self.checkButtonsGroup.setTitle("Check Buttons")
        self.checkButtonsGroup.setObjectName("checkButtonsGroup")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.checkButtonsGroup)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.radioButtonNotCheckable = QtWidgets.QRadioButton(parent=self.checkButtonsGroup)
        self.radioButtonNotCheckable.setText("RadioButton (not checkable)")
        self.radioButtonNotCheckable.setCheckable(False)
        self.radioButtonNotCheckable.setObjectName("radioButtonNotCheckable")
        self.verticalLayout_3.addWidget(self.radioButtonNotCheckable)
        self.radioButtonChecked = QtWidgets.QRadioButton(parent=self.checkButtonsGroup)
        self.radioButtonChecked.setText("RadioButton (checked)")
        self.radioButtonChecked.setChecked(True)
        self.radioButtonChecked.setObjectName("radioButtonChecked")
        self.verticalLayout_3.addWidget(self.radioButtonChecked)
        self.radioButtonUnchecked = QtWidgets.QRadioButton(parent=self.checkButtonsGroup)
        self.radioButtonUnchecked.setText("RadioButton (unchecked)")
        self.radioButtonUnchecked.setObjectName("radioButtonUnchecked")
        self.verticalLayout_3.addWidget(self.radioButtonUnchecked)
        self.checkBoxTristate = QtWidgets.QCheckBox(parent=self.checkButtonsGroup)
        self.checkBoxTristate.setText("CheckBox (tristate)")
        self.checkBoxTristate.setChecked(True)
        self.checkBoxTristate.setTristate(True)
        self.checkBoxTristate.setObjectName("checkBoxTristate")
        self.verticalLayout_3.addWidget(self.checkBoxTristate)
        self.gridLayout.addWidget(self.checkButtonsGroup, 0, 0, 1, 1)
        self.buttonsGroup = QtWidgets.QGroupBox(parent=self.testGrid)
        self.buttonsGroup.setTitle("Buttons")
        self.buttonsGroup.setObjectName("buttonsGroup")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.buttonsGroup)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.pushButton = QtWidgets.QPushButton(parent=self.buttonsGroup)
        self.pushButton.setText("PushButton")
        self.pushButton.setObjectName("pushButton")
        self.verticalLayout_4.addWidget(self.pushButton)
        self.pushButtonCheckable = QtWidgets.QPushButton(parent=self.buttonsGroup)
        self.pushButtonCheckable.setText("PushButton (checkable)")
        self.pushButtonCheckable.setCheckable(True)
        self.pushButtonCheckable.setChecked(True)
        self.pushButtonCheckable.setObjectName("pushButtonCheckable")
        self.verticalLayout_4.addWidget(self.pushButtonCheckable)
        self.pushButtonFlat = QtWidgets.QPushButton(parent=self.buttonsGroup)
        self.pushButtonFlat.setText("PushButton (flat)")
        self.pushButtonFlat.setAutoDefault(True)
        self.pushButtonFlat.setFlat(True)
        self.pushButtonFlat.setObjectName("pushButtonFlat")
        self.verticalLayout_4.addWidget(self.pushButtonFlat)
        self.gridLayout.addWidget(self.buttonsGroup, 0, 1, 1, 1)
        self.calendarGroup = QtWidgets.QGroupBox(parent=self.testGrid)
        self.calendarGroup.setTitle("CalendarWidget")
        self.calendarGroup.setObjectName("calendarGroup")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.calendarGroup)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.calendarWidget = QtWidgets.QCalendarWidget(parent=self.calendarGroup)
        self.calendarWidget.setObjectName("calendarWidget")
        self.gridLayout_3.addWidget(self.calendarWidget, 0, 0, 1, 1)
        self.gridLayout.addWidget(self.calendarGroup, 1, 0, 1, 1)
        self.textInputsGroup = QtWidgets.QGroupBox(parent=self.testGrid)
        self.textInputsGroup.setTitle("Text Inputs")
        self.textInputsGroup.setObjectName("textInputsGroup")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout(self.textInputsGroup)
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.comboBox = QtWidgets.QComboBox(parent=self.textInputsGroup)
        self.comboBox.setEditable(True)
        self.comboBox.setCurrentText("ComboBox (editable)")
        self.comboBox.setObjectName("comboBox")
        self.verticalLayout_8.addWidget(self.comboBox)
        self.lineEdit = QtWidgets.QLineEdit(parent=self.textInputsGroup)
        self.lineEdit.setText("")
        self.lineEdit.setPlaceholderText("LineEdit")
        self.lineEdit.setObjectName("lineEdit")
        self.verticalLayout_8.addWidget(self.lineEdit)
        self.splitter = QtWidgets.QSplitter(parent=self.textInputsGroup)
        self.splitter.setOrientation(QtCore.Qt.Orientation.Horizontal)
        self.splitter.setObjectName("splitter")
        self.plainTextEdit = QtWidgets.QPlainTextEdit(parent=self.splitter)
        self.plainTextEdit.setPlainText("")
        self.plainTextEdit.setPlaceholderText("PlainTextEdit")
        self.plainTextEdit.setObjectName("plainTextEdit")
        self.textEdit = QtWidgets.QTextEdit(parent=self.splitter)
        self.textEdit.setDocumentTitle("")
        self.textEdit.setPlaceholderText("TextEdit")
        self.textEdit.setObjectName("textEdit")
        self.verticalLayout_8.addWidget(self.splitter)
        self.gridLayout.addWidget(self.textInputsGroup, 1, 1, 1, 1)
        self.otherInputsGroup = QtWidgets.QGroupBox(parent=self.testGrid)
        self.otherInputsGroup.setTitle("Other Inputs")
        self.otherInputsGroup.setObjectName("otherInputsGroup")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.otherInputsGroup)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.keySequenceEdit = QtWidgets.QKeySequenceEdit(parent=self.otherInputsGroup)
        self.keySequenceEdit.setKeySequence("")
        self.keySequenceEdit.setObjectName("keySequenceEdit")
        self.gridLayout_2.addWidget(self.keySequenceEdit, 2, 1, 1, 2)
        self.spinBox = QtWidgets.QSpinBox(parent=self.otherInputsGroup)
        self.spinBox.setSuffix("")
        self.spinBox.setPrefix("")
        self.spinBox.setProperty("value", 1)
        self.spinBox.setObjectName("spinBox")
        self.gridLayout_2.addWidget(self.spinBox, 0, 1, 1, 2)
        self.keySequenceLabel = QtWidgets.QLabel(parent=self.otherInputsGroup)
        self.keySequenceLabel.setText("KeySequenceEdit")
        self.keySequenceLabel.setObjectName("keySequenceLabel")
        self.gridLayout_2.addWidget(self.keySequenceLabel, 2, 0, 1, 1)
        self.dateTimeLabel = QtWidgets.QLabel(parent=self.otherInputsGroup)
        self.dateTimeLabel.setText("DateTimeEdit")
        self.dateTimeLabel.setObjectName("dateTimeLabel")
        self.gridLayout_2.addWidget(self.dateTimeLabel, 1, 0, 1, 1)
        self.spinBoxLabel = QtWidgets.QLabel(parent=self.otherInputsGroup)
        self.spinBoxLabel.setText("SpinBox")
        self.spinBoxLabel.setObjectName("spinBoxLabel")
        self.gridLayout_2.addWidget(self.spinBoxLabel, 0, 0, 1, 1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.sliderLabel = QtWidgets.QLabel(parent=self.otherInputsGroup)
        self.sliderLabel.setText("Slider")
        self.sliderLabel.setObjectName("sliderLabel")
        self.horizontalLayout.addWidget(self.sliderLabel)
        self.horizontalSlider = QtWidgets.QSlider(parent=self.otherInputsGroup)
        self.horizontalSlider.setOrientation(QtCore.Qt.Orientation.Horizontal)
        self.horizontalSlider.setObjectName("horizontalSlider")
        self.horizontalLayout.addWidget(self.horizontalSlider)
        self.label = QtWidgets.QLabel(parent=self.otherInputsGroup)
        self.label.setText("Dial")
        self.label.setAlignment(QtCore.Qt.AlignmentFlag.AlignRight|QtCore.Qt.AlignmentFlag.AlignTrailing|QtCore.Qt.AlignmentFlag.AlignVCenter)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.dial = QtWidgets.QDial(parent=self.otherInputsGroup)
        self.dial.setObjectName("dial")
        self.horizontalLayout.addWidget(self.dial)
        self.gridLayout_2.addLayout(self.horizontalLayout, 3, 0, 1, 3)
        self.dateTimeEdit = QtWidgets.QDateTimeEdit(parent=self.otherInputsGroup)
        self.dateTimeEdit.setObjectName("dateTimeEdit")
        self.gridLayout_2.addWidget(self.dateTimeEdit, 1, 1, 1, 2)
        self.gridLayout.addWidget(self.otherInputsGroup, 2, 0, 1, 1)
        self.tabWidget = QtWidgets.QTabWidget(parent=self.testGrid)
        self.tabWidget.setObjectName("tabWidget")
        self.listTab = QtWidgets.QWidget()
        self.listTab.setObjectName("listTab")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.listTab)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.listWidget = QtWidgets.QListWidget(parent=self.listTab)
        self.listWidget.setObjectName("listWidget")
        self.verticalLayout_6.addWidget(self.listWidget)
        self.tabWidget.addTab(self.listTab, "ListWidget")
        self.treeTab = QtWidgets.QWidget()
        self.treeTab.setObjectName("treeTab")
        self.verticalLayout_7 = QtWidgets.QVBoxLayout(self.treeTab)
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.treeWidget = QtWidgets.QTreeWidget(parent=self.treeTab)
        self.treeWidget.setObjectName("treeWidget")
        self.treeWidget.headerItem().setText(0, "1")
        self.verticalLayout_7.addWidget(self.treeWidget)
        self.tabWidget.addTab(self.treeTab, "TreeWidget")
        self.tableTab = QtWidgets.QWidget()
        self.tableTab.setObjectName("tableTab")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.tableTab)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.tableWidget = QtWidgets.QTableWidget(parent=self.tableTab)
        self.tableWidget.setRowCount(0)
        self.tableWidget.setColumnCount(0)
        self.tableWidget.setObjectName("tableWidget")
        self.verticalLayout_5.addWidget(self.tableWidget)
        self.tabWidget.addTab(self.tableTab, "TableWidget")
        self.gridLayout.addWidget(self.tabWidget, 2, 1, 1, 1)
        self.progressBarLayout = QtWidgets.QHBoxLayout()
        self.progressBarLayout.setObjectName("progressBarLayout")
        self.progressBarLabel = QtWidgets.QLabel(parent=self.testGrid)
        self.progressBarLabel.setText("ProgressBar")
        self.progressBarLabel.setObjectName("progressBarLabel")
        self.progressBarLayout.addWidget(self.progressBarLabel)
        self.progressBar = QtWidgets.QProgressBar(parent=self.testGrid)
        self.progressBar.setProperty("value", 24)
        self.progressBar.setObjectName("progressBar")
        self.progressBarLayout.addWidget(self.progressBar)
        self.gridLayout.addLayout(self.progressBarLayout, 3, 0, 1, 2)
        self.verticalLayout.addWidget(self.testGrid)

        self.retranslateUi(Dialog)
        self.tabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        pass