import{a6 as b,bY as d,bX as c,br as i,bU as g,bq as m,aw as p,ad as v,ag as h,ao as k}from"./W1VwgJDT.mjs";function x(a=!1){const s=b,e=s.l.u;if(!e)return;let f=()=>v(s.s);if(a){let n=0,t={};const _=h(()=>{let l=!1;const r=s.s;for(const o in r)r[o]!==t[o]&&(t[o]=r[o],l=!0);return l&&n++,n});f=()=>p(_)}e.b.length&&d(()=>{u(s,f),i(e.b)}),c(()=>{const n=g(()=>e.m.map(m));return()=>{for(const t of n)typeof t=="function"&&t()}}),e.a.length&&c(()=>{u(s,f),i(e.a)})}function u(a,s){if(a.l.s)for(const e of a.l.s)p(e);s()}k();export{x as i};
