h2.svelte-1k8acwp{margin-bottom:1em;width:100%}.chapter-redirect.svelte-1k8acwp{width:100%;color:var(--fg-subtle);font-size:small}.icon.svelte-1k8acwp{display:inline-block;width:1em;fill:currentColor;margin-right:.25em;margin-bottom:1.25em}#nav.svelte-1ixh9xj{margin-bottom:1.5rem}.modal.svelte-1ixh9xj{z-index:1066;background-color:#00000080}.modal-title.svelte-1ixh9xj{margin-inline-end:.75rem}.modal-content.svelte-1ixh9xj{background-color:var(--canvas);color:var(--fg);border-radius:var(--border-radius-medium, 10px)}.invert.svelte-1ixh9xj{filter:invert(1) grayscale(100%) brightness(200%)}ul.svelte-1ixh9xj{list-style-type:none;margin:0;padding:0}li.svelte-1ixh9xj button:where(.svelte-1ixh9xj){display:block;padding:.5rem .75rem;text-decoration:none;text-align:start;min-width:250px;background-color:var(--canvas);border:1px solid transparent;cursor:pointer;border-radius:0}li.svelte-1ixh9xj button:where(.svelte-1ixh9xj):hover{background-color:var(--canvas-inset)}li.svelte-1ixh9xj button.active:where(.svelte-1ixh9xj){border-inline-start:4px solid var(--border-focus)}.modal-header.svelte-1ixh9xj{flex-direction:column;align-items:normal;padding-bottom:0}.chapter-redirect.svelte-1ixh9xj{width:100%;color:var(--fg-subtle);font-size:small}
