import"../chunks/E-vkuI0X.mjs";import{i as e}from"../chunks/8UuMqzDP.mjs";import{bg as p,bd as m,ad as n,bU as s}from"../chunks/W1VwgJDT.mjs";import{p as i}from"../chunks/D-j9Rl0W.mjs";import{Z as l}from"../chunks/FVFIwcTn.mjs";import{I as d}from"../chunks/qod7cbwd.mjs";const u=(async({params:t})=>({path:t.path})),v=Object.freeze(Object.defineProperty({__proto__:null,load:u},Symbol.toStringTag,{value:"Module"}));function y(t,o){p(o,!1);let a=i(o,"data",8);const r={doImport:()=>l({val:a().path},{})};e(),d(t,{get path(){return n(a()),s(()=>a().path)},get importer(){return r}}),m()}export{y as component,v as universal};
