# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: anki/sync.proto
# Protobuf Python Version: 7.34.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    7,
    34,
    1,
    '',
    'anki/sync.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from anki import generic_pb2 as anki_dot_generic__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0f\x61nki/sync.proto\x12\tanki.sync\x1a\x12\x61nki/generic.proto\"n\n\x08SyncAuth\x12\x0c\n\x04hkey\x18\x01 \x01(\t\x12\x15\n\x08\x65ndpoint\x18\x02 \x01(\tH\x00\x88\x01\x01\x12\x1c\n\x0fio_timeout_secs\x18\x03 \x01(\rH\x01\x88\x01\x01\x42\x0b\n\t_endpointB\x12\n\x10_io_timeout_secs\"Z\n\x10SyncLoginRequest\x12\x10\n\x08username\x18\x01 \x01(\t\x12\x10\n\x08password\x18\x02 \x01(\t\x12\x15\n\x08\x65ndpoint\x18\x03 \x01(\tH\x00\x88\x01\x01\x42\x0b\n\t_endpoint\"\xb6\x01\n\x12SyncStatusResponse\x12\x38\n\x08required\x18\x01 \x01(\x0e\x32&.anki.sync.SyncStatusResponse.Required\x12\x19\n\x0cnew_endpoint\x18\x04 \x01(\tH\x00\x88\x01\x01\":\n\x08Required\x12\x0e\n\nNO_CHANGES\x10\x00\x12\x0f\n\x0bNORMAL_SYNC\x10\x01\x12\r\n\tFULL_SYNC\x10\x02\x42\x0f\n\r_new_endpoint\"N\n\x15SyncCollectionRequest\x12!\n\x04\x61uth\x18\x01 \x01(\x0b\x32\x13.anki.sync.SyncAuth\x12\x12\n\nsync_media\x18\x02 \x01(\x08\"\xb7\x02\n\x16SyncCollectionResponse\x12\x13\n\x0bhost_number\x18\x01 \x01(\r\x12\x16\n\x0eserver_message\x18\x02 \x01(\t\x12\x43\n\x08required\x18\x03 \x01(\x0e\x32\x31.anki.sync.SyncCollectionResponse.ChangesRequired\x12\x19\n\x0cnew_endpoint\x18\x04 \x01(\tH\x00\x88\x01\x01\x12\x18\n\x10server_media_usn\x18\x05 \x01(\x05\"e\n\x0f\x43hangesRequired\x12\x0e\n\nNO_CHANGES\x10\x00\x12\x0f\n\x0bNORMAL_SYNC\x10\x01\x12\r\n\tFULL_SYNC\x10\x02\x12\x11\n\rFULL_DOWNLOAD\x10\x03\x12\x0f\n\x0b\x46ULL_UPLOAD\x10\x04\x42\x0f\n\r_new_endpoint\"Y\n\x17MediaSyncStatusResponse\x12\x0e\n\x06\x61\x63tive\x18\x01 \x01(\x08\x12.\n\x08progress\x18\x02 \x01(\x0b\x32\x1c.anki.sync.MediaSyncProgress\"D\n\x11MediaSyncProgress\x12\x0f\n\x07\x63hecked\x18\x01 \x01(\t\x12\r\n\x05\x61\x64\x64\x65\x64\x18\x02 \x01(\t\x12\x0f\n\x07removed\x18\x03 \x01(\t\"x\n\x1b\x46ullUploadOrDownloadRequest\x12!\n\x04\x61uth\x18\x01 \x01(\x0b\x32\x13.anki.sync.SyncAuth\x12\x0e\n\x06upload\x18\x02 \x01(\x08\x12\x17\n\nserver_usn\x18\x03 \x01(\x05H\x00\x88\x01\x01\x42\r\n\x0b_server_usn2\r\n\x0bSyncService2\xf9\x04\n\x12\x42\x61\x63kendSyncService\x12\x35\n\tSyncMedia\x12\x13.anki.sync.SyncAuth\x1a\x13.anki.generic.Empty\x12:\n\x0e\x41\x62ortMediaSync\x12\x13.anki.generic.Empty\x1a\x13.anki.generic.Empty\x12J\n\x0fMediaSyncStatus\x12\x13.anki.generic.Empty\x1a\".anki.sync.MediaSyncStatusResponse\x12=\n\tSyncLogin\x12\x1b.anki.sync.SyncLoginRequest\x1a\x13.anki.sync.SyncAuth\x12@\n\nSyncStatus\x12\x13.anki.sync.SyncAuth\x1a\x1d.anki.sync.SyncStatusResponse\x12U\n\x0eSyncCollection\x12 .anki.sync.SyncCollectionRequest\x1a!.anki.sync.SyncCollectionResponse\x12S\n\x14\x46ullUploadOrDownload\x12&.anki.sync.FullUploadOrDownloadRequest\x1a\x13.anki.generic.Empty\x12\x35\n\tAbortSync\x12\x13.anki.generic.Empty\x1a\x13.anki.generic.Empty\x12@\n\x14SetCustomCertificate\x12\x14.anki.generic.String\x1a\x12.anki.generic.BoolB\x02P\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'anki.sync_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'P\001'
  _globals['_SYNCAUTH']._serialized_start=50
  _globals['_SYNCAUTH']._serialized_end=160
  _globals['_SYNCLOGINREQUEST']._serialized_start=162
  _globals['_SYNCLOGINREQUEST']._serialized_end=252
  _globals['_SYNCSTATUSRESPONSE']._serialized_start=255
  _globals['_SYNCSTATUSRESPONSE']._serialized_end=437
  _globals['_SYNCSTATUSRESPONSE_REQUIRED']._serialized_start=362
  _globals['_SYNCSTATUSRESPONSE_REQUIRED']._serialized_end=420
  _globals['_SYNCCOLLECTIONREQUEST']._serialized_start=439
  _globals['_SYNCCOLLECTIONREQUEST']._serialized_end=517
  _globals['_SYNCCOLLECTIONRESPONSE']._serialized_start=520
  _globals['_SYNCCOLLECTIONRESPONSE']._serialized_end=831
  _globals['_SYNCCOLLECTIONRESPONSE_CHANGESREQUIRED']._serialized_start=713
  _globals['_SYNCCOLLECTIONRESPONSE_CHANGESREQUIRED']._serialized_end=814
  _globals['_MEDIASYNCSTATUSRESPONSE']._serialized_start=833
  _globals['_MEDIASYNCSTATUSRESPONSE']._serialized_end=922
  _globals['_MEDIASYNCPROGRESS']._serialized_start=924
  _globals['_MEDIASYNCPROGRESS']._serialized_end=992
  _globals['_FULLUPLOADORDOWNLOADREQUEST']._serialized_start=994
  _globals['_FULLUPLOADORDOWNLOADREQUEST']._serialized_end=1114
  _globals['_SYNCSERVICE']._serialized_start=1116
  _globals['_SYNCSERVICE']._serialized_end=1129
  _globals['_BACKENDSYNCSERVICE']._serialized_start=1132
  _globals['_BACKENDSYNCSERVICE']._serialized_end=1765
# @@protoc_insertion_point(module_scope)
