var I={exports:{}},q={exports:{}};/*!
  * Bootstrap data.js v5.3.8 (https://getbootstrap.com/)
  * Copyright 2011-2025 The Bootstrap Authors (https://github.com/twbs/bootstrap/graphs/contributors)
  * Licensed under MIT (https://github.com/twbs/bootstrap/blob/main/LICENSE)
  */var G=q.exports,V;function tt(){return V||(V=1,(function(v,C){(function(s,f){v.exports=f()})(G,(function(){const s=new Map;return{set(p,n,e){s.has(p)||s.set(p,new Map);const r=s.get(p);if(!r.has(n)&&r.size!==0){console.error(`Bootstrap doesn't allow more than one instance per element. Bound instance: ${Array.from(r.keys())[0]}.`);return}r.set(n,e)},get(p,n){return s.has(p)&&s.get(p).get(n)||null},remove(p,n){if(!s.has(p))return;const e=s.get(p);e.delete(n),e.size===0&&s.delete(p)}}}))})(q)),q.exports}var P={exports:{}},_={exports:{}};/*!
  * Bootstrap index.js v5.3.8 (https://getbootstrap.com/)
  * Copyright 2011-2025 The Bootstrap Authors (https://github.com/twbs/bootstrap/graphs/contributors)
  * Licensed under MIT (https://github.com/twbs/bootstrap/blob/main/LICENSE)
  */var et=_.exports,Y;function U(){return Y||(Y=1,(function(v,C){(function(s,f){f(C)})(et,(function(s){const n="transitionend",e=t=>(t&&window.CSS&&window.CSS.escape&&(t=t.replace(/#([^\s"#']+)/g,(u,o)=>`#${CSS.escape(o)}`)),t),r=t=>t==null?`${t}`:Object.prototype.toString.call(t).match(/\s([a-z]+)/i)[1].toLowerCase(),g=t=>{do t+=Math.floor(Math.random()*1e6);while(document.getElementById(t));return t},l=t=>{if(!t)return 0;let{transitionDuration:u,transitionDelay:o}=window.getComputedStyle(t);const h=Number.parseFloat(u),m=Number.parseFloat(o);return!h&&!m?0:(u=u.split(",")[0],o=o.split(",")[0],(Number.parseFloat(u)+Number.parseFloat(o))*1e3)},E=t=>{t.dispatchEvent(new Event(n))},S=t=>!t||typeof t!="object"?!1:(typeof t.jquery<"u"&&(t=t[0]),typeof t.nodeType<"u"),F=t=>S(t)?t.jquery?t[0]:t:typeof t=="string"&&t.length>0?document.querySelector(e(t)):null,j=t=>{if(!S(t)||t.getClientRects().length===0)return!1;const u=getComputedStyle(t).getPropertyValue("visibility")==="visible",o=t.closest("details:not([open])");if(!o)return u;if(o!==t){const h=t.closest("summary");if(h&&h.parentNode!==o||h===null)return!1}return u},$=t=>!t||t.nodeType!==Node.ELEMENT_NODE||t.classList.contains("disabled")?!0:typeof t.disabled<"u"?t.disabled:t.hasAttribute("disabled")&&t.getAttribute("disabled")!=="false",x=t=>{if(!document.documentElement.attachShadow)return null;if(typeof t.getRootNode=="function"){const u=t.getRootNode();return u instanceof ShadowRoot?u:null}return t instanceof ShadowRoot?t:t.parentNode?x(t.parentNode):null},R=()=>{},M=t=>{t.offsetHeight},L=()=>window.jQuery&&!document.body.hasAttribute("data-bs-no-jquery")?window.jQuery:null,N=[],w=t=>{document.readyState==="loading"?(N.length||document.addEventListener("DOMContentLoaded",()=>{for(const u of N)u()}),N.push(t)):t()},T=()=>document.documentElement.dir==="rtl",a=t=>{w(()=>{const u=L();if(u){const o=t.NAME,h=u.fn[o];u.fn[o]=t.jQueryInterface,u.fn[o].Constructor=t,u.fn[o].noConflict=()=>(u.fn[o]=h,t.jQueryInterface)}})},i=(t,u=[],o=t)=>typeof t=="function"?t.call(...u):o,c=(t,u,o=!0)=>{if(!o){i(t);return}const m=l(u)+5;let b=!1;const y=({target:A})=>{A===u&&(b=!0,u.removeEventListener(n,y),i(t))};u.addEventListener(n,y),setTimeout(()=>{b||E(u)},m)},d=(t,u,o,h)=>{const m=t.length;let b=t.indexOf(u);return b===-1?!o&&h?t[m-1]:t[0]:(b+=o?1:-1,h&&(b=(b+m)%m),t[Math.max(0,Math.min(b,m-1))])};s.defineJQueryPlugin=a,s.execute=i,s.executeAfterTransition=c,s.findShadowRoot=x,s.getElement=F,s.getNextActiveElement=d,s.getTransitionDurationFromElement=l,s.getUID=g,s.getjQuery=L,s.isDisabled=$,s.isElement=S,s.isRTL=T,s.isVisible=j,s.noop=R,s.onDOMContentLoaded=w,s.parseSelector=e,s.reflow=M,s.toType=r,s.triggerTransitionEnd=E,Object.defineProperty(s,Symbol.toStringTag,{value:"Module"})}))})(_,_.exports)),_.exports}/*!
  * Bootstrap event-handler.js v5.3.8 (https://getbootstrap.com/)
  * Copyright 2011-2025 The Bootstrap Authors (https://github.com/twbs/bootstrap/graphs/contributors)
  * Licensed under MIT (https://github.com/twbs/bootstrap/blob/main/LICENSE)
  */var nt=P.exports,z;function rt(){return z||(z=1,(function(v,C){(function(s,f){v.exports=f(U())})(nt,(function(s){const f=/[^.]*(?=\..*)\.|.*/,p=/\..*/,n=/::\d+$/,e={};let r=1;const g={mouseenter:"mouseover",mouseleave:"mouseout"},l=new Set(["click","dblclick","mouseup","mousedown","contextmenu","mousewheel","DOMMouseScroll","mouseover","mouseout","mousemove","selectstart","selectend","keydown","keypress","keyup","orientationchange","touchstart","touchmove","touchend","touchcancel","pointerdown","pointermove","pointerup","pointerleave","pointercancel","gesturestart","gesturechange","gestureend","focus","blur","change","reset","select","submit","focusin","focusout","load","unload","beforeunload","resize","move","DOMContentLoaded","readystatechange","error","abort","scroll"]);function E(a,i){return i&&`${i}::${r++}`||a.uidEvent||r++}function S(a){const i=E(a);return a.uidEvent=i,e[i]=e[i]||{},e[i]}function F(a,i){return function c(d){return T(d,{delegateTarget:a}),c.oneOff&&w.off(a,d.type,i),i.apply(a,[d])}}function j(a,i,c){return function d(t){const u=a.querySelectorAll(i);for(let{target:o}=t;o&&o!==this;o=o.parentNode)for(const h of u)if(h===o)return T(t,{delegateTarget:o}),d.oneOff&&w.off(a,t.type,i,c),c.apply(o,[t])}}function $(a,i,c=null){return Object.values(a).find(d=>d.callable===i&&d.delegationSelector===c)}function x(a,i,c){const d=typeof i=="string",t=d?c:i||c;let u=N(a);return l.has(u)||(u=a),[d,t,u]}function R(a,i,c,d,t){if(typeof i!="string"||!a)return;let[u,o,h]=x(i,c,d);i in g&&(o=(Z=>function(O){if(!O.relatedTarget||O.relatedTarget!==O.delegateTarget&&!O.delegateTarget.contains(O.relatedTarget))return Z.call(this,O)})(o));const m=S(a),b=m[h]||(m[h]={}),y=$(b,o,u?c:null);if(y){y.oneOff=y.oneOff&&t;return}const A=E(o,i.replace(f,"")),D=u?j(a,c,o):F(a,o);D.delegationSelector=u?c:null,D.callable=o,D.oneOff=t,D.uidEvent=A,b[A]=D,a.addEventListener(h,D,u)}function M(a,i,c,d,t){const u=$(i[c],d,t);u&&(a.removeEventListener(c,u,!!t),delete i[c][u.uidEvent])}function L(a,i,c,d){const t=i[c]||{};for(const[u,o]of Object.entries(t))u.includes(d)&&M(a,i,c,o.callable,o.delegationSelector)}function N(a){return a=a.replace(p,""),g[a]||a}const w={on(a,i,c,d){R(a,i,c,d,!1)},one(a,i,c,d){R(a,i,c,d,!0)},off(a,i,c,d){if(typeof i!="string"||!a)return;const[t,u,o]=x(i,c,d),h=o!==i,m=S(a),b=m[o]||{},y=i.startsWith(".");if(typeof u<"u"){if(!Object.keys(b).length)return;M(a,m,o,u,t?c:null);return}if(y)for(const A of Object.keys(m))L(a,m,A,i.slice(1));for(const[A,D]of Object.entries(b)){const Q=A.replace(n,"");(!h||i.includes(Q))&&M(a,m,o,D.callable,D.delegationSelector)}},trigger(a,i,c){if(typeof i!="string"||!a)return null;const d=s.getjQuery(),t=N(i),u=i!==t;let o=null,h=!0,m=!0,b=!1;u&&d&&(o=d.Event(i,c),d(a).trigger(o),h=!o.isPropagationStopped(),m=!o.isImmediatePropagationStopped(),b=o.isDefaultPrevented());const y=T(new Event(i,{bubbles:h,cancelable:!0}),c);return b&&y.preventDefault(),m&&a.dispatchEvent(y),y.defaultPrevented&&o&&o.preventDefault(),y}};function T(a,i={}){for(const[c,d]of Object.entries(i))try{a[c]=d}catch{Object.defineProperty(a,c,{configurable:!0,get(){return d}})}return a}return w}))})(P)),P.exports}var k={exports:{}},H={exports:{}};/*!
  * Bootstrap manipulator.js v5.3.8 (https://getbootstrap.com/)
  * Copyright 2011-2025 The Bootstrap Authors (https://github.com/twbs/bootstrap/graphs/contributors)
  * Licensed under MIT (https://github.com/twbs/bootstrap/blob/main/LICENSE)
  */var ot=H.exports,B;function st(){return B||(B=1,(function(v,C){(function(s,f){v.exports=f()})(ot,(function(){function s(n){if(n==="true")return!0;if(n==="false")return!1;if(n===Number(n).toString())return Number(n);if(n===""||n==="null")return null;if(typeof n!="string")return n;try{return JSON.parse(decodeURIComponent(n))}catch{return n}}function f(n){return n.replace(/[A-Z]/g,e=>`-${e.toLowerCase()}`)}return{setDataAttribute(n,e,r){n.setAttribute(`data-bs-${f(e)}`,r)},removeDataAttribute(n,e){n.removeAttribute(`data-bs-${f(e)}`)},getDataAttributes(n){if(!n)return{};const e={},r=Object.keys(n.dataset).filter(g=>g.startsWith("bs")&&!g.startsWith("bsConfig"));for(const g of r){let l=g.replace(/^bs/,"");l=l.charAt(0).toLowerCase()+l.slice(1),e[l]=s(n.dataset[g])}return e},getDataAttribute(n,e){return s(n.getAttribute(`data-bs-${f(e)}`))}}}))})(H)),H.exports}/*!
  * Bootstrap config.js v5.3.8 (https://getbootstrap.com/)
  * Copyright 2011-2025 The Bootstrap Authors (https://github.com/twbs/bootstrap/graphs/contributors)
  * Licensed under MIT (https://github.com/twbs/bootstrap/blob/main/LICENSE)
  */var it=k.exports,W;function ut(){return W||(W=1,(function(v,C){(function(s,f){v.exports=f(st(),U())})(it,(function(s,f){class p{static get Default(){return{}}static get DefaultType(){return{}}static get NAME(){throw new Error('You have to implement the static method "NAME", for each component!')}_getConfig(e){return e=this._mergeConfigObj(e),e=this._configAfterMerge(e),this._typeCheckConfig(e),e}_configAfterMerge(e){return e}_mergeConfigObj(e,r){const g=f.isElement(r)?s.getDataAttribute(r,"config"):{};return{...this.constructor.Default,...typeof g=="object"?g:{},...f.isElement(r)?s.getDataAttributes(r):{},...typeof e=="object"?e:{}}}_typeCheckConfig(e,r=this.constructor.DefaultType){for(const[g,l]of Object.entries(r)){const E=e[g],S=f.isElement(E)?"element":f.toType(E);if(!new RegExp(l).test(S))throw new TypeError(`${this.constructor.NAME.toUpperCase()}: Option "${g}" provided type "${S}" but expected type "${l}".`)}}}return p}))})(k)),k.exports}/*!
  * Bootstrap base-component.js v5.3.8 (https://getbootstrap.com/)
  * Copyright 2011-2025 The Bootstrap Authors (https://github.com/twbs/bootstrap/graphs/contributors)
  * Licensed under MIT (https://github.com/twbs/bootstrap/blob/main/LICENSE)
  */var at=I.exports,J;function lt(){return J||(J=1,(function(v,C){(function(s,f){v.exports=f(tt(),rt(),ut(),U())})(at,(function(s,f,p,n){const e="5.3.8";class r extends p{constructor(l,E){super(),l=n.getElement(l),l&&(this._element=l,this._config=this._getConfig(E),s.set(this._element,this.constructor.DATA_KEY,this))}dispose(){s.remove(this._element,this.constructor.DATA_KEY),f.off(this._element,this.constructor.EVENT_KEY);for(const l of Object.getOwnPropertyNames(this))this[l]=null}_queueCallback(l,E,S=!0){n.executeAfterTransition(l,E,S)}_getConfig(l){return l=this._mergeConfigObj(l,this._element),l=this._configAfterMerge(l),this._typeCheckConfig(l),l}static getInstance(l){return s.get(n.getElement(l),this.DATA_KEY)}static getOrCreateInstance(l,E={}){return this.getInstance(l)||new this(l,typeof E=="object"?E:null)}static get VERSION(){return e}static get DATA_KEY(){return`bs.${this.NAME}`}static get EVENT_KEY(){return`.${this.DATA_KEY}`}static eventName(l){return`${l}${this.EVENT_KEY}`}}return r}))})(I)),I.exports}var K={exports:{}};/*!
  * Bootstrap selector-engine.js v5.3.8 (https://getbootstrap.com/)
  * Copyright 2011-2025 The Bootstrap Authors (https://github.com/twbs/bootstrap/graphs/contributors)
  * Licensed under MIT (https://github.com/twbs/bootstrap/blob/main/LICENSE)
  */var ct=K.exports,X;function ft(){return X||(X=1,(function(v,C){(function(s,f){v.exports=f(U())})(ct,(function(s){const f=n=>{let e=n.getAttribute("data-bs-target");if(!e||e==="#"){let r=n.getAttribute("href");if(!r||!r.includes("#")&&!r.startsWith("."))return null;r.includes("#")&&!r.startsWith("#")&&(r=`#${r.split("#")[1]}`),e=r&&r!=="#"?r.trim():null}return e?e.split(",").map(r=>s.parseSelector(r)).join(","):null},p={find(n,e=document.documentElement){return[].concat(...Element.prototype.querySelectorAll.call(e,n))},findOne(n,e=document.documentElement){return Element.prototype.querySelector.call(e,n)},children(n,e){return[].concat(...n.children).filter(r=>r.matches(e))},parents(n,e){const r=[];let g=n.parentNode.closest(e);for(;g;)r.push(g),g=g.parentNode.closest(e);return r},prev(n,e){let r=n.previousElementSibling;for(;r;){if(r.matches(e))return[r];r=r.previousElementSibling}return[]},next(n,e){let r=n.nextElementSibling;for(;r;){if(r.matches(e))return[r];r=r.nextElementSibling}return[]},focusableChildren(n){const e=["a","button","input","textarea","select","details","[tabindex]",'[contenteditable="true"]'].map(r=>`${r}:not([tabindex^="-"])`).join(",");return this.find(e,n).filter(r=>!s.isDisabled(r)&&s.isVisible(r))},getSelectorFromElement(n){const e=f(n);return e&&p.findOne(e)?e:null},getElementFromSelector(n){const e=f(n);return e?p.findOne(e):null},getMultipleElementsFromSelector(n){const e=f(n);return e?p.find(e):[]}};return p}))})(K)),K.exports}export{ut as a,rt as b,st as c,ft as d,U as e,lt as r};
