# Form implementation generated from reading ui file 'qt/aqt/forms/modelopts.ui'
#
# Created by: PyQt6 UI code generator 6.11.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets
from aqt.utils import tr



class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.setWindowModality(QtCore.Qt.WindowModality.ApplicationModal)
        Dialog.resize(374, 344)
        Dialog.setWindowTitle("")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(Dialog)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.qtabwidget = QtWidgets.QTabWidget(parent=Dialog)
        self.qtabwidget.setObjectName("qtabwidget")
        self.tab = QtWidgets.QWidget()
        self.tab.setObjectName("tab")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.tab)
        self.verticalLayout.setObjectName("verticalLayout")
        self.latexsvg = QtWidgets.QCheckBox(parent=self.tab)
        self.latexsvg.setObjectName("latexsvg")
        self.verticalLayout.addWidget(self.latexsvg, 0, QtCore.Qt.AlignmentFlag.AlignLeft)
        self.label_6 = QtWidgets.QLabel(parent=self.tab)
        self.label_6.setObjectName("label_6")
        self.verticalLayout.addWidget(self.label_6)
        self.latexHeader = QtWidgets.QTextEdit(parent=self.tab)
        self.latexHeader.setTabChangesFocus(True)
        self.latexHeader.setObjectName("latexHeader")
        self.verticalLayout.addWidget(self.latexHeader)
        self.label_7 = QtWidgets.QLabel(parent=self.tab)
        self.label_7.setObjectName("label_7")
        self.verticalLayout.addWidget(self.label_7)
        self.latexFooter = QtWidgets.QTextEdit(parent=self.tab)
        self.latexFooter.setTabChangesFocus(True)
        self.latexFooter.setObjectName("latexFooter")
        self.verticalLayout.addWidget(self.latexFooter)
        self.qtabwidget.addTab(self.tab, "")
        self.verticalLayout_2.addWidget(self.qtabwidget)
        self.buttonBox = QtWidgets.QDialogButtonBox(parent=Dialog)
        self.buttonBox.setOrientation(QtCore.Qt.Orientation.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.StandardButton.Close|QtWidgets.QDialogButtonBox.StandardButton.Help)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout_2.addWidget(self.buttonBox)

        self.retranslateUi(Dialog)
        self.qtabwidget.setCurrentIndex(0)
        self.buttonBox.accepted.connect(Dialog.accept) # type: ignore  # type: ignore
        self.buttonBox.rejected.connect(Dialog.reject) # type: ignore  # type: ignore
        QtCore.QMetaObject.connectSlotsByName(Dialog)
        Dialog.setTabOrder(self.qtabwidget, self.buttonBox)
        Dialog.setTabOrder(self.buttonBox, self.latexHeader)
        Dialog.setTabOrder(self.latexHeader, self.latexFooter)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        self.latexsvg.setText(tr.notetypes_create_scalable_images_with_dvisvgm())
        self.label_6.setText(tr.notetypes_header())
        self.label_7.setText(tr.notetypes_footer())
        self.qtabwidget.setTabText(self.qtabwidget.indexOf(self.tab), tr.editing_latex())