# Form implementation generated from reading ui file 'qt/aqt/forms/browserdisp.ui'
#
# Created by: PyQt6 UI code generator 6.11.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets
from aqt.utils import tr



class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(412, 241)
        self.verticalLayout = QtWidgets.QVBoxLayout(Dialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.label = QtWidgets.QLabel(parent=Dialog)
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label)
        self.qfmt = QtWidgets.QLineEdit(parent=Dialog)
        self.qfmt.setObjectName("qfmt")
        self.verticalLayout.addWidget(self.qfmt)
        self.label_2 = QtWidgets.QLabel(parent=Dialog)
        self.label_2.setObjectName("label_2")
        self.verticalLayout.addWidget(self.label_2)
        self.afmt = QtWidgets.QLineEdit(parent=Dialog)
        self.afmt.setObjectName("afmt")
        self.verticalLayout.addWidget(self.afmt)
        self.overrideFont = QtWidgets.QCheckBox(parent=Dialog)
        self.overrideFont.setObjectName("overrideFont")
        self.verticalLayout.addWidget(self.overrideFont)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.font = QtWidgets.QFontComboBox(parent=Dialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(5)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.font.sizePolicy().hasHeightForWidth())
        self.font.setSizePolicy(sizePolicy)
        self.font.setObjectName("font")
        self.horizontalLayout.addWidget(self.font)
        self.fontSize = QtWidgets.QSpinBox(parent=Dialog)
        self.fontSize.setMinimum(6)
        self.fontSize.setObjectName("fontSize")
        self.horizontalLayout.addWidget(self.fontSize)
        self.verticalLayout.addLayout(self.horizontalLayout)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.buttonBox = QtWidgets.QDialogButtonBox(parent=Dialog)
        self.buttonBox.setOrientation(QtCore.Qt.Orientation.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.StandardButton.Cancel|QtWidgets.QDialogButtonBox.StandardButton.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(Dialog)
        self.buttonBox.accepted.connect(Dialog.accept) # type: ignore  # type: ignore
        self.buttonBox.rejected.connect(Dialog.reject) # type: ignore  # type: ignore
        QtCore.QMetaObject.connectSlotsByName(Dialog)
        Dialog.setTabOrder(self.qfmt, self.afmt)
        Dialog.setTabOrder(self.afmt, self.font)
        Dialog.setTabOrder(self.font, self.fontSize)
        Dialog.setTabOrder(self.fontSize, self.buttonBox)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(tr.browsing_browser_appearance())
        self.label.setText(tr.browsing_override_front_template())
        self.label_2.setText(tr.browsing_override_back_template())
        self.overrideFont.setText(tr.browsing_override_font())