.toast-container.svelte-1bn4psc{position:fixed;bottom:3rem;z-index:100;width:100%;text-align:center;display:flex;justify-content:center}.toast.svelte-1bn4psc{display:flex;align-items:center;padding:1rem;background-color:#fff;border-radius:.5rem;box-shadow:0 .5rem 1rem #0000001a;width:60%;justify-content:space-between}.success.svelte-1bn4psc{background:#66bb6a;color:#fff}.error.svelte-1bn4psc{background:#ef5350;color:#fff}.toast-icon{background:unset!important;color:#fff!important;border:none!important}.top-tool-bar-container.svelte-fupcfw{display:flex;overflow-y:scroll;z-index:99;margin-left:106px;margin-top:2px}[dir=rtl] .top-tool-bar-container{margin-left:unset;margin-right:28px}.undo-redo-button.svelte-fupcfw{margin-right:2px;display:flex}.tool-button-container.svelte-fupcfw{margin-left:2px;margin-right:2px;display:flex}[dir=rtl] .left-border-radius,.right-border-radius{border-radius:0 5px 5px 0!important}[dir=rtl] .right-border-radius{border-radius:5px 0 0 5px!important}.border-radius{border-radius:5px!important}.top-tool-icon-button{border:unset;display:inline;width:32px;height:32px;margin:unset;padding:6px!important;font-size:16px!important}.top-tool-icon-button:active{background:var(--highlight-bg)!important}.dropdown-content.svelte-fupcfw{display:none;position:absolute;z-index:100;top:40px;margin-top:1px}.show.svelte-fupcfw{display:table}.svelte-fupcfw::-webkit-scrollbar{width:.1em!important;height:.1em!important}.tool-bar-container.svelte-fupcfw{position:fixed;top:42px;left:2px;height:100%;border-right:1px solid var(--border);overflow-y:auto;width:32px;z-index:99;background:var(--canvas-elevated);padding-bottom:100px}.fill-mask svg{fill:var(--fill-tool-colour)!important;stroke:#000;stroke-width:1px}[dir=rtl] .tool-bar-container{left:unset;right:2px}.tool-icon-button{border:unset;display:block;width:32px;height:32px;margin:unset;padding:6px!important}.active-tool{color:#fff!important;background:var(--button-primary-bg)!important}.icon-border-radius{border-radius:5px!important}.dropdown-tool-mode{height:38px!important;display:inline}.editor-main.svelte-1rjlpjk{position:absolute;top:42px;left:36px;bottom:2px;right:2px;border:1px solid var(--border);overflow:auto;outline:none!important}[dir=rtl] .editor-main.svelte-1rjlpjk{left:2px;right:36px}.editor-container.svelte-1rjlpjk{width:100%;height:100%;position:relative;direction:ltr;overflow:hidden}#image.svelte-1rjlpjk{position:absolute}.upper-canvas{border:.5px solid var(--border-strong);border-width:thin}.canvas-container{position:absolute}.note-tool-icon-button{border-radius:2px!important;padding:0 6px!important}.left-border-radius{border-radius:5px 0 0 5px!important}.right-border-radius{border-radius:0 5px 5px 0!important}.tag-editor{margin:10px 0}.text-area.svelte-6mwjli{height:120px;width:100%;display:none;background:var(--canvas-elevated);border:2px solid var(--border);border-radius:var(--border-radius);outline:none;resize:none;overflow:auto}.text-editor.svelte-6mwjli{height:80px;border:1px solid var(--border);border-radius:var(--border-radius);padding:5px;overflow:auto;outline:none;background:var(--canvas-elevated)}.note-toolbar.svelte-6mwjli{margin-left:106px;margin-top:2px;display:flex;overflow-x:auto;height:38px}.svelte-6mwjli::-webkit-scrollbar{width:.1em!important;height:.1em!important}.note-container.svelte-6mwjli{width:100%}.tab-buttons.svelte-18xd4xc{display:flex;position:absolute;top:2px;left:2px}.tab-buttons.svelte-18xd4xc .active:where(.svelte-18xd4xc){background:var(--button-primary-bg);color:#fff}.tab-item.svelte-18xd4xc{justify-content:center;align-items:center;display:flex;padding:0 6px;height:38px;max-width:60px;font-size:small;white-space:normal;word-break:break-all;-webkit-hyphens:auto;hyphens:auto}.image-occlusion{--gutter-inline: .5rem}.image-occlusion .row{--bs-gutter-x: 0;min-height:2.5em;align-items:center}
