.progress.svelte-h8igvb{position:absolute;top:50%;left:50%;transform:translate(-50%,-50%)}.spinner.svelte-h8igvb{display:block;position:relative;width:80px;height:80px;margin:0 auto}.spinner.svelte-h8igvb div:where(.svelte-h8igvb){display:block;position:absolute;width:64px;height:64px;margin:8px;border:8px solid #000;border-radius:50%;animation:svelte-h8igvb-spin 1.2s cubic-bezier(.5,0,.5,1) infinite;border-color:#000 transparent transparent transparent}.spinner.nightMode.svelte-h8igvb div:where(.svelte-h8igvb){border-top-color:#fff}.spinner.svelte-h8igvb div:where(.svelte-h8igvb):nth-child(1){animation-delay:-.45s}.spinner.svelte-h8igvb div:where(.svelte-h8igvb):nth-child(2){animation-delay:-.3s}.spinner.svelte-h8igvb div:where(.svelte-h8igvb):nth-child(3){animation-delay:-.15s}@keyframes svelte-h8igvb-spin{0%{transform:rotate(0)}to{transform:rotate(360deg)}}#label.svelte-h8igvb{text-align:center}.message.svelte-11g2c32{text-align:center;margin:50px 0 0}.import.svelte-crlmvj{margin:.2rem .75rem}.filename.svelte-crlmvj{word-break:break-word}.outer.svelte-1kvcg4{width:100%;overflow:auto;max-height:var(--container-height);margin:0 auto}.table.svelte-1kvcg4{border-collapse:collapse;white-space:nowrap}.table.svelte-1kvcg4 th,.table.svelte-1kvcg4 td{text-overflow:ellipsis;overflow:hidden;border:1px solid var(--border-subtle);padding:.25rem .5rem;max-width:15em}.table.svelte-1kvcg4 th{background:var(--border);text-align:center}.table.svelte-1kvcg4 thead{position:sticky;top:-1px;overflow-y:auto;overflow-x:hidden;z-index:1}.table.svelte-1kvcg4 tbody{overflow-y:scroll;overflow-x:hidden}.details-table{margin:0 auto;width:100%}.details-table .search-icon{border:none!important;background:transparent!important}.details-table tr.svelte-bz5xdb{height:40px;text-align:center}.details-table .index-cell.svelte-bz5xdb{width:3em}.details-table .status-cell{width:5em}.details-table .contents-cell{text-align:left}.details-table .search-cell{width:3em}li.svelte-1l3s192{list-style-type:none}.row{--bs-gutter-x: 0;margin-bottom:.5rem}.pre-import-page.svelte-45sjlv{margin:0 auto}
