.col.svelte-c0c30t{display:flex;flex-flow:row nowrap;align-items:var(--col-align, flex-start);justify-content:var(--col-justify, flex-start);padding:0 var(--gutter-inline, 0);flex:1 0 100%}.col-xs.svelte-c0c30t{flex-basis:calc(100% / var(--cols, 1) * var(--col-size, 1))}@media(min-width:576px){.col-sm.svelte-c0c30t{flex-basis:calc(100% / var(--cols, 1) * var(--col-size, 1))}}@media(min-width:768px){.col-md.svelte-c0c30t{flex-basis:calc(100% / var(--cols, 1) * var(--col-size, 1))}}@media(min-width:992px){.col-lg.svelte-c0c30t{flex-basis:calc(100% / var(--cols, 1) * var(--col-size, 1))}}@media(min-width:1200px){.col-xl.svelte-c0c30t{flex-basis:calc(100% / var(--cols, 1) * var(--col-size, 1))}}@media(min-width:1400px){.col-xxl.svelte-c0c30t{flex-basis:calc(100% / var(--cols, 1) * var(--col-size, 1))}}
