span.svelte-1jtvsz5{display:inline-block;position:relative;vertical-align:var(--icon-align, middle);min-width:calc((var(--buttons-size, 22px) - 2px) * var(--width-multiplier));height:calc(var(--buttons-size, 22px) - 2px)}span.svelte-1jtvsz5>svg,span.svelte-1jtvsz5>img{position:absolute;width:var(--icon-size);height:var(--icon-size);top:calc((100% - var(--icon-size)) / 2);bottom:calc((100% - var(--icon-size)) / 2);left:calc((100% - var(--icon-size)) / 2);right:calc((100% - var(--icon-size)) / 2);fill:currentColor;vertical-align:unset}span.flip-x.svelte-1jtvsz5>svg,span.flip-x.svelte-1jtvsz5>img{transform:scaleX(-1)}
