svg.svelte-188r9z9{transition:opacity var(--transition-slow)}.counts-outer.svelte-188r9z9{display:flex;justify-content:center;margin:0 4vw;flex-wrap:wrap;flex:1}.counts-outer.svelte-188r9z9 .svg-container:where(.svelte-188r9z9){width:225px}.counts-outer.svelte-188r9z9 .counts-table:where(.svelte-188r9z9){display:flex;flex-direction:column;justify-content:center}.counts-outer.svelte-188r9z9 .counts-table:where(.svelte-188r9z9) table:where(.svelte-188r9z9){border-spacing:1em 0;padding-left:4vw}.counts-outer.svelte-188r9z9 .counts-table:where(.svelte-188r9z9) table:where(.svelte-188r9z9) td:where(.svelte-188r9z9){white-space:nowrap;padding:0 min(4vw,40px)}.counts-outer.svelte-188r9z9 .counts-table:where(.svelte-188r9z9) table:where(.svelte-188r9z9) td.right:where(.svelte-188r9z9){text-align:right}.search-link.svelte-188r9z9{border:1px transparent solid;background:transparent;cursor:pointer;box-shadow:none;padding:0 2px;margin-bottom:0}.search-link.svelte-188r9z9:hover{color:var(--fg-link);text-decoration:underline}label.svelte-1jgpzb1{display:inline-flex;align-items:center}input[type=radio].svelte-1jgpzb1{margin-inline-end:.3em}.range-box.svelte-1jgpzb1{position:sticky;z-index:1;top:0;width:100vw;color:var(--fg);background:var(--canvas);padding:.5em;border-bottom:1px solid var(--border)}@media print{.range-box.svelte-1jgpzb1{position:absolute}}@keyframes svelte-1jgpzb1-spin{0%{-webkit-transform:rotate(0deg)}to{-webkit-transform:rotate(360deg)}}.spin.svelte-1jgpzb1{display:inline-block;position:absolute;font-size:2em;animation:svelte-1jgpzb1-spin;animation-duration:1s;animation-iteration-count:infinite;z-index:-1;opacity:0}.spin.loading.svelte-1jgpzb1{opacity:.5;z-index:1;transition:opacity var(--transition-slow)}.range-box-pad.svelte-1jgpzb1{height:1.5em}.graphs-container.svelte-107zgtd{display:grid;gap:1em;grid-template-columns:repeat(3,minmax(0,1fr));width:calc(100vw - 3em);margin-left:1em;margin-right:1em}@media only screen and (max-width:600px){.graphs-container.svelte-107zgtd{width:calc(100vw - 1rem);margin-left:.5rem;margin-right:.5rem}}@media only screen and (max-width:1400px){.graphs-container.svelte-107zgtd{grid-template-columns:1fr 1fr}}@media only screen and (max-width:1200px){.graphs-container.svelte-107zgtd{grid-template-columns:1fr}}@media only screen and (max-width:600px){.graphs-container.svelte-107zgtd{font-size:12px}}@media only print{.graphs-container.svelte-107zgtd{display:block;margin-top:3em}}.spacer.svelte-107zgtd{height:1.5em}.legend.svelte-5cj8zb{text-align:center}table.svelte-uurxng{border-collapse:collapse;margin-left:auto;margin-right:auto}tr.svelte-uurxng{border-top:1px solid var(--border);border-bottom:1px solid var(--border)}td.svelte-uurxng,th.svelte-uurxng{padding-left:.5em;padding-right:.5em}.row-header.svelte-uurxng{color:var(--fg)}.young.svelte-uurxng,.mature.svelte-uurxng,.total.svelte-uurxng,.count.svelte-uurxng{text-align:right}.young.svelte-uurxng{color:#64c476}.mature.svelte-uurxng{color:#31a354}.total.svelte-uurxng{color:var(--fg)}.count.svelte-uurxng{color:var(--fg-subtle)}table.svelte-h4f6yh{border-collapse:collapse;margin-left:auto;margin-right:auto}tr.svelte-h4f6yh{border-top:1px solid var(--border);border-bottom:1px solid var(--border)}td.svelte-h4f6yh,th.svelte-h4f6yh{padding-left:.5em;padding-right:.5em}.row-header.svelte-h4f6yh{color:var(--fg)}.pass.svelte-h4f6yh,.fail.svelte-h4f6yh,.retention.svelte-h4f6yh{text-align:right}.pass.svelte-h4f6yh{color:#3bc464}.fail.svelte-h4f6yh{color:#c43b3b}.retention.svelte-h4f6yh{color:var(--fg)}.table-container.svelte-1v1vsx5{margin-top:1rem;overflow-x:auto;display:flex;align-items:center}
