#image-occlusion-container{position:relative;margin:0 auto;max-height:calc(95vh - 40px)}#image-occlusion-container img{position:absolute;top:0;left:0;width:100%;height:100%;max-width:unset;max-height:unset}#image-occlusion-canvas{position:absolute;top:0;left:0;width:100%;height:100%}hr{background-color:#737373;margin:1em 0;border:none;height:1px}body{margin:20px;overflow-wrap:break-word;background-size:cover;background-repeat:no-repeat;background-position:top;background-attachment:fixed}body.nightMode{background-color:var(--canvas);color:var(--fg)}img{max-width:100%;max-height:95vh}li{text-align:start}pre{text-align:left}#_flag{position:fixed;top:0;font-size:30px;-webkit-text-stroke-width:1px;-webkit-text-stroke-color:#000}[dir=ltr] #_flag{right:10px}[dir=rtl] #_flag{left:10px}#_mark{position:fixed;top:0;font-size:30px;color:#ff0;-webkit-text-stroke-width:1px;-webkit-text-stroke-color:#000}[dir=ltr] #_mark{left:10px}[dir=rtl] #_mark{right:10px}#typeans{width:100%;box-sizing:border-box;line-height:1.75}code#typeans{white-space:pre-wrap;font-variant-ligatures:none}.typeGood{background:#afa;color:#000}.typeBad{color:#000;background:#faa}.typeMissed{color:#000;background:#ccc}button{margin:1em .5em}.replay-button{text-decoration:none;display:inline-flex;vertical-align:middle;margin:3px}.replay-button svg{width:40px;height:40px}.replay-button svg circle{fill:#fff;stroke:#414141}.replay-button svg path{fill:#414141}.nightMode .latex{filter:invert(100%)}.drawing{zoom:50%}.nightMode img.drawing{filter:invert(1) hue-rotate(180deg)}/*# sourceMappingURL=reviewer.css.map */
