import{f3 as i}from"./ceRbsEBD.mjs";function d(a,r){return`<a href="javascript:bridgeCommand('${a}')">${r}</a>`}function n(a,r){window.bridgeCommand(a,r)}i("anki/bridgecommand",{bridgeCommand:n});export{d as a,n as b};
