@charset "UTF-8";*{max-width:100%}p{margin-top:0;margin-bottom:1rem}p:empty:after{content:"\a";white-space:pre}[hidden]{display:none}:host(body)::-webkit-scrollbar,:host(body) *::-webkit-scrollbar{background-color:var(--canvas)}:host(body)::-webkit-scrollbar:horizontal,:host(body) *::-webkit-scrollbar:horizontal{height:12px}:host(body)::-webkit-scrollbar:vertical,:host(body) *::-webkit-scrollbar:vertical{width:12px}:host(body)::-webkit-scrollbar-thumb,:host(body) *::-webkit-scrollbar-thumb{background:var(--scrollbar-bg);border-radius:var(--border-radius)}:host(body)::-webkit-scrollbar-thumb:horizontal,:host(body) *::-webkit-scrollbar-thumb:horizontal{min-width:50px}:host(body)::-webkit-scrollbar-thumb:vertical,:host(body) *::-webkit-scrollbar-thumb:vertical{min-height:50px}:host(body)::-webkit-scrollbar-thumb:hover,:host(body) *::-webkit-scrollbar-thumb:hover{background:var(--scrollbar-bg-hover)}:host(body)::-webkit-scrollbar-thumb:active,:host(body) *::-webkit-scrollbar-thumb:active{background:var(--scrollbar-bg-active)}:host(body)::-webkit-scrollbar-corner,:host(body) *::-webkit-scrollbar-corner{background-color:var(--canvas)}:host(body)::-webkit-scrollbar-track,:host(body) *::-webkit-scrollbar-track{border-radius:5px;background-color:transparent}pre{white-space:pre-wrap}:host-context(.shrink-image) img:not(.mathjax):not([data-editor-shrink=false]){max-width:var(--editor-default-max-width);max-height:var(--editor-default-max-height);width:unset;height:unset}img:not(.mathjax)[data-editor-shrink=true]{max-width:var(--editor-shrink-max-width);max-height:var(--editor-shrink-max-height)}anki-editable.svelte-1fjuj6c{display:block;position:relative;overflow:auto;overflow-wrap:anywhere;word-break:break-word;min-height:1.5em}anki-editable.svelte-1fjuj6c:focus{outline:none}anki-mathjax{white-space:pre}img.svelte-1ouc3ps{vertical-align:var(--vertical-center)}.block.svelte-1ouc3ps{display:block;margin:1rem auto;transform:scale(1.1)}.empty.svelte-1ouc3ps{vertical-align:text-bottom;width:var(--font-size);height:var(--font-size)}
