import{c as L,e as _,p as N,U as A,l as G,d as g,b as E,o as b,h as R,S as V,f as q,g as K}from"./8AYMN14U.mjs";import{i as P,r as W,S as m,a as U}from"./C1QGnB9O.mjs";var $=Object.create,Y=(function(){function e(){}return function(t){if(!P(t))return{};if($)return $(t);e.prototype=t;var n=new e;return e.prototype=void 0,n}})(),w=(function(){try{var e=L(Object,"defineProperty");return e({},"",{}),e}catch{}})();function H(e,t){for(var n=-1,a=e==null?0:e.length;++n<a&&t(e[n],n,e)!==!1;);return e}function J(e,t,n){t=="__proto__"&&w?w(e,t,{configurable:!0,enumerable:!0,value:n,writable:!0}):e[t]=n}var Q=Object.prototype,X=Q.hasOwnProperty;function Z(e,t,n){var a=e[t];(!(X.call(e,t)&&_(a,n))||n===void 0&&!(t in e))&&J(e,t,n)}var z=N(Object.getPrototypeOf,Object),I=typeof exports=="object"&&exports&&!exports.nodeType&&exports,O=I&&typeof module=="object"&&module&&!module.nodeType&&module,k=O&&O.exports===I,v=k?W.Buffer:void 0;v&&v.allocUnsafe;function ee(e,t){return e.slice()}var te=Object.prototype,re=te.hasOwnProperty;function ne(e){var t=e.length,n=new e.constructor(t);return t&&typeof e[0]=="string"&&re.call(e,"index")&&(n.index=e.index,n.input=e.input),n}function p(e){var t=new e.constructor(e.byteLength);return new A(t).set(new A(e)),t}function oe(e,t){var n=p(e.buffer);return new e.constructor(n,e.byteOffset,e.byteLength)}var ae=/\w*$/;function ce(e){var t=new e.constructor(e.source,ae.exec(e));return t.lastIndex=e.lastIndex,t}var h=m?m.prototype:void 0,S=h?h.valueOf:void 0;function ie(e){return S?Object(S.call(e)):{}}function se(e,t){var n=p(e.buffer);return new e.constructor(n,e.byteOffset,e.length)}var fe="[object Boolean]",ue="[object Date]",be="[object Map]",ge="[object Number]",pe="[object RegExp]",le="[object Set]",Te="[object String]",je="[object Symbol]",ye="[object ArrayBuffer]",de="[object DataView]",Ae="[object Float32Array]",me="[object Float64Array]",$e="[object Int8Array]",we="[object Int16Array]",Oe="[object Int32Array]",ve="[object Uint8Array]",he="[object Uint8ClampedArray]",Se="[object Uint16Array]",xe="[object Uint32Array]";function Ce(e,t,n){var a=e.constructor;switch(t){case ye:return p(e);case fe:case ue:return new a(+e);case de:return oe(e);case Ae:case me:case $e:case we:case Oe:case ve:case he:case Se:case xe:return se(e);case be:return new a;case ge:case Te:return new a(e);case pe:return ce(e);case le:return new a;case je:return ie(e)}}function Ee(e){return typeof e.constructor=="function"&&!G(e)?Y(z(e)):{}}var Pe="[object Map]";function Ue(e){return U(e)&&g(e)==Pe}var x=b&&b.isMap,Ie=x?E(x):Ue,Be="[object Set]";function De(e){return U(e)&&g(e)==Be}var C=b&&b.isSet,Me=C?E(C):De,B="[object Arguments]",Fe="[object Array]",Le="[object Boolean]",_e="[object Date]",Ne="[object Error]",D="[object Function]",Ge="[object GeneratorFunction]",Re="[object Map]",Ve="[object Number]",M="[object Object]",qe="[object RegExp]",Ke="[object Set]",We="[object String]",Ye="[object Symbol]",He="[object WeakMap]",Je="[object ArrayBuffer]",Qe="[object DataView]",Xe="[object Float32Array]",Ze="[object Float64Array]",ze="[object Int8Array]",ke="[object Int16Array]",et="[object Int32Array]",tt="[object Uint8Array]",rt="[object Uint8ClampedArray]",nt="[object Uint16Array]",ot="[object Uint32Array]",r={};r[B]=r[Fe]=r[Je]=r[Qe]=r[Le]=r[_e]=r[Xe]=r[Ze]=r[ze]=r[ke]=r[et]=r[Re]=r[Ve]=r[M]=r[qe]=r[Ke]=r[We]=r[Ye]=r[tt]=r[rt]=r[nt]=r[ot]=!0;r[Ne]=r[D]=r[He]=!1;function u(e,t,n,a,l,i){var o;if(o!==void 0)return o;if(!P(e))return e;var T=q(e);if(T)o=ne(e);else{var f=g(e),j=f==D||f==Ge;if(R(e))return ee(e);if(f==M||f==B||j&&!l)o=j?{}:Ee(e);else{if(!r[f])return l?e:{};o=Ce(e,f)}}i||(i=new V);var y=i.get(e);if(y)return y;i.set(e,o),Me(e)?e.forEach(function(c){o.add(u(c,t,n,c,e,i))}):Ie(e)&&e.forEach(function(c,s){o.set(s,u(c,t,n,s,e,i))});var F=K,d=T?void 0:F(e);return H(d||e,function(c,s){d&&(s=c,c=e[s]),Z(o,s,u(c,t,n,s,e,i))}),o}var at=1,ct=4;function ft(e){return u(e,at|ct)}export{ft as c};
