@charset "UTF-8";.card-info-placeholder.svelte-111eaxc{margin:0;position:absolute;top:50%;left:50%;transform:translate(-50%,-50%);color:gray}.stats-table.svelte-8ebbmy{width:100%;border-spacing:1em 0;border-collapse:collapse}.align-start.svelte-8ebbmy{text-align:start}.revlog-table.svelte-ljyvi0{width:100%;max-width:50em;display:flex;flex-direction:row;justify-content:space-between;align-items:center;white-space:nowrap}.column.svelte-ljyvi0{display:flex;flex-direction:column;justify-content:center;align-items:center}.column.svelte-ljyvi0:not(:last-child){margin-right:.5em}.column-head.svelte-ljyvi0{font-weight:700}.column-content.svelte-ljyvi0{display:inline-flex;flex-direction:column;justify-content:center;align-items:center}.column-content.right.svelte-ljyvi0{align-items:flex-end}.column-content.svelte-ljyvi0>div:where(.svelte-ljyvi0):empty:after{content:" "}.revlog-learn.svelte-ljyvi0{color:var(--state-new)}.revlog-review.svelte-ljyvi0{color:var(--state-review)}.revlog-relearn.svelte-ljyvi0,.revlog-ease1.svelte-ljyvi0{color:var(--state-learn)}@media only screen and (max-device-width:480px)and (orientation:portrait){.hidden-xs.svelte-ljyvi0{display:none}}.forgetting-curve.svelte-6ru10f{width:100%;max-width:50em;margin-bottom:10em}.time-range-selector.svelte-6ru10f{display:flex;justify-content:space-around;margin-bottom:1em;width:100%;max-width:50em}.time-range-selector.svelte-6ru10f label:where(.svelte-6ru10f){display:flex;align-items:center}.time-range-selector.svelte-6ru10f input:where(.svelte-6ru10f){margin-right:.5em}
