# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: anki/ankidroid.proto
# Protobuf Python Version: 7.34.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    7,
    34,
    1,
    '',
    'anki/ankidroid.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from anki import generic_pb2 as anki_dot_generic__pb2
from anki import scheduler_pb2 as anki_dot_scheduler__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x14\x61nki/ankidroid.proto\x12\x0e\x61nki.ankidroid\x1a\x12\x61nki/generic.proto\x1a\x14\x61nki/scheduler.proto\"F\n*DebugActiveDatabaseSequenceNumbersResponse\x12\x18\n\x10sequence_numbers\x18\x01 \x03(\x05\"\xab\x01\n\x1dSchedTimingTodayLegacyRequest\x12\x14\n\x0c\x63reated_secs\x18\x01 \x01(\x03\x12\x1e\n\x11\x63reated_mins_west\x18\x02 \x01(\x11H\x00\x88\x01\x01\x12\x10\n\x08now_secs\x18\x03 \x01(\x03\x12\x15\n\rnow_mins_west\x18\x04 \x01(\x11\x12\x15\n\rrollover_hour\x18\x05 \x01(\x11\x42\x14\n\x12_created_mins_west\"j\n\x08SqlValue\x12\x15\n\x0bstringValue\x18\x01 \x01(\tH\x00\x12\x13\n\tlongValue\x18\x02 \x01(\x03H\x00\x12\x15\n\x0b\x64oubleValue\x18\x03 \x01(\x01H\x00\x12\x13\n\tblobValue\x18\x04 \x01(\x0cH\x00\x42\x06\n\x04\x44\x61ta\"/\n\x03Row\x12(\n\x06\x66ields\x18\x01 \x03(\x0b\x32\x18.anki.ankidroid.SqlValue\"-\n\x08\x44\x62Result\x12!\n\x04rows\x18\x01 \x03(\x0b\x32\x13.anki.ankidroid.Row\"t\n\nDbResponse\x12(\n\x06result\x18\x01 \x01(\x0b\x32\x18.anki.ankidroid.DbResult\x12\x16\n\x0esequenceNumber\x18\x02 \x01(\x05\x12\x10\n\x08rowCount\x18\x03 \x01(\x05\x12\x12\n\nstartIndex\x18\x04 \x01(\x03\";\n\x18GetNextResultPageRequest\x12\x10\n\x08sequence\x18\x01 \x01(\x05\x12\r\n\x05index\x18\x02 \x01(\x03\"3\n GetActiveSequenceNumbersResponse\x12\x0f\n\x07numbers\x18\x01 \x03(\x05\x32\x89\x05\n\x10\x41nkidroidService\x12\x36\n\x0cRunDbCommand\x12\x12.anki.generic.Json\x1a\x12.anki.generic.Json\x12\x43\n\x11RunDbCommandProto\x12\x12.anki.generic.Json\x1a\x1a.anki.ankidroid.DbResponse\x12\x36\n\x0bInsertForId\x12\x12.anki.generic.Json\x1a\x13.anki.generic.Int64\x12\x42\n\x17RunDbCommandForRowCount\x12\x12.anki.generic.Json\x1a\x13.anki.generic.Int64\x12;\n\x0f\x46lushAllQueries\x12\x13.anki.generic.Empty\x1a\x13.anki.generic.Empty\x12\x36\n\nFlushQuery\x12\x13.anki.generic.Int32\x1a\x13.anki.generic.Empty\x12Y\n\x11GetNextResultPage\x12(.anki.ankidroid.GetNextResultPageRequest\x1a\x1a.anki.ankidroid.DbResponse\x12I\n\x17GetColumnNamesFromQuery\x12\x14.anki.generic.String\x1a\x18.anki.generic.StringList\x12\x61\n\x18GetActiveSequenceNumbers\x12\x13.anki.generic.Empty\x1a\x30.anki.ankidroid.GetActiveSequenceNumbersResponse2\xc9\x02\n\x17\x42\x61\x63kendAnkidroidService\x12q\n\x16SchedTimingTodayLegacy\x12-.anki.ankidroid.SchedTimingTodayLegacyRequest\x1a(.anki.scheduler.SchedTimingTodayResponse\x12\x42\n\x16LocalMinutesWestLegacy\x12\x13.anki.generic.Int64\x1a\x13.anki.generic.Int32\x12\x37\n\x0bSetPageSize\x12\x13.anki.generic.Int64\x1a\x13.anki.generic.Empty\x12>\n\x11\x44\x65\x62ugProduceError\x12\x14.anki.generic.String\x1a\x13.anki.generic.EmptyB\x02P\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'anki.ankidroid_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'P\001'
  _globals['_DEBUGACTIVEDATABASESEQUENCENUMBERSRESPONSE']._serialized_start=82
  _globals['_DEBUGACTIVEDATABASESEQUENCENUMBERSRESPONSE']._serialized_end=152
  _globals['_SCHEDTIMINGTODAYLEGACYREQUEST']._serialized_start=155
  _globals['_SCHEDTIMINGTODAYLEGACYREQUEST']._serialized_end=326
  _globals['_SQLVALUE']._serialized_start=328
  _globals['_SQLVALUE']._serialized_end=434
  _globals['_ROW']._serialized_start=436
  _globals['_ROW']._serialized_end=483
  _globals['_DBRESULT']._serialized_start=485
  _globals['_DBRESULT']._serialized_end=530
  _globals['_DBRESPONSE']._serialized_start=532
  _globals['_DBRESPONSE']._serialized_end=648
  _globals['_GETNEXTRESULTPAGEREQUEST']._serialized_start=650
  _globals['_GETNEXTRESULTPAGEREQUEST']._serialized_end=709
  _globals['_GETACTIVESEQUENCENUMBERSRESPONSE']._serialized_start=711
  _globals['_GETACTIVESEQUENCENUMBERSRESPONSE']._serialized_end=762
  _globals['_ANKIDROIDSERVICE']._serialized_start=765
  _globals['_ANKIDROIDSERVICE']._serialized_end=1414
  _globals['_BACKENDANKIDROIDSERVICE']._serialized_start=1417
  _globals['_BACKENDANKIDROIDSERVICE']._serialized_end=1746
# @@protoc_insertion_point(module_scope)
