@charset "UTF-8";.icon-button.svelte-9egv84{-webkit-appearance:none;cursor:pointer;border:1px solid var(--border-subtle);border-bottom-color:var(--shadow);background:var(--button-bg);color:var(--fg);border-top-left-radius:var(--border-left-radius);border-bottom-left-radius:var(--border-left-radius);border-top-right-radius:var(--border-right-radius);border-bottom-right-radius:var(--border-right-radius);padding:0 var(--padding-inline, 0);font-size:var(--font-size);height:var(--buttons-size);min-width:calc(var(--buttons-size) * .75)}.icon-button.svelte-9egv84:hover{background:linear-gradient(180deg,var(--button-gradient-start) 0%,var(--button-gradient-end) 100%);border:1px solid var(--shadow)}.icon-button.svelte-9egv84:active{box-shadow:inset 0 calc(var(--buttons-size, 10px) / 15) calc(var(--buttons-size, 10px) / 5) #00000059;border-color:var(--border-subtle)}.icon-button.active.svelte-9egv84{box-shadow:inset 0 calc(var(--buttons-size, 10px) / 15) calc(var(--buttons-size, 10px) / 5) #00000059;background:var(--button-primary-bg);color:#fff;border-color:var(--border)}.icon-button[disabled].svelte-9egv84,.icon-button[disabled].svelte-9egv84:hover{cursor:not-allowed;color:var(--fg-disabled);box-shadow:none!important;background-color:var(--button-gradient-end);border-bottom-color:var(--border-subtle)}.icon-button.primary.svelte-9egv84{-webkit-appearance:none;cursor:pointer;border:none;background:var(--button-primary-bg);color:#fff}.icon-button.primary.svelte-9egv84:hover{background:linear-gradient(180deg,var(--button-primary-gradient-start) 0%,var(--button-primary-gradient-end) 100%)}.icon-button.primary.svelte-9egv84:active{box-shadow:inset 0 calc(var(--buttons-size, 10px) / 15) calc(var(--buttons-size, 10px) / 5) #00000059;border-color:var(--border-subtle)}.icon-button.primary[disabled].svelte-9egv84,.icon-button.primary[disabled].svelte-9egv84:hover{cursor:not-allowed;color:var(--fg-disabled);box-shadow:none!important;background-color:var(--button-gradient-end);border-bottom-color:var(--border-subtle)}
