import"./E-vkuI0X.mjs";import{i as T}from"./8UuMqzDP.mjs";import{bg as B,b0 as b,bu as r,ad as c,aw as a,b1 as D,a3 as W,bo as k,bP as x,W as C,bd as E,bI as F,bC as I,b8 as m,au as M}from"./W1VwgJDT.mjs";import{g as N}from"./CCT7NDl_.mjs";import{a7 as P}from"./FVFIwcTn.mjs";import{fD as S,fE as j,fF as q}from"./ceRbsEBD.mjs";import{e as A}from"./D78duh1q.mjs";import{p as i}from"./D-j9Rl0W.mjs";import{p as G}from"./CD0nhXdt.mjs";var H=M('<div role="toolbar"><!></div>');function Y(_,e){B(e,!1);const g=()=>F(G,"$pageTheme",v),[v,h]=I(),l=m(),u=m();let w=i(e,"id",8,void 0),y=i(e,"class",8,""),n=i(e,"size",8,void 0),f=i(e,"wrap",8,void 0),s=m();b(()=>c(n()),()=>{r(l,n()?`--buttons-size: ${n()}rem; `:"")}),b(()=>c(f()),()=>{f()===void 0?r(s,""):r(s,f()?"--buttons-wrap: wrap; ":"--buttons-wrap: nowrap; ")}),b(()=>(a(l),a(s)),()=>{r(u,a(l)+a(s))}),D(),T();var t=H();let d,p;var z=W(t);P(z,e,"default",{}),k(t),x(()=>{var o;S(t,"id",w()),d=j(t,1,`button-toolbar btn-toolbar ${(o=y())!=null?o:""}`,"svelte-1v9a56g",d,{nightMode:g().isDark}),p=q(t,a(u),p,{"--icon-align":"baseline"})}),N("focusout",t,function(o){A.call(this,e,o)}),C(_,t),E(),h()}export{Y as B};
