# Form implementation generated from reading ui file 'qt/aqt/forms/addons.ui'
#
# Created by: PyQt6 UI code generator 6.11.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets
from aqt.utils import tr



class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.setWindowModality(QtCore.Qt.WindowModality.ApplicationModal)
        Dialog.resize(800, 800)
        Dialog.setWindowTitle("ADD-ONS")
        Dialog.setModal(True)
        self.horizontalLayout = QtWidgets.QHBoxLayout(Dialog)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.label = QtWidgets.QLabel(parent=Dialog)
        self.label.setObjectName("label")
        self.verticalLayout_2.addWidget(self.label)
        self.addonList = QtWidgets.QListWidget(parent=Dialog)
        self.addonList.setSelectionMode(QtWidgets.QAbstractItemView.SelectionMode.ExtendedSelection)
        self.addonList.setObjectName("addonList")
        self.verticalLayout_2.addWidget(self.addonList)
        self.horizontalLayout.addLayout(self.verticalLayout_2)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.getAddons = QtWidgets.QPushButton(parent=Dialog)
        self.getAddons.setObjectName("getAddons")
        self.verticalLayout.addWidget(self.getAddons)
        self.installFromFile = QtWidgets.QPushButton(parent=Dialog)
        self.installFromFile.setObjectName("installFromFile")
        self.verticalLayout.addWidget(self.installFromFile)
        self.checkForUpdates = QtWidgets.QPushButton(parent=Dialog)
        self.checkForUpdates.setObjectName("checkForUpdates")
        self.verticalLayout.addWidget(self.checkForUpdates)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.viewPage = QtWidgets.QPushButton(parent=Dialog)
        self.viewPage.setObjectName("viewPage")
        self.verticalLayout.addWidget(self.viewPage)
        self.config = QtWidgets.QPushButton(parent=Dialog)
        self.config.setObjectName("config")
        self.verticalLayout.addWidget(self.config)
        self.viewFiles = QtWidgets.QPushButton(parent=Dialog)
        self.viewFiles.setObjectName("viewFiles")
        self.verticalLayout.addWidget(self.viewFiles)
        self.toggleEnabled = QtWidgets.QPushButton(parent=Dialog)
        self.toggleEnabled.setObjectName("toggleEnabled")
        self.verticalLayout.addWidget(self.toggleEnabled)
        self.delete_2 = QtWidgets.QPushButton(parent=Dialog)
        self.delete_2.setObjectName("delete_2")
        self.verticalLayout.addWidget(self.delete_2)
        self.horizontalLayout.addLayout(self.verticalLayout)

        self.retranslateUi(Dialog)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        self.label.setText(tr.addons_changes_will_take_effect_when_anki())
        self.getAddons.setText(tr.addons_get_addons())
        self.installFromFile.setText(tr.addons_install_from_file())
        self.checkForUpdates.setText(tr.addons_check_for_updates())
        self.viewPage.setText(tr.addons_view_addon_page())
        self.config.setText(tr.addons_config())
        self.viewFiles.setText(tr.addons_view_files())
        self.toggleEnabled.setText(tr.addons_toggle_enabled())
        self.delete_2.setText(tr.actions_delete())